#ifdef WIN32
#  include <windows.h>
#endif

#include "Jellyfish.h"

#define GLH_EXT_SINGLE_FILE
#define GLH_NVEB_USING_NVPARSE

#include <glh_nveb.h>
#include <glh_extensions.h>
#include <glh_obs.h>
#include <glh_glut.h>
#include <glh_glut_callfunc.h>
#include <nvparse.h>


NVEB_EFFECT_NAME("Procedurally Animated Jellyfish");
NVEB_EFFECT_VERSION("1.0");
NVEB_EFFECT_LOCATION("3rd Party\\Deformation");
NVEB_EFFECT_ABOUT( 0, NULL, "Jellyfish", "NVEffectsExplained.htm#jellyfish");
NVEB_EFFECT_ABOUT( 1, NULL, "3d Pipeline Corporation", "http://www.3dpipeline.com");
NVEB_EFFECT_ABOUT( 2, NULL, "Programmed by Wade Lutgen", "mailto:wade@3dpipeline.com");
NVEB_EFFECT_ABOUT( 3, "Date", "May-June 2001", NULL);


using namespace glh;

Jellyfish *jellyfish;
unsigned int winWidth = 512;
unsigned int winHeight = 512;
float fovy = 35.0f;
float dist = 75.0f;

struct Mouse
{
  float phi;
  float theta;
  int x, y;
  bool leftButton;
};

Mouse mouseState;


float queryElapsedTime (  )
{
  return glutGet(GLenum(GLUT_ELAPSED_TIME)) * .001f;
}

void mouseButton( int button, int state, int x, int y )
{
  if ( button == GLUT_LEFT_BUTTON)
  {
    mouseState.x = 0;
    mouseState.y = 0;
    switch (state)
    {
    case GLUT_DOWN:
      mouseState.x = x;
      mouseState.y = y;
      mouseState.leftButton = true;
      break;
    case GLUT_UP:
      mouseState.leftButton = false;
      break;
    }
  }
}

void motion ( int x, int y )
{
  if ( mouseState.leftButton )
  {
    mouseState.theta += (x - mouseState.x) * 0.5f;
    mouseState.phi += (y - mouseState.y) * 0.5;
    mouseState.x = x;
    mouseState.y = y;
  }
}

void reshape(int x, int y)
{
  float aspectRatio = GLfloat(x) / GLfloat(y);

  winWidth = x;
  winHeight = y;

  glMatrixMode(GL_PROJECTION);					
  glLoadIdentity();	
  gluPerspective( fovy, aspectRatio, 0.1f, 10000.0f );
  glMatrixMode(GL_MODELVIEW);

  glViewport(0, 0, winWidth, winHeight);
}

void initializeVertexProgramExtensions( )
{
	glLoadProgramNV = (PFNGLLOADPROGRAMNVPROC) 
		wglGetProcAddress("glLoadProgramNV");
	
	if ( glLoadProgramNV == 0 )
	{
		cerr << "Error: could not load glLoadProgramNV" << endl;
		exit(0);
	}
	
	glProgramParameter4fNV = (PFNGLPROGRAMPARAMETER4FNVPROC)
		wglGetProcAddress("glProgramParameter4fNV");
	
	if ( glProgramParameter4fNV == 0 )
	{
		cerr << "Error: could not load glProgramParameter4fNV" << endl;
		exit(0);
	}
	
	glVertexAttrib4fvNV = (PFNGLVERTEXATTRIB4FVNVPROC)
		wglGetProcAddress("glVertexAttrib4fvNV");
	
	if ( glVertexAttrib4fvNV == 0 )
	{
		cerr << "Error: could not load glVertexAttrib4fvNV" << endl;
		exit(0);
	}
	
	glVertexAttrib3fvNV = (PFNGLVERTEXATTRIB3FVNVPROC)
		wglGetProcAddress("glVertexAttrib3fvNV");
	
	if ( glVertexAttrib3fvNV == 0 )
	{
		cerr << "Error: could not load glVertexAttrib3fvNV" << endl;
		exit(0);
	}
	
	glVertexAttrib2fvNV = (PFNGLVERTEXATTRIB2FVNVPROC)
		wglGetProcAddress("glVertexAttrib2fvNV");
	
	if ( glVertexAttrib2fvNV == 0 )
	{
		cerr << "Error: could not load glVertexAttrib2fvNV" << endl;
		exit(0);
	}
	
	
	
	glBindProgramNV = (PFNGLBINDPROGRAMNVPROC)
		wglGetProcAddress("glBindProgramNV");
	
	if ( glBindProgramNV == 0)
	{
		cerr << "Error: could not load glBindProgramNV" << endl;
		exit(0);
	}
	
	
	glTrackMatrixNV = (PFNGLTRACKMATRIXNVPROC)
		wglGetProcAddress("glTrackMatrixNV");
	
	if ( glTrackMatrixNV == 0 )
	{
		cerr << "Error: could not load glTrackMatrixNV" << endl;
		exit(0);
	}
	
	
	glExecuteProgramNV = (PFNGLEXECUTEPROGRAMNVPROC)
		wglGetProcAddress("glExecuteProgramNV");
	
	if ( glExecuteProgramNV == 0 )
	{
		cerr << "Error: could not load glExecuteProgramNV" << endl;
        exit(0);
	}
	
	glGetProgramParameterfvNV = (PFNGLGETPROGRAMPARAMETERFVNVPROC)
		wglGetProcAddress("glGetProgramParameterfvNV");
	
	if ( glGetProgramParameterfvNV == 0 )
	{
		cerr << "Error: could not load glGetProgramParameterfvNV" << endl;
		exit(0);
	}

}

void loadVertexPrograms( )
{
	glBindProgramNV(GL_VERTEX_PROGRAM_NV, 1);

  // vertex program to light, transform, animate, and texturemap jellyfish
  nvparse
  (
    "!!VP1.0"

    "\n\n# R0 = (y coordinate of vertex * k) + (w * t)\n"
    "MAD R0.x, v[OPOS].y, c[16].x, c[16].z;\n"

    "\n# R1 = cos(R0), R0 = any floating point number\n"
    "MUL R0.x, c[11].w, R0.x;\n" 
    "EXP R0.y, R0.x;\n" 
    "SLT R2.x, R0.y, c[11];\n" 
    "SGE R2.yz, R0.y, c[11];\n" 
    "DP3 R2.y, R2, c[14].zwzw;\n"   
    "ADD R5.xyz, -R0.y, c[10];\n" 
    "MUL R5, R5, R5;\n"
    "MAD R0, c[12].xyxy, R5, c[12].zwzw;\n" 
    "MAD R0, R0, R5, c[13].xyxy;\n"      
    "MAD R0, R0, R5, c[13].zwzw;\n"      
    "MAD R0, R0, R5, c[14].xyxy;\n"      
    "MAD R0, R0, R5, c[14].zwzw;\n"      
    "DP3 R1.x, R0, -R2;\n" 

    "\n# R2 = R1 * c[10] + 1.0\n"
    "MAD R2.x, R1.x, c[10].w, c[10].z;\n"

    "\n# R3 = perturbed vertex\n"
    "# R4 = perturbed normal\n"
    "MOV R2.yw, c[16];\n"
    "MUL R3, v[OPOS], R2.xyxw;\n"
    "MUL R4, v[NRML], R2.yxyw;\n"

    "\n# Transform vertices into clip space via modelview-projection matrix\n"
    "DP4 o[HPOS].x, R3, c[0];\n"
    "DP4 o[HPOS].y, R3, c[1];\n"
    "DP4 o[HPOS].z, R3, c[2];\n"
    "DP4 o[HPOS].w, R3, c[3];\n"

    "\n# Transform normals via inverse transpose modelview matrix & normalize\n"
    "# R5 = transformed normal\n"
    "DP3 R5.x, R4, c[4];\n"
    "DP3 R5.y, R4, c[5];\n"
    "DP3 R5.z, R4, c[6];\n"
    "DP3 R5.w, R5, R5;\n"
    "RSQ R5.w, R5.w;\n"
    "MUL R5, R5, R5.w;\n"

    "\n# Get unit length eye vector to vertex....this is how we texture map\n"
    "# R6 = vector\n"
    "ADD R6, c[8], v[OPOS];\n"
    "DP3 R6.w, R6, R6;\n"
    "RSQ R6.w, R6.w;\n"
    "MUL R6, R6, R6.w;\n"

    "\n# Multiply by 0.5 for texture wrapping\n"
    "MUL R6, R6, c[10].y;\n"
    
    "\n# Texture coord is dot product of normal and vector from eye to vertex\n"
    "DP3 o[TEX0].x, R5, R6;\n"

    "\n# Pass color through\n"
    "MOV o[COL0], v[COL0];\n\n"

    "END"
  );
	
	// Track the Projection matrix in c[0] to c[3].
	glTrackMatrixNV(GL_VERTEX_PROGRAM_NV, 0, GL_MODELVIEW_PROJECTION_NV, GL_IDENTITY_NV);

	// Track the inverse of the Modelview matrix in c[4] to c[7]
	glTrackMatrixNV(GL_VERTEX_PROGRAM_NV, 4, GL_MODELVIEW, GL_INVERSE_TRANSPOSE_NV);

  // Load the view position into c[8]
	glProgramParameter4fNV(GL_VERTEX_PROGRAM_NV, 8, 0.0f, 0.0f, 100.0f, 1.0f);	

  // Load cosine series constants into c[9]
 	glProgramParameter4fNV(GL_VERTEX_PROGRAM_NV, 9, 1.0f, -1.0f/2.0f, 1.0f/24.0f, -1.0f/720.0f);	

  // Constants for cosine
  glProgramParameter4fNV(GL_VERTEX_PROGRAM_NV, 10, 0.0f, 0.5f, 1.0f, 0.2f);
  glProgramParameter4fNV(GL_VERTEX_PROGRAM_NV, 11, 0.25f, -9.0f, 0.75f, 1.0f / (2.0f * M_PI) );
  glProgramParameter4fNV(GL_VERTEX_PROGRAM_NV, 12, 24.9808039603f, -24.9808039603f, -60.1458091736f, 60.1458091736f );
  glProgramParameter4fNV(GL_VERTEX_PROGRAM_NV, 13, 85.4537887573f, -85.4537887573f, -64.9393539429f, 64.9393539429f );
  glProgramParameter4fNV(GL_VERTEX_PROGRAM_NV, 14, 19.7392082214f, -19.7392082214f, - 1.0f,	1.0f );

  // Light position
  glProgramParameter4fNV(GL_VERTEX_PROGRAM_NV, 15, 0.0f, 0.7f, 0.7f, 1.0f );

  glProgramParameter4fNV(GL_VERTEX_PROGRAM_NV, 17, 0.3f, 0.3f, 0.3f, 1.0f );
}


void init()
{
  reshape( winWidth, winHeight );					

  glEnable(GL_BLEND);
  glShadeModel(GL_SMOOTH);
  glEnable(GL_DEPTH_TEST);
  glDisable(GL_CULL_FACE);
  glClearColor(0.0f, 0.1f, 0.2f, 1.0f);
  glBlendFunc(GL_ONE, GL_ONE_MINUS_SRC_ALPHA);

  initializeVertexProgramExtensions( );
  loadVertexPrograms( );

  jellyfish = new Jellyfish( );
}

void keyboard(unsigned char key, int x, int y)
{
  static bool blend = true;
  static bool wireframe = false;

  switch(key)
  {
  //case 'q':  case 27:
  //  exit(1);
  //  break;
  case 'w':
    wireframe = !wireframe;
    if ( wireframe )
      glPolygonMode(GL_FRONT_AND_BACK, GL_LINE);
    else
      glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);
    break;
  case 'z':
    if ( dist > 30 )
      dist--;
    break;
  case 'x':
    if ( dist < 75 )
      dist++;
    break;
  }
}

void menu(int entry) 
{ 
  keyboard((unsigned char)entry, 0, 0); 
}

void idle(void)
{
  glutPostRedisplay();
}

void display()
{
  glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
  glLoadIdentity( );
  gluLookAt(0.0f, 0.0f, dist, 0.0f, 0.0f, 0.0f, 0.0f, 1.0f, 0.0f);
  glRotatef(mouseState.phi, 1.0f, 0.0f, 0.0f);
  glRotatef(mouseState.theta, 0.0f, 1.0f, 0.0f);

  glTranslatef(0.0f, 10.0f, 0.0f);

  jellyfish->render( );

  glutSwapBuffers();
}

int main(int argc, char **argv)
{
  glutInit(&argc, argv);
  glutInitWindowPosition(0, 0);
  glutInitWindowSize(winWidth, winHeight);
  glutInitDisplayMode(GLUT_RGBA | GLUT_DOUBLE | GLUT_DEPTH);
  glutCreateWindow("Jellyfish");
  
  glutDisplayFunc(display);
  glutKeyboardFunc(keyboard);
  glutIdleFunc(idle);
  glutReshapeFunc(reshape);
  glutMotionFunc(motion);
  glutMouseFunc(mouseButton);

	init();

	int tweakables = glutCreateMenu(menu);
  glutAddMenuEntry("Zoom in [z]", 'z');
  glutAddMenuEntry("Zoom out [x]", 'x');
	glutAddMenuEntry("Toggle wireframe [w]", 'w');
	glutAttachMenu(GLUT_RIGHT_BUTTON);

	glutMainLoop();

	return 0;
}