#pragma warning(disable:4244)   // No warnings on precision truncation
#pragma warning(disable:4305)   // No warnings on precision truncation
#pragma warning(disable:4786)   // stupid symbol size limitation


#ifdef WIN32
#  include <windows.h>
#endif

#define GLH_EXT_SINGLE_FILE
#define GLH_NVEB_USING_NVPARSE
#include <glh_nveb.h>

NVEB_EFFECT_NAME("Volume Lighting");
NVEB_EFFECT_VERSION("1.0");
NVEB_EFFECT_LOCATION("3rd Party\\Research");
NVEB_EFFECT_ABOUT( 0, NULL, "Volume Lighting", "NVEffectsExplained.htm#volumelighting");
NVEB_EFFECT_ABOUT( 1, NULL, "WSI/GRIS", "http://www.gris.uni-tuebingen.de");
NVEB_EFFECT_ABOUT( 2, NULL, "University of Tbingen", "http://www.gris.uni-tuebingen.de");
NVEB_EFFECT_ABOUT( 3, "Date", "June 2001", NULL);

#include "glh_extensions.h"
#include "glh_obs.h"
#include "glh_glut.h"
#include "glh_cube_map.h"

#include <nv_util/nv_jpeg.h>

#include "nvparse.h"

using namespace glh;
using namespace std;

glut_callbacks cb;
glut_simple_mouse_interactor camera, object;
glut_perspective_reshaper reshaper;

bool b[256];
int phong;
float light_rotation = 0;
int size = 8;
int component = 0;

// lighting map
tex_object_cube_map diffusemap;
tex_object_cube_map specularmap;
tex_object_2D** texture;

// frame per second counter
LARGE_INTEGER Frequency, Timer;

static double LastTime = 0.0;
static double CurrentTime = 0.0;
static double ElapsedTime = 0.0;
static double framedisplay = 10.0;
static int    framedisplayinc = 0;

static float GetSystemSeconds( void )
{
    QueryPerformanceFrequency(&Frequency);
    QueryPerformanceCounter(&Timer);
	return (float) Timer.QuadPart/ (float)Frequency.QuadPart;
}

// glut-ish callbacks
void display();
void key(unsigned char k, int x, int y);
void menu(int entry) { key((unsigned char)entry, 0, 0); }
void idle();

// my functions
void init_opengl();
void set_up_combiners();
void set_up_shaders();
void set_up_cube_maps();
void load_volume();
void down_sample_volume();

// single map
struct single_map
{
	typedef GLubyte Type;
	int components;
	GLenum type;
	GLenum format;
	single_map(float _power) :
	components(1), format(GL_LUMINANCE), type(GL_UNSIGNED_BYTE), power(_power) {}
	
	void operator() (const vec3f & v, Type * t)
	{
		// set up cube map using a single white light source
		float z = v[2] > 0 ? v[2] : 0;
		z = pow(z, power);
		t[0] = 255.0*z;
	}
	float power;
};

// color map
struct color_map
{
	typedef GLubyte Type;
	int components;
	GLenum type;
	GLenum format;
	color_map(float _power) :
	components(3), format(GL_RGB), type(GL_UNSIGNED_BYTE), power(_power) {}
	
	void operator() (const vec3f & v, Type * t)
	{
		// set up cube map using 3 colored light sources (red, green and blue)
		vec3f rv; rv[0]= 0.0  ; rv[1]= 0.5 ; rv[2]= 1.0; rv.normalize(); 
		vec3f gv; gv[0]= 0.433; gv[1]=-0.25; gv[2]= 1.0; gv.normalize();
		vec3f bv; bv[0]=-0.433; bv[1]=-0.25; bv[2]= 1.0; bv.normalize();
		float r = rv.dot(v); r = r > 0 ? r : 0;
		float g = gv.dot(v); g = g > 0 ? g : 0;
		float b = bv.dot(v); b = b > 0 ? b : 0;
		r = pow(r, power);
		g = pow(g, power);
		b = pow(b, power);
		t[0] = 255.0*r;
		t[1] = 255.0*g;
		t[2] = 255.0*b;
	}
	float power;
};

int main(int argc, char **argv)
{
	glutInit(&argc, argv);
	glutInitDisplayMode(GLUT_DOUBLE | GLUT_DEPTH | GLUT_RGBA );
	glutCreateWindow("volume lighting");

	b['d'] = true;  // diffuse lighting on by default
	b['s'] = true;  // specular lighting on by default
	b['c'] = false; // colored light off dy befault
	b['m'] = false; // magnitude off by default
	b['r'] = false; // large volume by default
	b[' '] = false; // no light rotation by default
	b['0'] = true;  // no classification by default
	b['1'] = false; // no classification by default
	b['2'] = false; // no classification by default
	b['3'] = false; // no classification by default
	b['4'] = false; // no classification by default
	phong  = 50;    // phong exponent 50 by default

	texture=new tex_object_2D*[960];
	for (int i=0; i<960; i++) texture[i]=new tex_object_2D();

	init_opengl();

	glut_helpers_initialize();

	cb.keyboard_function = key;
	camera.configure_buttons(1);
	camera.set_camera_mode(true);
	object.configure_buttons(1);
	object.dolly.dolly[2] = -1;

	glut_add_interactor(&cb);
	glut_add_interactor(&object);
	glut_add_interactor(&reshaper);

	int toggles = glutCreateMenu(menu);
	glutAddMenuEntry("diffuse lighting [d]", 'd');
	glutAddMenuEntry("specular lighting [s]", 's');
	glutAddMenuEntry("colored lights [c]", 'c');
	glutAddMenuEntry("gradient magnitude [m]", 'm');
	glutAddMenuEntry("reduced volume [r]", 'r');
	glutAddMenuEntry("rotate light source(s) [ ]", ' ');
	glutAddMenuEntry("cycle components [*]", '*');

	int tweak = glutCreateMenu(menu);
	glutAddMenuEntry("+ increase phong exponent [+]", '+');
	glutAddMenuEntry("- reduce phong exponent [-]", '-');
	glutAddMenuEntry("> increase cube map size [>]", '>');
	glutAddMenuEntry("< reduce cube map size [<]", '<');
	glutAddMenuEntry("0 no classifications [0]", '0');
	glutAddMenuEntry("1 remove noise classification [1]", '1');
	glutAddMenuEntry("2 simple classification [2]", '2');
	glutAddMenuEntry("3 alpha ramp classification [3]", '3');
	glutAddMenuEntry("4 color ramp classification [4]", '4');

	int copyrights = glutCreateMenu(menu);
	glutAddMenuEntry("volume_light (c) 2001 WSI-GRIS Stefan Guthe", 0);
	glutAddMenuEntry("GLH -- Copyright (c) 2001 NVIDIA Corporation", 0);
	glutAddMenuEntry("GLH -- Copyright (c) 2000 Cass Everitt", 0);

	glutCreateMenu(menu);
	glutAddSubMenu("toggles", toggles);
	glutAddSubMenu("tweakables", tweak);
	glutAddSubMenu("copyright info", copyrights);
	glutAddMenuEntry("quit [esc]", 27);
	glutAttachMenu(GLUT_RIGHT_BUTTON);

	glutDisplayFunc(display);
	glutMainLoop();
	return 0;
}

void init_opengl()
{
	if(! glh_init_extensions(
							 "GL_ARB_multitexture "
							 "GL_EXT_secondary_color "
							 "GL_NV_register_combiners "
							 "GL_NV_texture_shader "
							 "GL_ARB_texture_cube_map "
							 ))
	{
		cerr << "Unable to initialize because the following extensions were not supported:" << endl
			<< glh_get_unsupported_extensions() << endl << endl
			<< "Press <return> to quit." << endl;
		char buff[10];
		cin.getline(buff, 10);
		exit(0);
	}
	glClearColor(1.0f, 1.0f, 1.0f, 1.0f);
	glDisable(GL_CULL_FACE);
	glDisable(GL_LIGHTING);
	glEnable(GL_COLOR_MATERIAL);
	glEnable(GL_BLEND);
	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
	glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);
	glDepthMask(GL_FALSE);
	glDisable(GL_DEPTH_TEST);
	glEnable(GL_TEXTURE_2D);
	set_up_cube_maps();
	set_up_combiners();
	set_up_shaders();
	load_volume();
}

void key(unsigned char k, int x, int y)
{
	b[k] = ! b[k];
	if(k==27 || 'q'==k) exit(0);
	if('+'==k && phong<128) phong++;
	if('-'==k && phong>2) phong--;
	if('>'==k && size<16) size<<=1;
	if('<'==k && size>4) size>>=1;

	// toggle combiners features
	if('d'==k || 's'==k || 'm'==k || 'r'==k)
		set_up_combiners();
	if('d'==k || 's'==k)
		set_up_shaders();

	if(' '==k)
	{
		glutIdleFunc(b[k] ? idle : 0);
	}

	// change transfer function
	if('0'==k || '1'==k || '2'==k || '3'==k || '4'==k)
	{
		b['0']=b['1']=b['2']=b['3']=b['4']=false;
		b[k]=true;
		set_up_combiners();
	}

	// rebuild cube maps
	if('c'==k || '+'==k || '-'==k || '>'==k || '<'==k)
		set_up_cube_maps();

	if('*'==k) {
		component++; if (component>2) component=0;
		if (component) {
			glEnable(GL_TEXTURE_CUBE_MAP_ARB);
			glDisable(GL_TEXTURE_2D);
			glDisable(GL_BLEND);
			glDisable(GL_REGISTER_COMBINERS_NV);
			glDisable(GL_TEXTURE_SHADER_NV);
			glDepthMask(GL_TRUE);
			glEnable(GL_DEPTH_TEST);
		} else {
			glDisable(GL_TEXTURE_CUBE_MAP_ARB);
			glEnable(GL_TEXTURE_2D);
			glEnable(GL_BLEND);
			glEnable(GL_REGISTER_COMBINERS_NV);
			if (b['d'] || b['s']) glEnable(GL_TEXTURE_SHADER_NV);
			glDepthMask(GL_FALSE);
			glDisable(GL_DEPTH_TEST);
		}
	}

	glutPostRedisplay();
}

void idle()
{
	light_rotation += to_radians(2);
	glutPostRedisplay();
}

void set_up_combiners()
{
	string combiner_program;
	combiner_program="!!RC1.0\n"
					 "const0 = (1.0, 1.0, 1.0, 0.0);\n";
	if (b['m']) {
		combiner_program+="{\n"
						  "  const0 = (0.0, 0.0, 0.0, 0.25);\n"
						  "  rgb {\n"
						  "    tex0 = expand(tex0) . expand(tex0); /* calculated squared norm of gradient */\n"
						  "  }\n";
		if (b['0']) {
			combiner_program+="}\n";
		}
	} else if (!b['0']) {
		combiner_program+="{\n";
		if (!b['4']) combiner_program+="  const0 = (0.0, 0.0, 0.0, 0.25);\n";
	}
	if (b['1']) {
		combiner_program+="  alpha {\n"
						  "    discard = tex0;\n"
						  "    discard = -const0;\n"
						  "    tex0 = sum(); /* remove noise: tex0.a = 2*(tex0.a-0.25) */\n"
						  "    scale_by_two();\n"
						  "  }\n"
						  "}\n";
	} else if (b['2']) {
		combiner_program+="  alpha {\n"
						  "    discard = tex0*-half_bias(zero);\n"
						  "    discard = unsigned_invert(const0)*-half_bias(zero);\n"
						  "    spare0 = sum(); /* noise: spare0.a < 0.5, spare0.a = 0.5*(tex0.a+1.0-0.25) */\n"
						  "  }\n"
						  "}\n"
						  "{\n"
						  "  const0 = (0.0, 0.0, 0.0, 0.0);\n"
						  "  const1 = (1.0, 1.0, 1.0, 0.071);\n"
						  "  rgb {\n"
						  "    discard = const0;\n"
						  "    discard = const1;\n"
						  "    tex1 = mux(); /* tex0.a < 0.25: tex1 = const0 otherwise tex1 = const1 */\n"
						  "  }\n"
						  "  alpha {\n"
						  "    discard = const0;\n"
						  "    discard = const1;\n"
						  "    tex1 = mux(); /* tex0.a < 0.25: tex1 = const0 otherwise tex1 = const1 */\n"
						  "  }\n"
						  "}\n"
						  "{\n"
						  "  const0 = (0.0, 0.0, 0.0, 0.75);\n"
						  "  rgb {\n"
						  "    discard = zero;\n"
						  "  }\n"
						  "  alpha {\n"
						  "    discard = tex0*-half_bias(zero);\n"
						  "    discard = unsigned_invert(const0)*-half_bias(zero);\n"
						  "    spare0 = sum(); /* high opacity: spare0.a > 0.5, spare0.a = 0.5*(tex0.a+1.0-0.75) */\n"
						  "  }\n"
						  "}\n"
						  "{\n"
						  "  const1 = (1.0, 0.0, 0.0, 1.0);\n"
						  "  rgb {\n"
						  "    discard = tex1;\n"
						  "    discard = const1;\n"
						  "    tex1 = mux(); /* tex0.a > 0.75: tex1 = const1 otherwise tex1 = tex1 */\n"
						  "  }\n"
						  "  alpha {\n"
						  "    discard = tex1;\n"
						  "    discard = const1;\n"
						  "    tex1 = mux(); /* tex0.a > 0.75: tex1 = const1 otherwise tex1 = tex1 */\n"
						  "  }\n"
						  "}\n";
	} else if (b['3']) {
		combiner_program+="  alpha {\n"
						  "    discard = tex0*-half_bias(zero);\n"
						  "    discard = unsigned_invert(const0)*-half_bias(zero);\n"
						  "    spare0 = sum(); /* noise: spare0.a < 0.5, spare0.a = 0.5*(tex0.a+1.0-0.25) */\n"
						  "  }\n"
						  "}\n"
						  "{\n"
						  "  const0 = (0.0, 0.0, 0.0, 0.0);\n"
						  "  const1 = (1.0, 1.0, 1.0, 0.25);\n"
						  "  rgb {\n"
						  "    discard = const0;\n"
						  "    discard = const1;\n"
						  "    tex1 = mux(); /* tex0.a < 0.25: tex1 = const0 otherwise tex1 = const1 */\n"
						  "  }\n"
						  "  alpha {\n"
						  "    discard = tex0;\n"
						  "    discard = -const1;\n"
						  "    tex1 = sum(); /* tex1.a = 0.5*(tex0.a-0.25)*/\n"
						  "    scale_by_one_half();\n"
						  "  }\n"
						  "}\n"
						  "{\n"
						  "  const0 = (0.0, 0.0, 0.0, 0.75);\n"
						  "  rgb {\n"
						  "    discard = zero;\n"
						  "  }\n"
						  "  alpha {\n"
						  "    discard = tex0*-half_bias(zero);\n"
						  "    discard = unsigned_invert(const0)*-half_bias(zero);\n"
						  "    spare0 = sum(); /* high opacity: spare0.a > 0.5, spare0.a = 0.5*(tex0.a+1.0-0.75) */\n"
						  "  }\n"
						  "}\n"
						  "{\n"
						  "  const1 = (1.0, 0.0, 0.0, 1.0);\n"
						  "  rgb {\n"
						  "    discard = tex1;\n"
						  "    discard = const1;\n"
						  "    tex1 = mux(); /* tex0.a > 0.75: tex1 = const1 otherwise tex1 = tex1 */\n"
						  "  }\n"
						  "  alpha {\n"
						  "    discard = tex1;\n"
						  "    discard = const1;\n"
						  "    tex1 = mux(); /* tex0.a > 0.75: tex1 = const1 otherwise tex1 = tex1 */\n"
						  "  }\n"
						  "}\n";
	} else if (b['4']) {
		combiner_program+="  alpha {\n"
						  "    discard = expand(tex0);\n"
						  "    discard = -half_bias(zero);\n"
						  "    spare0 = sum(); /* spare0 = 2*tex0-0.5 */\n"
						  "  }\n"
						  "}\n"
						  "{\n"
						  "  const0 = (0.0, 0.0, 1.0, 0.25);\n"
						  "  const1 = (1.0, 0.0, 1.0, 0.0);\n"
						  "  rgb {\n"
						  "    discard = const0*unsigned_invert(spare0.a);\n"
						  "    discard = const1*spare0.a;\n"
						  "    tex1 = sum(); /* interpolate between blue and magenta depending on spare0.a */\n"
						  "  }\n"
						  "  alpha {\n"
						  "    discard = tex0*-half_bias(zero);\n"
						  "    discard = unsigned_invert(const0)*-half_bias(zero);\n"
						  "    spare0 = sum(); /* noise: spare0.a < 0.5, spare0.a = 0.5*(tex0.a+1.0-0.25) */\n"
						  "  }\n"
						  "}\n"
						  "{\n"
						  "  const0 = (0.0, 0.0, 0.0, 0.0);\n"
						  "  const1 = (1.0, 1.0, 1.0, 0.071);\n"
						  "  rgb {\n"
						  "    discard = const0;\n"
						  "    discard = tex1;\n"
						  "    tex1 = mux(); /* tex0.a < 0.25: tex1 = const0 otherwise tex1 = tex1 */\n"
						  "  }\n"
						  "  alpha {\n"
						  "    discard = const0;\n"
						  "    discard = const1;\n"
						  "    tex1 = mux(); /* tex0.a < 0.25: tex1 = const0 otherwise tex1 = const1 */\n"
						  "  }\n"
						  "}\n"
						  "{\n"
						  "  const0 = (0.0, 0.0, 0.0, 0.75);\n"
						  "  rgb {\n"
						  "    discard = zero;\n"
						  "  }\n"
						  "  alpha {\n"
						  "    discard = tex0*-half_bias(zero);\n"
						  "    discard = unsigned_invert(const0)*-half_bias(zero);\n"
						  "    spare0 = sum(); /* high opacity: spare0.a > 0.5, spare0.a = 0.5*(tex0.a+1.0-0.75) */\n"
						  "  }\n"
						  "}\n"
						  "{\n"
						  "  const1 = (1.0, 0.0, 0.0, 1.0);\n"
						  "  rgb {\n"
						  "    discard = tex1;\n"
						  "    discard = const1;\n"
						  "    tex1 = mux(); /* tex0.a > 0.75: tex1 = const1 otherwise tex1 = tex1 */\n"
						  "  }\n"
						  "  alpha {\n"
						  "    discard = tex1;\n"
						  "    discard = const1;\n"
						  "    tex1 = mux(); /* tex0.a > 0.75: tex1 = const1 otherwise tex1 = tex1 */\n"
						  "  }\n"
						  "}\n";
	}
	combiner_program+="{\n";
	if (b['d'] && b['s']) {
		combiner_program+="  const0 = (0.2, 0.2, 0.2, 0.0);\n"
						  "  rgb {\n"
						  "    discard = col0 * tex2;\n"
						  "    discard = const0;\n"
						  "    col0 = sum(); /* col0 = Lambient + Cdiffuse*Ldiffuse */\n";
	} else if (b['d']) {
		combiner_program+="  const0 = (0.2, 0.2, 0.2, 0.0);\n"
						  "  rgb {\n"
						  "    discard = col0 * tex3;\n"
						  "    discard = const0;\n"
						  "    col0 = sum(); /* col0 = Lambient + Cdiffuse*Ldiffuse */\n";
	} else {
		combiner_program+="  const0 = (1.0, 1.0, 1.0, 0.0);\n"
						  "  rgb {\n"
						  "    col0 = const0; /* col0 = Lambient */\n";
	}
	combiner_program+="  }\n"
					  "  alpha {\n";
	if (b['m']) {
		if (b['0'] || b['1']) {
			combiner_program+="    tex1 = tex0 * tex0.b; /* use gradient magnitude to modify transparancy */\n"
							  "    scale_by_two();\n";
		} else {
			combiner_program+="    tex1 = tex1 * tex0.b; /* use gradient magnitude to modify transparancy */\n"
							  "    scale_by_four();\n";
		}
		if (b['r']) {
			combiner_program+="  }\n"
							  "}\n"
							  "{\n"
							  "  rgb {\n"
							  "    discard = zero;\n"
							  "  }\n"
							  "  alpha {\n"
							  "    tex1 = unsigned_invert(tex1)*unsigned_invert(tex1); /* correct alpha value for lowres texture */\n";
		}
	} else {
		if (b['0'] || b['1']) {
			if (b['r']) {
				combiner_program+="    tex1 = unsigned_invert(tex0)*unsigned_invert(tex0); /* correct alpha value for lowres texture */\n";
			} else {
				combiner_program+="    tex1 = tex0;\n";
			}
		} else {
			if (b['r']) {
				combiner_program+="    tex1 = unsigned_invert(tex1)*unsigned_invert(tex1); /* correct alpha value for lowres texture */\n";
			} else {
				combiner_program+="    discard = zero;\n";
			}
		}
	}
	combiner_program+="  }\n"
					  "}\n";
	// calculate final lighting
	if (b['0'] || b['1']) {
		if (b['d']) {
			if (b['s']) {
				combiner_program+="/* output = Lambient + Cdiffuse*Ldiffuse + Cspecular*Lspecular */\n"
								  "final_product = tex3 * const0;\n"
								  "out.rgb = unsigned_invert(zero) * col0 + final_product;\n";
			} else {
				combiner_program+="/* output = Lambient + Cdiffuse*Ldiffuse */\n"
								  "out.rgb = unsigned_invert(zero) * col0;\n";
			}
		} else {
			if (b['s']) {
				combiner_program+="/* output = Cvolume*Lambient + Cspecular*Lspecular */\n"
								  "final_product = tex3 * const0;\n"
								  "out.rgb = tex0.a * col0 + final_product;\n";
			} else {
				combiner_program+="/* output = Cvolume*Lambient */\n"
								  "out.rgb = tex0.a * col0;\n";
			}
		}
	} else {
		if (b['s']) {
			combiner_program+="/* output = Cvolume*(Lambient + Cdiffuse*Ldiffuse) + Cspecular*Lspecular */\n"
							  "final_product = tex3 * const0;\n"
							  "out.rgb = tex1 * col0 + final_product;\n";
		} else {
			combiner_program+="/* output = Cvolume*(Lambient + Cdiffuse*Ldiffuse)*/\n"
							  "out.rgb = tex1 * col0;\n";
		}
	}
	if (b['r']) {
		combiner_program+="out.a = unsigned_invert(tex1); /* invert alpha due to correction for lowres texture */\n";
	} else {
		combiner_program+="out.a = tex1;\n";
	}

	nvparse(combiner_program.c_str());

	for (char * const *  errors = nvparse_get_errors(); *errors; errors++)
	{
		const char * errstr = *errors;
		fprintf(stderr, errstr);
	}

	glEnable(GL_PER_STAGE_CONSTANTS_NV);
	glEnable(GL_REGISTER_COMBINERS_NV);
}

void set_up_shaders()
{
	if (b['s'] && b['d']) {
		nvparse("!!TS1.0\n"
				"/* volume slice stored in 2d texture */\n"
				"texture_2d();\n"
				"/* cube maps used for lighting (diffuse and specular)*/\n"
				"dot_product_cube_map_and_reflect_cube_map_eye_from_qs_1of3(expand(tex0));\n"
				"dot_product_cube_map_and_reflect_cube_map_eye_from_qs_2of3(expand(tex0));\n"
				"dot_product_cube_map_and_reflect_cube_map_eye_from_qs_3of3(expand(tex0));\n");
		for (char * const *  errors = nvparse_get_errors(); *errors; errors++)
		{
			const char * errstr = *errors;
			fprintf(stderr, errstr);
		}
		glActiveTextureARB( GL_TEXTURE2_ARB );
		diffusemap.bind();
		glActiveTextureARB( GL_TEXTURE3_ARB );
		specularmap.bind();
		glActiveTextureARB( GL_TEXTURE0_ARB );
	} else if (b['s']) {
		nvparse("!!TS1.0\n"
				"/* volume slice stored in 2d texture */\n"
				"texture_2d();\n"
				"/* cube map used for lighting (specular)*/\n"
				"dot_product_reflect_cube_map_eye_from_qs_1of3(expand(tex0));\n"
				"dot_product_reflect_cube_map_eye_from_qs_2of3(expand(tex0));\n"
				"dot_product_reflect_cube_map_eye_from_qs_3of3(expand(tex0));\n");
		for (char * const *  errors = nvparse_get_errors(); *errors; errors++)
		{
			const char * errstr = *errors;
			fprintf(stderr, errstr);
		}
		glActiveTextureARB( GL_TEXTURE2_ARB );
		glActiveTextureARB( GL_TEXTURE3_ARB );
		specularmap.bind();
		glActiveTextureARB( GL_TEXTURE0_ARB );
	} else if (b['d']) {
		nvparse("!!TS1.0\n"
				"/* volume slice stored in 2d texture */\n"
				"texture_2d();\n"
				"/* cube map used for lighting (diffuse)*/\n"
				"dot_product_cube_map_1of3(expand(tex0));\n"
				"dot_product_cube_map_2of3(expand(tex0));\n"
				"dot_product_cube_map_3of3(expand(tex0));\n");
		for (char * const *  errors = nvparse_get_errors(); *errors; errors++)
		{
			const char * errstr = *errors;
			fprintf(stderr, errstr);
		}
		glActiveTextureARB( GL_TEXTURE2_ARB );
		glActiveTextureARB( GL_TEXTURE3_ARB );
		diffusemap.bind();
		glActiveTextureARB( GL_TEXTURE0_ARB );
	} else {
		nvparse("!!TS1.0\n"
				"/* volume slice stored in 2d texture */\n"
				"texture_2d();\n"
				"nop();\n"
				"nop();\n"
				"nop();\n");
		for (char * const *  errors = nvparse_get_errors(); *errors; errors++)
		{
			const char * errstr = *errors;
			fprintf(stderr, errstr);
		}
		glActiveTextureARB( GL_TEXTURE2_ARB );
		glActiveTextureARB( GL_TEXTURE3_ARB );
		glActiveTextureARB( GL_TEXTURE0_ARB );
	}
	glEnable(GL_TEXTURE_SHADER_NV);
}

void set_up_cube_maps()
{
	if (b['c']) {
		diffusemap.bind();
		color_map diffuse_color(1);
		make_cube_map(diffuse_color, GL_RGB, size);
		diffusemap.parameter(GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
		diffusemap.parameter(GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);
		diffusemap.parameter(GL_TEXTURE_WRAP_R, GL_CLAMP_TO_EDGE);
		diffusemap.parameter(GL_TEXTURE_MIN_FILTER, GL_LINEAR);
		diffusemap.parameter(GL_TEXTURE_MAG_FILTER, GL_LINEAR);

		specularmap.bind();
		color_map specular_color(phong);
		make_cube_map(specular_color, GL_RGB, size*8);
		specularmap.parameter(GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
		specularmap.parameter(GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);
		specularmap.parameter(GL_TEXTURE_WRAP_R, GL_CLAMP_TO_EDGE);
		specularmap.parameter(GL_TEXTURE_MIN_FILTER, GL_LINEAR);
		specularmap.parameter(GL_TEXTURE_MAG_FILTER, GL_LINEAR);
	} else {
		diffusemap.bind();
		single_map diffuse_single(1);
		make_cube_map(diffuse_single, GL_LUMINANCE, size);
		diffusemap.parameter(GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
		diffusemap.parameter(GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);
		diffusemap.parameter(GL_TEXTURE_WRAP_R, GL_CLAMP_TO_EDGE);
		diffusemap.parameter(GL_TEXTURE_MIN_FILTER, GL_LINEAR);
		diffusemap.parameter(GL_TEXTURE_MAG_FILTER, GL_LINEAR);

		specularmap.bind();
		single_map specular_single(phong);
		make_cube_map(specular_single, GL_LUMINANCE, size*8);
		specularmap.parameter(GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
		specularmap.parameter(GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);
		specularmap.parameter(GL_TEXTURE_WRAP_R, GL_CLAMP_TO_EDGE);
		specularmap.parameter(GL_TEXTURE_MIN_FILTER, GL_LINEAR);
		specularmap.parameter(GL_TEXTURE_MAG_FILTER, GL_LINEAR);
	}
}

void display()
{
	vec3f l=(1,1,3);  l.set_value (-1.0,0.0,0.0);
	vec3f l1=(1,1,3); l1.set_value(0.0,-1.0,0.0);
	vec3f l2=(1,1,3); l2.set_value(0.0,0.0,-1.0);
	vec3f e=(1,1,3);

	l.normalize();
	quaternionf q(vec3f(0,1,0), light_rotation);
	q.mult_vec(l); 
	q.mult_vec(l1);
	q.mult_vec(l2);

	vec3f eye(0,0,1);
	object.trackball.r.inverse().mult_vec(eye); // rotate eye vector into quad's object space
	e.set_value(eye.dot(l),eye.dot(l1),eye.dot(l2));
	e.normalize();

	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	glPushMatrix();
	camera.apply_inverse_transform();

	object.apply_transform();

	int dir=0;
	// determine main viewing direction
	if(fabs(eye[0])>fabs(eye[1]) && fabs(eye[0])>fabs(eye[2]) && eye[0]<0) dir=0;
	else if(fabs(eye[0])>fabs(eye[1]) && fabs(eye[0])>fabs(eye[2]) && eye[0]>0) dir=1;
	else if(fabs(eye[1])>fabs(eye[0]) && fabs(eye[1])>fabs(eye[2]) && eye[1]<0) dir=2;
	else if(fabs(eye[1])>fabs(eye[0]) && fabs(eye[1])>fabs(eye[2]) && eye[1]>0) dir=3;
	else if(fabs(eye[2])>fabs(eye[0]) && fabs(eye[2])>fabs(eye[1]) && eye[2]<0) dir=4;
	else if(fabs(eye[2])>fabs(eye[0]) && fabs(eye[2])>fabs(eye[1]) && eye[2]>0) dir=5;

	int x,y,z,i;
	float p;
	float sx,sy,sz;
	int dimX,dimY,dimZ;
	int o;

	if (b['r']) {
		dimX=128;
		dimY=128;
		dimZ=64;
		o=640;
	} else {
		dimX=256;
		dimY=256;
		dimZ=128;
		o=0;
	}

	sx=2.0/(dimX-1);
	sy=2.0/(dimY-1);
	sz=2.0/(dimZ-1);

	// show cube maps used for lighting
	if (component>0) {
		glScalef(0.4, 0.4, 0.4);
		glColor4f(1.0f, 1.0f, 1.0f, 1.0f);
		// show diffuse map ...
		if (component==1) diffusemap.bind();
		// ... or specular map
		else specularmap.bind();
		// draw cube
		glBegin(GL_QUADS);
			glTexCoord3f(-1.0f,-1.0f,-1.0f);
			glVertex3f  (-1.0f,-1.0f,-1.0f);
			glTexCoord3f( 1.0f,-1.0f,-1.0f);
			glVertex3f  ( 1.0f,-1.0f,-1.0f);
			glTexCoord3f( 1.0f, 1.0f,-1.0f);
			glVertex3f  ( 1.0f, 1.0f,-1.0f);
			glTexCoord3f(-1.0f, 1.0f,-1.0f);
			glVertex3f  (-1.0f, 1.0f,-1.0f);
		glEnd();
		glBegin(GL_QUADS);
			glTexCoord3f(-1.0f,-1.0f, 1.0f);
			glVertex3f  (-1.0f,-1.0f, 1.0f);
			glTexCoord3f( 1.0f,-1.0f, 1.0f);
			glVertex3f  ( 1.0f,-1.0f, 1.0f);
			glTexCoord3f( 1.0f, 1.0f, 1.0f);
			glVertex3f  ( 1.0f, 1.0f, 1.0f);
			glTexCoord3f(-1.0f, 1.0f, 1.0f);
			glVertex3f  (-1.0f, 1.0f, 1.0f);
		glEnd();
		glBegin(GL_QUADS);
			glTexCoord3f(-1.0f,-1.0f,-1.0f);
			glVertex3f  (-1.0f,-1.0f,-1.0f);
			glTexCoord3f(-1.0f, 1.0f,-1.0f);
			glVertex3f  (-1.0f, 1.0f,-1.0f);
			glTexCoord3f(-1.0f, 1.0f, 1.0f);
			glVertex3f  (-1.0f, 1.0f, 1.0f);
			glTexCoord3f(-1.0f,-1.0f, 1.0f);
			glVertex3f  (-1.0f,-1.0f, 1.0f);
		glEnd();
		glBegin(GL_QUADS);
			glTexCoord3f( 1.0f,-1.0f,-1.0f);
			glVertex3f  ( 1.0f,-1.0f,-1.0f);
			glTexCoord3f( 1.0f, 1.0f,-1.0f);
			glVertex3f  ( 1.0f, 1.0f,-1.0f);
			glTexCoord3f( 1.0f, 1.0f, 1.0f);
			glVertex3f  ( 1.0f, 1.0f, 1.0f);
			glTexCoord3f( 1.0f,-1.0f, 1.0f);
			glVertex3f  ( 1.0f,-1.0f, 1.0f);
		glEnd();
		glBegin(GL_QUADS);
			glTexCoord3f(-1.0f,-1.0f,-1.0f);
			glVertex3f  (-1.0f,-1.0f,-1.0f);
			glTexCoord3f( 1.0f,-1.0f,-1.0f);
			glVertex3f  ( 1.0f,-1.0f,-1.0f);
			glTexCoord3f( 1.0f,-1.0f, 1.0f);
			glVertex3f  ( 1.0f,-1.0f, 1.0f);
			glTexCoord3f(-1.0f,-1.0f, 1.0f);
			glVertex3f  (-1.0f,-1.0f, 1.0f);
		glEnd();
		glBegin(GL_QUADS);
			glTexCoord3f(-1.0f, 1.0f,-1.0f);
			glVertex3f  (-1.0f, 1.0f,-1.0f);
			glTexCoord3f( 1.0f, 1.0f,-1.0f);
			glVertex3f  ( 1.0f, 1.0f,-1.0f);
			glTexCoord3f( 1.0f, 1.0f, 1.0f);
			glVertex3f  ( 1.0f, 1.0f, 1.0f);
			glTexCoord3f(-1.0f, 1.0f, 1.0f);
			glVertex3f  (-1.0f, 1.0f, 1.0f);
		glEnd();
	} else {
		glScalef(0.4, 0.4, 0.2);
		glColor4f(0.6f, 0.6f, 0.6f, 1.0f);

		glMultiTexCoord4fARB(GL_TEXTURE1_ARB, l [0], l [1], l [2], -e[0]);
		glMultiTexCoord4fARB(GL_TEXTURE2_ARB, l1[0], l1[1], l1[2], -e[1]);
		glMultiTexCoord4fARB(GL_TEXTURE3_ARB, l2[0], l2[1], l2[2], -e[2]);
		// depending on the main viewing direction select a stack of textures and
		// draw the corresponsding in the correct order
		switch (dir) {
		case 0:
			for (x=0,i=o+dimX-1,p=1.0; x<dimX; x++,i--,p-=sx) {
				texture[i]->bind();
				glBegin(GL_QUADS);
					glMultiTexCoord2fARB(GL_TEXTURE0_ARB, 0.0f, 0.0f);
					glVertex3f(p, -1.0, -1.0);
					glMultiTexCoord2fARB(GL_TEXTURE0_ARB, 0.0f, 1.0f);
					glVertex3f(p, -1.0,  1.0);
					glMultiTexCoord2fARB(GL_TEXTURE0_ARB, 1.0f, 1.0f);
					glVertex3f(p,  1.0,  1.0);
					glMultiTexCoord2fARB(GL_TEXTURE0_ARB, 1.0f, 0.0f);
					glVertex3f(p,  1.0, -1.0);
				glEnd();
			}
			break;
		case 1:
			for (x=0,i=o,p=-1.0; x<dimX; x++,i++,p+=sx) {
				texture[i]->bind();
				glBegin(GL_QUADS);
					glMultiTexCoord2fARB(GL_TEXTURE0_ARB, 0.0f, 0.0f);
					glVertex3f(p, -1.0, -1.0);
					glMultiTexCoord2fARB(GL_TEXTURE0_ARB, 1.0f, 0.0f);
					glVertex3f(p,  1.0, -1.0);
					glMultiTexCoord2fARB(GL_TEXTURE0_ARB, 1.0f, 1.0f);
					glVertex3f(p,  1.0,  1.0);
					glMultiTexCoord2fARB(GL_TEXTURE0_ARB, 0.0f, 1.0f);
					glVertex3f(p, -1.0,  1.0);
				glEnd();
			}
			break;
		case 2:
			for (y=0,i=o+dimX+dimY-1,p=1.0; y<dimY; y++,i--,p-=sy) {
				texture[i]->bind();
				glBegin(GL_QUADS);
					glMultiTexCoord2fARB(GL_TEXTURE0_ARB, 0.0f, 0.0f);
					glVertex3f(-1.0, p, -1.0);
					glMultiTexCoord2fARB(GL_TEXTURE0_ARB, 0.0f, 1.0f);
					glVertex3f( 1.0, p, -1.0);
					glMultiTexCoord2fARB(GL_TEXTURE0_ARB, 1.0f, 1.0f);
					glVertex3f( 1.0, p,  1.0);
					glMultiTexCoord2fARB(GL_TEXTURE0_ARB, 1.0f, 0.0f);
					glVertex3f(-1.0, p,  1.0);
				glEnd();
			}
			break;
		case 3:
			for (y=0,i=o+dimX,p=-1.0; y<dimY; y++,i++,p+=sy) {
				texture[i]->bind();
				glBegin(GL_QUADS);
					glMultiTexCoord2fARB(GL_TEXTURE0_ARB, 0.0f, 0.0f);
					glVertex3f(-1.0, p, -1.0);
					glMultiTexCoord2fARB(GL_TEXTURE0_ARB, 1.0f, 0.0f);
					glVertex3f(-1.0, p,  1.0);
					glMultiTexCoord2fARB(GL_TEXTURE0_ARB, 1.0f, 1.0f);
					glVertex3f( 1.0, p,  1.0);
					glMultiTexCoord2fARB(GL_TEXTURE0_ARB, 0.0f, 1.0f);
					glVertex3f( 1.0, p, -1.0);
				glEnd();
			}
			break;
		case 4:
			for (z=0,i=o+dimX+dimY+dimZ-1,p=1.0; z<dimZ; z++,i--,p-=sz) {
				texture[i]->bind();
				glBegin(GL_QUADS);
					glMultiTexCoord2fARB(GL_TEXTURE0_ARB, 0.0f, 0.0f);
					glVertex3f(-1.0, -1.0, p);
					glMultiTexCoord2fARB(GL_TEXTURE0_ARB, 0.0f, 1.0f);
					glVertex3f(-1.0,  1.0, p);
					glMultiTexCoord2fARB(GL_TEXTURE0_ARB, 1.0f, 1.0f);
					glVertex3f( 1.0,  1.0, p);
					glMultiTexCoord2fARB(GL_TEXTURE0_ARB, 1.0f, 0.0f);
					glVertex3f( 1.0, -1.0, p);
				glEnd();
			}
			break;
		default:
		case 5:
			for (z=0,i=o+dimX+dimY,p=-1.0; z<dimZ; z++,i++,p+=sz) {
				texture[i]->bind();
				glBegin(GL_QUADS);
					glMultiTexCoord2fARB(GL_TEXTURE0_ARB, 0.0f, 0.0f);
					glVertex3f(-1.0, -1.0, p);
					glMultiTexCoord2fARB(GL_TEXTURE0_ARB, 1.0f, 0.0f);
					glVertex3f( 1.0, -1.0, p);
					glMultiTexCoord2fARB(GL_TEXTURE0_ARB, 1.0f, 1.0f);
					glVertex3f( 1.0,  1.0, p);
					glMultiTexCoord2fARB(GL_TEXTURE0_ARB, 0.0f, 1.0f);
					glVertex3f(-1.0,  1.0, p);
				glEnd();
			}
		}
	}
	glPopMatrix();
    static char buf[64];
	glutSwapBuffers();


}

void load_volume()
{
	// load the volumes and their corresponding normals stored in a different images
	int x,y,z;
	unsigned char *vector;
	unsigned char *volume;
	int width, height, components;

	glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_REPLACE);
	// large volume (256x256x128)
	GLubyte* data=new GLubyte[262144];
	jpeg::read("./../../data/images/enginelv.jpg", &width, &height, &vector, &components);
	jpeg::read("./../../data/images/enginel.jpg",  &width, &height, &volume, &components);

	// the voxels have been stored in a different order for better compression
	for (x=0; x<256; x++) {
		for (z=0; z<128; z++)
			for (y=0; y<256; y++) {
				int p =(z<<10)+(y<<2);
				int p1=(((z&7)<<8)+y)+((((z&120)<<5)+x)<<11);
				int p0=p1*3;
				data[p++]=vector[p0++];
				data[p++]=vector[p0++];
				data[p++]=vector[p0  ];
				data[p  ]=volume[p1  ];
			}
		texture[x]->bind();
		texture[x]->parameter(GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
		texture[x]->parameter(GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);
		texture[x]->parameter(GL_TEXTURE_MIN_FILTER, GL_LINEAR);
		texture[x]->parameter(GL_TEXTURE_MAG_FILTER, GL_LINEAR);
		glTexImage2D(GL_TEXTURE_2D,0,GL_RGBA,256,128,0,GL_RGBA,GL_UNSIGNED_BYTE,&(data[0]));
	}
	for (y=0; y<256; y++) {
		for (x=0; x<256; x++)
			for (z=0; z<128; z++) {
				int p =(x<<9)+(z<<2);
				int p1=(((z&7)<<8)+y)+((((z&120)<<5)+x)<<11);
				int p0=p1*3;
				data[p++]=vector[p0++];
				data[p++]=vector[p0++];
				data[p++]=vector[p0  ];
				data[p  ]=volume[p1  ];
			}
		texture[y+256]->bind();
		texture[y+256]->parameter(GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
		texture[y+256]->parameter(GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);
		texture[y+256]->parameter(GL_TEXTURE_MIN_FILTER, GL_LINEAR);
		texture[y+256]->parameter(GL_TEXTURE_MAG_FILTER, GL_LINEAR);
		glTexImage2D(GL_TEXTURE_2D,0,GL_RGBA,128,256,0,GL_RGBA,GL_UNSIGNED_BYTE,&(data[0]));
	}
	for (z=0; z<128; z++) {
		for (y=0; y<256; y++)
			for (x=0; x<256; x++) {
				int p =(y<<10)+(x<<2);
				int p1=(((z&7)<<8)+y)+((((z&120)<<5)+x)<<11);
				int p0=p1*3;
				data[p++]=vector[p0++];
				data[p++]=vector[p0++];
				data[p++]=vector[p0  ];
				data[p  ]=volume[p1  ];
			}
		texture[z+512]->bind();
		texture[z+512]->parameter(GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
		texture[z+512]->parameter(GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);
		texture[z+512]->parameter(GL_TEXTURE_MIN_FILTER, GL_LINEAR);
		texture[z+512]->parameter(GL_TEXTURE_MAG_FILTER, GL_LINEAR);
		glTexImage2D(GL_TEXTURE_2D,0,GL_RGBA,256,256,0,GL_RGBA,GL_UNSIGNED_BYTE,&(data[0]));
	}

	// small volume (128x128x64)
	delete vector;
	delete volume;
	jpeg::read("./../../data/images/enginesv.jpg", &width, &height, &vector, &components);
	jpeg::read("./../../data/images/engines.jpg",  &width, &height, &volume, &components);
	// the voxels have been stored in a different order for better compression
	for (x=0; x<128; x++) {
		for (z=0; z<64; z++)
			for (y=0; y<128; y++) {
				int p =(z<<9)+(y<<2);
				int p1=(((z&7)<<7)+y)+((((z&56)<<4)+x)<<10);
				int p0=p1*3;
				data[p++]=vector[p0++];
				data[p++]=vector[p0++];
				data[p++]=vector[p0  ];
				data[p  ]=volume[p1  ];
			}
		texture[x+640]->bind();
		texture[x+640]->parameter(GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
		texture[x+640]->parameter(GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);
		texture[x+640]->parameter(GL_TEXTURE_MIN_FILTER, GL_LINEAR);
		texture[x+640]->parameter(GL_TEXTURE_MAG_FILTER, GL_LINEAR);
		glTexImage2D(GL_TEXTURE_2D,0,GL_RGBA,128,64,0,GL_RGBA,GL_UNSIGNED_BYTE,&(data[0]));
	}
	for (y=0; y<128; y++) {
		for (x=0; x<128; x++)
			for (z=0; z<64; z++) {
				int p =(x<<8)+(z<<2);
				int p1=(((z&7)<<7)+y)+((((z&56)<<4)+x)<<10);
				int p0=p1*3;
				data[p++]=vector[p0++];
				data[p++]=vector[p0++];
				data[p++]=vector[p0  ];
				data[p  ]=volume[p1  ];
			}
		texture[y+768]->bind();
		texture[y+768]->parameter(GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
		texture[y+768]->parameter(GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);
		texture[y+768]->parameter(GL_TEXTURE_MIN_FILTER, GL_LINEAR);
		texture[y+768]->parameter(GL_TEXTURE_MAG_FILTER, GL_LINEAR);
		glTexImage2D(GL_TEXTURE_2D,0,GL_RGBA,64,128,0,GL_RGBA,GL_UNSIGNED_BYTE,&(data[0]));
	}
	for (z=0; z<64; z++) {
		for (y=0; y<128; y++)
			for (x=0; x<128; x++) {
				int p =(y<<9)+(x<<2);
				int p1=(((z&7)<<7)+y)+((((z&56)<<4)+x)<<10);
				int p0=p1*3;
				data[p++]=vector[p0++];
				data[p++]=vector[p0++];
				data[p++]=vector[p0  ];
				data[p  ]=volume[p1  ];
			}
		texture[z+896]->bind();
		texture[z+896]->parameter(GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
		texture[z+896]->parameter(GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);
		texture[z+896]->parameter(GL_TEXTURE_MIN_FILTER, GL_LINEAR);
		texture[z+896]->parameter(GL_TEXTURE_MAG_FILTER, GL_LINEAR);
		glTexImage2D(GL_TEXTURE_2D,0,GL_RGBA,128,128,0,GL_RGBA,GL_UNSIGNED_BYTE,&(data[0]));
	}
	delete vector;
	delete volume;
	delete data;
}
