//
// shader_BumpRefract.h
// Copyright (C) 2001 Tomohide Kano. All rights reserved.
//

#ifndef __SHADER_BUMPREFRACT_H
#define __SHADER_BUMPREFRACT_H

#include <vector>

class CShaderBumpRefract;
class MouseUI;

typedef enum _eRenderStyle
{
	RENDER_REFLECT			= 1,
	RENDER_REFRACT			= 2,
	RENDER_REFLECT_REFRACT	= RENDER_REFLECT | RENDER_REFRACT,
	RENDER_FORCEDWORD		= 0xFFFFFFFF
} eRenderStyle;

typedef enum _eMouseState
{
	MOUSE_OBJ			= 0,
	MOUSE_ENV			= 1,
	MOUSE_FORCEDWORD	= 0xFFFFFFFF
} eMouseState;

struct BumpVertex
{
	D3DXVECTOR3 pos;
	D3DXVECTOR2 tex;
	D3DXVECTOR3 dpds;
	D3DXVECTOR3 dpdt;
};

class CShaderBumpRefract : public EBEffect
{
public:
	CShaderBumpRefract();
	virtual ~CShaderBumpRefract();

	virtual HRESULT Initialize(IDirect3DDevice8 *pDev);
	virtual HRESULT Free();
	virtual HRESULT Start();
	virtual HRESULT Tick(EBTimer* pTimer);
	virtual HRESULT ConfirmDevice(D3DCAPS8 *pCaps, DWORD dwBehavior, D3DFORMAT Format);
	virtual void    MouseButton(HWND hWnd, eButtonID button, bool bDown, int x, int y);
	virtual void    MouseMove(HWND hWnd, int x, int y);
	virtual void    Keyboard(DWORD dwKey, UINT nFlags, bool bDown);
	virtual void    UpdateProperties();
	virtual void    PropertyUpdateCallback(const EBProperty *pProperty, bool bWritten);

protected:
	HRESULT CreateShaders();
	HRESULT CreateTextures();

	HRESULT GenerateEnvBox();
	HRESULT GenerateQuad(D3DXVECTOR3& vCenter, FLOAT fRadius);
	HRESULT GenerateBezierPatch();
	void	InitBezierParams();
	HRESULT AnimateBezierPatch(EBTimer *pTimer);

	void	UpdateMatrices(EBTimer *pTimer);
	void	UpdateShaderConstants();
	void	DrawEnvBox();
	void	DrawObject();

protected:
	std::vector<LPDIRECT3DTEXTURE8>				m_pBumpMaps;
	std::vector<LPDIRECT3DTEXTURE8>::iterator	m_ppCurrentBumpMap;
	LPDIRECT3DCUBETEXTURE8						m_pCubeMap;

	LPDIRECT3DVERTEXBUFFER8	m_pBoxVertexBuffer;
	LPDIRECT3DINDEXBUFFER8	m_pBoxIndexBuffer;
	DWORD					m_dwBoxNumVertices;
	DWORD					m_dwBoxNumFaces;

	LPDIRECT3DVERTEXBUFFER8	m_pQuadVertexBuffer;
	LPDIRECT3DINDEXBUFFER8	m_pQuadIndexBuffer;
	DWORD					m_dwQuadNumVertices;
	DWORD					m_dwQuadNumFaces;

	LPDIRECT3DVERTEXBUFFER8	m_pPatchBuffer;
	D3DRECTPATCH_INFO		m_patchInfo;
	float					m_fNumSegs[4];
	float					m_fA[16];
	float					m_fB[16];
	float					m_fC[16];

	DWORD					m_dwEnvBoxVertexShader;
	DWORD					m_dwReflectVertexShader;
	DWORD					m_dwReflectVertexShader_Patch;
	DWORD					m_dwReflectPixelShader;
	DWORD					m_dwRefractVertexShader;
	DWORD					m_dwRefractVertexShader_Patch;
	DWORD					m_dwRefractPixelShader;

	eMouseState				m_eMouseState;
	MouseUI *				m_pMouseObj;
	MouseUI *				m_pMouseEnv;

	bool					m_bPause;
	bool					m_bWireframe;
	bool					m_bSupportsPatches;
	DWORD					m_bUsePatch;
	eRenderStyle			m_eRenderStyle;

	float					m_fTime;
	float					m_fBumpScale;
	float					m_fRefractIndex;

	D3DXMATRIX				m_matObj;
	D3DXMATRIX				m_matEnv;
	D3DXMATRIX				m_matProj;
};

#endif // __SHADER_BUMPREFRACT_H
