//
// shader_BumpUtil.cpp
// Copyright (C) 2001 Tomohide Kano. All rights reserved.
//

#include "eb_effect.h"
#include "nvdebug.h"
#include "shader_BumpUtil.h"

inline DWORD VectorToQ8W8V8U8(const D3DXVECTOR3& inVector)
{
	D3DXVECTOR3 vecScaled = inVector * 127.0f;

    signed char u = (signed char)vecScaled.x;
    signed char v = (signed char)vecScaled.y;
	signed char w = (signed char)vecScaled.z;
	signed char q = (signed char)0;

    return ( ((DWORD)(unsigned char)q << 24) |
			 ((DWORD)(unsigned char)w << 16) |
			 ((DWORD)(unsigned char)v <<  8) |
			 ((DWORD)(unsigned char)u <<  0) );
}

inline DWORD VectorToNVHS(const D3DXVECTOR3& inVector)
{
	D3DXVECTOR3 vecScaled = inVector * 32767.0f;
	
    signed short hi = (signed short)vecScaled.x;
    signed short lo = (signed short)vecScaled.y;

    return ( ((DWORD)(unsigned short)hi << 16) |
			 ((DWORD)(unsigned short)lo <<  0) );
}

_inline DWORD& MapData(D3DLOCKED_RECT& LockData, DWORD i, DWORD j)
{
	return *(DWORD*)(((BYTE*)LockData.pBits) + j*LockData.Pitch + i*sizeof(DWORD));
}

// Assumes height texture is 8888 and the height is in the red channel
static HRESULT CreateBumpMapFromHeight(
	LPDIRECT3DDEVICE8	pD3DDev,
	LPDIRECT3DTEXTURE8	pSource,
	LPDIRECT3DTEXTURE8	*pDestTexture,
	D3DFORMAT			format,
	D3DPOOL				pool,
  	DWORD (*VectorToDWORD)(const D3DXVECTOR3&))
{
	LPDIRECT3DTEXTURE8 pNormalMap = NULL;
	D3DLOCKED_RECT LockedDest;
	D3DLOCKED_RECT LockedSource;
	D3DSURFACE_DESC ddsdDescDest;
	D3DSURFACE_DESC ddsdDescSource;
	D3DXVECTOR3 normal;
	HRESULT hr;
	DWORD i, j;

	assert(pSource);

	pSource->GetLevelDesc(0, &ddsdDescSource);

	hr = pD3DDev->CreateTexture(ddsdDescSource.Width, ddsdDescSource.Height,
		1, 0, format, pool, &pNormalMap);
	if (FAILED(hr))
		return hr;

	pNormalMap->GetLevelDesc(0, &ddsdDescDest);

	DWORD dwWidth  = ddsdDescDest.Width;
	DWORD dwHeight = ddsdDescDest.Height;

	LPDIRECT3DTEXTURE8 pSourceTex = pSource;
	LPDIRECT3DTEXTURE8 pDestTex   = pNormalMap;

	pSourceTex->LockRect(0, &LockedSource, NULL, 0);
	pDestTex->LockRect(0, &LockedDest, NULL, 0);

	ZeroMemory(LockedDest.pBits, (LockedDest.Pitch * dwHeight));

	for(i=1; i<dwWidth-1; i++)
	{
		for(j=1; j<dwHeight-1; j++)
		{
			float fRight = float(MapData(LockedSource, i+1, j) & 0xFF) / 255.0f;
			float fLeft  = float(MapData(LockedSource, i-1, j) & 0xFF) / 255.0f;
			float fUp    = float(MapData(LockedSource, i, j-1) & 0xFF) / 255.0f;
			float fDown  = float(MapData(LockedSource, i, j+1) & 0xFF) / 255.0f;

			normal.x = fLeft - fRight;
			normal.y = fUp - fDown;
			normal.z = 2.0f;
			D3DXVec3Normalize(&normal, &normal);

			MapData(LockedDest, i, j) = VectorToDWORD(normal);
		}
	}

	for(i=0; i<dwWidth; i++)
	{
		MapData(LockedDest, i, 0)          = MapData(LockedDest, i, 1);
		MapData(LockedDest, i, dwHeight-1) = MapData(LockedDest, i, dwHeight-2);
	}
	for(j=0; j<dwHeight; j++)
	{
		MapData(LockedDest, 0, j)         = MapData(LockedDest, j, 1);
		MapData(LockedDest, dwWidth-1, j) = MapData(LockedDest, dwWidth-2, j);
	}

	pDestTex->UnlockRect(0);
	pSourceTex->UnlockRect(0);

	*pDestTexture = pNormalMap;
	return S_OK;
}

HRESULT CreateQ8W8V8U8FromHeight(
	LPDIRECT3DDEVICE8	pD3DDev,
	LPDIRECT3DTEXTURE8	pSource,
	LPDIRECT3DTEXTURE8	*pDestTexture)
{
	return CreateBumpMapFromHeight(pD3DDev, pSource, pDestTexture,
			D3DFMT_Q8W8V8U8, D3DPOOL_MANAGED, VectorToQ8W8V8U8);
}

HRESULT CreateNVHSFromHeight(
	LPDIRECT3DDEVICE8	pD3DDev,
	LPDIRECT3DTEXTURE8	pSource,
	LPDIRECT3DTEXTURE8	*pDestTexture)
{
	return CreateBumpMapFromHeight(pD3DDev, pSource, pDestTexture,
			(D3DFORMAT)MAKEFOURCC('N', 'V', 'H', 'S'), D3DPOOL_DEFAULT, VectorToNVHS);
}

