/******************************************************************************

  Copyright (C) 1999, 2000 NVIDIA Corporation
  This file is provided without support, instruction, or implied warranty of any
  kind.  NVIDIA makes no guarantee of its fitness for a particular purpose and is
  not liable under any circumstances for any damages or loss whatsoever arising
  from the use or inability to use this file or items derived from it.
  
    Comments:
    
      
        
******************************************************************************/

#include "eb_effect.h"
#include "shader_Flying2.h"
#include "nvinc.h"

using namespace std;

DECLARE_EFFECT_MAIN()


const TestModeOn = false;


extern "C"
{

__declspec(dllexport) unsigned int GetNumEffects() { return (TestModeOn ? 2 : 1); }

__declspec(dllexport) EBEffect* CreateEffect(unsigned int EffectNum)
{	
	switch(EffectNum)
	{
		case 0:
			return new CShaderFlying2(true);
			break;
		case 1:
			return new CShaderFlying2(false);
			break;
		default:
			return NULL;
	}
}

}




const ArraySize = 12;
const float ArrayDistance = 1.0f; 
const float ArrayOffset = -5.5f;
const float SphereRadius = 0.20f; 

const float CameraStartZ = 1.8f;


const NumOfSpheres = ArraySize * ArraySize * ArraySize+1;



CShaderFlying2::CShaderFlying2(bool _UsePixelShader)
	: m_pVertexBuffer(NULL),
	m_pVertexBufferLight(NULL),
	m_NormalLookUp(NULL),
	m_EarthCubeMap(NULL),
	m_dwFlyVertCubeShader(0),
	m_dwFlyPixCubeShader(0),
	m_dwFlyVertLightShader(0),
	m_dwFlyPixLightShader(0),
	m_bWireframe(false),
	m_bManySpheres(true),
	m_bLighting(true),
	m_bTexture(true),
	UsePixelShader(_UsePixelShader),
	m_pUI(NULL)
{
	m_strEffectLocation = "3rd Party\\Imposters";

	if (UsePixelShader)
		m_strEffectName = "FlyingWorlds";
	else
		m_strEffectName = "FlyingWorlds-Test";

	m_strEffectVersion = "1.0";
}

void CShaderFlying2::UpdateProperties()
{
	EBEffect::UpdateProperties();

	m_pVertexShaderEnum->AddEnumerant(new EBEnumValue(m_pVertexShaderEnum, "Flying1", GetFilePath("FlyVertCube.nvv"), EBTYPE_STRING_PROP));
	m_pVertexShaderEnum->AddEnumerant(new EBEnumValue(m_pVertexShaderEnum, "Flying2", GetFilePath("FlyVertLight.nvv"), EBTYPE_STRING_PROP));

	if (UsePixelShader)
	{
		m_pPixelShaderEnum->AddEnumerant(new EBEnumValue(m_pPixelShaderEnum, "Flying1", GetFilePath("FlyPixCube.nvp"), EBTYPE_STRING_PROP));
		m_pPixelShaderEnum->AddEnumerant(new EBEnumValue(m_pPixelShaderEnum, "Flying2", GetFilePath("FlyPixLight.nvp"), EBTYPE_STRING_PROP));
	}

	AddProperty(new EBProperty("Wireframe", OBJECT_MEMBER(m_bWireframe), EBTYPE_BOOL_PROP));
	AddProperty(new EBProperty("Texture", OBJECT_MEMBER(m_bTexture), EBTYPE_BOOL_PROP));	
	AddProperty(new EBProperty("Many Spheres", OBJECT_MEMBER(m_bManySpheres), EBTYPE_BOOL_PROP));	
	AddProperty(new EBProperty("Lighting", OBJECT_MEMBER(m_bLighting), EBTYPE_BOOL_PROP));	

    string aboutText = "file://";
    aboutText += GetFilePath("..\\..\\..\\NVEffectsExplained.htm");
    aboutText += "#flyingworlds";
	SetAboutInfo( NULL, _T("Flying Worlds"), _T(aboutText.c_str()));

}

HRESULT CShaderFlying2::ConfirmDevice(D3DCAPS8* pCaps, DWORD dwBehavior, D3DFORMAT Format)
{
	if (UsePixelShader)
	{
		if(D3DSHADER_VERSION_MAJOR(pCaps->PixelShaderVersion) < 1.0)
		{
			m_strLastError = "Device does not support pixel shaders!";
			return E_FAIL;
		}
	}



	return S_OK;
}

CShaderFlying2::~CShaderFlying2()
{
	Free();	
}


inline DWORD F2DW( FLOAT f ) { return *((DWORD*)&f); }


// Write vertex data for a single billboard sphere
Fly2Vertex* InitSphere(Fly2Vertex* _pBuff, float _x, float _y, float _z, float _radius)
{
	// A Simple Quad
	_pBuff->Position = D3DXVECTOR3(_x, _y, _z);	
	_pBuff->Texture = D3DXVECTOR2(0.0f, 0.0f);
	_pBuff->RelativePosition = D3DXVECTOR2(-_radius, +_radius);
	_pBuff++;

	_pBuff->Position = D3DXVECTOR3(_x, _y, _z);	
	_pBuff->Texture = D3DXVECTOR2(1.0f, 0.0f);
	_pBuff->RelativePosition = D3DXVECTOR2(+_radius, +_radius);
	_pBuff++;

	_pBuff->Position = D3DXVECTOR3(_x, _y, _z);	
	_pBuff->Texture = D3DXVECTOR2(1.0f, 1.0f);
	_pBuff->RelativePosition = D3DXVECTOR2(+_radius, -_radius);
	_pBuff++;

	_pBuff->Position = D3DXVECTOR3(_x, _y, _z);			
	_pBuff->Texture = D3DXVECTOR2(0.0f, 1.0f);
	_pBuff->RelativePosition = D3DXVECTOR2(-_radius, -_radius);
	_pBuff ++;

	_pBuff->Position = D3DXVECTOR3(_x, _y, _z);	
	_pBuff->Texture = D3DXVECTOR2(0.0f, 0.0f);
	_pBuff->RelativePosition = D3DXVECTOR2(-_radius, +_radius);
	_pBuff++;

	_pBuff->Position = D3DXVECTOR3(_x, _y, _z);	
	_pBuff->Texture = D3DXVECTOR2(1.0f, 1.0f);
	_pBuff->RelativePosition = D3DXVECTOR2(+_radius, -_radius);
	_pBuff++;

	return(_pBuff);
}

HRESULT CShaderFlying2::Initialize(LPDIRECT3DDEVICE8 pDev)
{
	HRESULT hr;
	int i,j,k;

	m_pD3DDev = pDev;
	pDev->AddRef();

	RECT rect;
	rect.left = rect.top = 0;
	D3DVIEWPORT8 viewport;
	m_pD3DDev->GetViewport(&viewport);
	rect.bottom = viewport.Height;
	rect.right  = viewport.Width;
	m_pUI = new MouseUI((const RECT)rect);

	m_pUI->Translate( 0.0, 0.0, CameraStartZ);

	vector<DWORD> Declaration;
	Declaration.push_back(D3DVSD_STREAM(0));
	Declaration.push_back(D3DVSD_REG(0, D3DVSDT_FLOAT3));
	Declaration.push_back(D3DVSD_REG(1, D3DVSDT_FLOAT2));
	Declaration.push_back(D3DVSD_REG(2, D3DVSDT_FLOAT2));
	Declaration.push_back(D3DVSD_END());

	/* Creating shaders */

	hr = LoadAndCreateShader(GetFilePath("FlyVertCube.vso"), &Declaration[0], 0, SHADERTYPE_VERTEX, &m_dwFlyVertCubeShader);
	if (FAILED(hr))
		return hr;

	hr = LoadAndCreateShader(GetFilePath("FlyVertLight.vso"), &Declaration[0], 0, SHADERTYPE_VERTEX, &m_dwFlyVertLightShader);
	if (FAILED(hr))
		return hr;

	if (UsePixelShader)
	{
		hr = LoadAndCreateShader(GetFilePath("FlyPixCube.pso"), NULL, 0, SHADERTYPE_PIXEL, &m_dwFlyPixCubeShader);
		if (FAILED(hr))
			return hr;

		hr = LoadAndCreateShader(GetFilePath("FlyPixLight.pso"), NULL, 0, SHADERTYPE_PIXEL, &m_dwFlyPixLightShader);
		if (FAILED(hr))
			return hr;
	}

	/* Creating buffers */

	hr = m_pD3DDev->CreateVertexBuffer( 4 * sizeof(Fly2Vertex), D3DUSAGE_POINTS | D3DUSAGE_WRITEONLY, 0, D3DPOOL_DEFAULT, &m_pVertexBufferLight);
	if (FAILED(hr))
		return hr;

	hr = m_pD3DDev->CreateVertexBuffer( 6 * NumOfSpheres * sizeof(Fly2Vertex), D3DUSAGE_POINTS | D3DUSAGE_WRITEONLY, 0, D3DPOOL_DEFAULT, &m_pVertexBuffer);
	if (FAILED(hr))
		return hr;


	/* Init vertex buffer */
	
	Fly2Vertex* pBuff;
	hr = m_pVertexBuffer->Lock(0, 6 * NumOfSpheres * sizeof(Fly2Vertex),(BYTE**)&pBuff, 0);
	if (FAILED(hr))
	{
		m_strLastError = "Couldn't lock buffer!";
		return hr;
	}

	pBuff = InitSphere(pBuff, 0.0, 0.0, 0.0, 2.0*SphereRadius);

	for ( i = 0; i < ArraySize; i++)
		for ( j = 0; j < ArraySize; j++)
			for ( k = 0; k < ArraySize; k++)
				pBuff = InitSphere(pBuff, i*ArrayDistance + ArrayOffset, j*ArrayDistance + ArrayOffset, k*ArrayDistance + ArrayOffset, SphereRadius);
			
	m_pVertexBuffer->Unlock();
	

	// Create the normal texture
	hr = D3DXCreateTextureFromFileEx(m_pD3DDev, 
		GetFilePath("Normal.tga").c_str(),
		D3DX_DEFAULT,
		D3DX_DEFAULT,
		0,
		0,
		D3DFMT_UNKNOWN,
		D3DPOOL_MANAGED,
		D3DX_FILTER_LINEAR,
		D3DX_FILTER_LINEAR,
		0,
		NULL,
		NULL,
		&m_NormalLookUp);

	if (FAILED(hr))
	{
		m_strLastError = "Could not create Normal.tga";
		return hr;
	}

	// Create the color texture
	hr = D3DXCreateCubeTextureFromFileEx(m_pD3DDev, 
		GetFilePath("earth_cubemap.dds").c_str(),
		D3DX_DEFAULT,
		0,
		0,
		D3DFMT_UNKNOWN,
		D3DPOOL_MANAGED,
		D3DX_FILTER_LINEAR,
		D3DX_FILTER_LINEAR,
		0,
		NULL,
		NULL,
		&m_EarthCubeMap);

	if (FAILED(hr))
	{
		m_strLastError = "Could not create earth_cubemap.dds";
		return hr;
	}

	return S_OK;

}

HRESULT CShaderFlying2::Free()
{
	SAFE_RELEASE(m_EarthCubeMap);
	SAFE_RELEASE(m_NormalLookUp);
	SAFE_RELEASE(m_pVertexBuffer);
	SAFE_RELEASE(m_pVertexBufferLight);
	SAFE_DELETE(m_pUI);

	if (m_pD3DDev)
	{
		if (m_dwFlyVertCubeShader)
			m_pD3DDev->DeleteVertexShader(m_dwFlyVertCubeShader);

		if (m_dwFlyVertLightShader)
			m_pD3DDev->DeleteVertexShader(m_dwFlyVertLightShader);

		if (UsePixelShader)
		{
			if (m_dwFlyPixCubeShader)
				m_pD3DDev->DeletePixelShader(m_dwFlyPixCubeShader);
			if (m_dwFlyPixLightShader)
				m_pD3DDev->DeletePixelShader(m_dwFlyPixLightShader);
		}
		SAFE_RELEASE(m_pD3DDev);
	}
	
	return S_OK;
}

HRESULT CShaderFlying2::Start()
{
	return S_OK;
}

// Only for visualizing the ponit light position
HRESULT CShaderFlying2::DrawLight(D3DXVECTOR3 _light)
{
	HRESULT hr = S_OK;
	Fly2Vertex* pBuff;
	
	hr = m_pVertexBufferLight->Lock(0, sizeof(Fly2Vertex) * 4,(BYTE**)&pBuff, 0);
	if (FAILED(hr))
	{
		m_strLastError = "Couldn't lock buffer!";
		return hr;
	}

	const float light_size = 0.05f;

	// A Simple Quad
	pBuff->Position = _light;	
	pBuff->Texture = D3DXVECTOR2(0.0f, 0.0f);
	pBuff->RelativePosition = D3DXVECTOR2(-light_size, +light_size);
	pBuff++;

	pBuff->Position = _light;	
	pBuff->Texture = D3DXVECTOR2(1.0f, 0.0f);
	pBuff->RelativePosition = D3DXVECTOR2(+light_size, +light_size);
	pBuff++;

	pBuff->Position = _light;	
	pBuff->Texture = D3DXVECTOR2(0.0f, 1.0f);
	pBuff->RelativePosition = D3DXVECTOR2(-light_size, -light_size);
	pBuff ++;

	pBuff->Position = _light;	
	pBuff->Texture = D3DXVECTOR2(1.0f, 1.0f);
	pBuff->RelativePosition = D3DXVECTOR2(+light_size, -light_size);
	pBuff++;

	m_pVertexBufferLight->Unlock();

	// only draw texture
	m_pD3DDev->SetRenderState(D3DRS_ALPHAREF, (DWORD)127);
	m_pD3DDev->SetRenderState(D3DRS_ALPHATESTENABLE, TRUE); 
	m_pD3DDev->SetRenderState(D3DRS_ALPHAFUNC, D3DCMP_GREATER);
	m_pD3DDev->SetRenderState(D3DRS_ZFUNC, D3DCMP_LESSEQUAL); 
	m_pD3DDev->SetRenderState(D3DRS_ALPHABLENDENABLE, FALSE);
	m_pD3DDev->SetTextureStageState(0, D3DTSS_COLOROP , D3DTOP_SELECTARG1);
	m_pD3DDev->SetTextureStageState(0, D3DTSS_COLORARG1, D3DTA_TEXTURE ); 
	m_pD3DDev->SetTextureStageState(0, D3DTSS_ALPHAOP , D3DTOP_SELECTARG1);
	m_pD3DDev->SetTextureStageState(0, D3DTSS_ALPHAARG1, D3DTA_TEXTURE ); 

	hr = m_pD3DDev->SetPixelShader(NULL);
	if (FAILED(hr))
		return hr;
	
	hr = m_pD3DDev->SetStreamSource(0, m_pVertexBufferLight, sizeof(Fly2Vertex));
	if (FAILED(hr))
		return hr;

	// have no better one
	hr = m_pD3DDev->SetVertexShader(m_dwFlyVertLightShader);
	if (FAILED(hr))
		return hr;

	hr = m_pD3DDev->DrawPrimitive(D3DPT_TRIANGLESTRIP, 0, 2);

	return (hr);
}

HRESULT CShaderFlying2::Tick(EBTimer* pTimer)
{
	HRESULT hr = S_OK;
	D3DXMATRIX temp;
    D3DXMATRIX matView;
	D3DXMATRIX matProj;
	D3DXVECTOR3	light;
	D3DXVECTOR3	lightview;
	D3DXVECTOR3	temp_vec;

	float time = pTimer->GetDuration();

	hr = m_pD3DDev->Clear(0, NULL, D3DCLEAR_TARGET|D3DCLEAR_ZBUFFER, D3DCOLOR_XRGB( 0xAA, 0xAA, 0xAA ), 1.0, 0);

	m_pD3DDev->SetRenderState(D3DRS_FILLMODE, m_bWireframe ? D3DFILL_WIREFRAME : D3DFILL_SOLID);

	// get view matrix
	matView = m_pUI->GetRotationMatrix();
	m_pUI->GetTranslationMatrix( &temp );
	D3DXMatrixMultiply(&matView, &matView, &temp);

	// calculate light position
	D3DXMatrixRotationY(&temp, time*0.5);
	D3DXVec3TransformCoord(&light, &D3DXVECTOR3((float)(1.5 - cos(time*0.1)), 0.25f, 0.0f), &temp);

	// transform light into view
	D3DXVec3TransformCoord(&lightview, &light, &matView);
  

	/* Set shader constants*/

	D3DXMatrixTranspose(&matView, &matView);
	m_pD3DDev->SetVertexShaderConstant(0, &matView(0, 0), 4);

	D3DXMatrixPerspectiveFovLH(&matProj, D3DXToRadian(60.0f), 1.0f, 0.1f, 50.0f);
	D3DXMatrixTranspose(&matProj, &matProj);
	m_pD3DDev->SetVertexShaderConstant(4, &matProj(0, 0), 4);

	D3DXVECTOR4	zeroConstant(0.0f, 0.0f, 0.0f, 1.0f);
	m_pD3DDev->SetVertexShaderConstant(8, &zeroConstant, 1);
	
	D3DXVECTOR4	halfConstant(0.5f, 0.5f, 0.5f, 0.5f);
	m_pD3DDev->SetVertexShaderConstant(9, &halfConstant, 1);

	m_pD3DDev->SetVertexShaderConstant(10, &lightview, 1);

	// set ambient color
	if (m_bLighting==false)
		m_pD3DDev->SetPixelShaderConstant(0, &D3DXVECTOR4(1.0f, 1.0f, 1.0f, 1.0f), 1);
	else
		m_pD3DDev->SetPixelShaderConstant(0, &D3DXVECTOR4(0.15f, 0.15f, 0.15f, 0.15f), 1);


	/* Set render states*/

	m_pD3DDev->SetRenderState(D3DRS_CULLMODE, D3DCULL_NONE);

	m_pD3DDev->SetTexture(0, m_NormalLookUp);
	m_pD3DDev->SetTexture(3, m_EarthCubeMap);

	m_pD3DDev->SetTextureStageState(0, D3DTSS_MINFILTER, D3DTEXF_LINEAR);
    m_pD3DDev->SetTextureStageState(0, D3DTSS_MAGFILTER, D3DTEXF_LINEAR);
	m_pD3DDev->SetTextureStageState(0, D3DTSS_MIPFILTER, D3DTEXF_NONE);
	m_pD3DDev->SetTextureStageState(0, D3DTSS_ADDRESSU , D3DTADDRESS_CLAMP);
	m_pD3DDev->SetTextureStageState(0, D3DTSS_ADDRESSV , D3DTADDRESS_CLAMP);
	
	m_pD3DDev->SetTextureStageState(3, D3DTSS_MINFILTER, D3DTEXF_LINEAR);
    m_pD3DDev->SetTextureStageState(3, D3DTSS_MAGFILTER, D3DTEXF_LINEAR);
	m_pD3DDev->SetTextureStageState(3, D3DTSS_MIPFILTER, D3DTEXF_NONE);
	m_pD3DDev->SetTextureStageState(3, D3DTSS_ADDRESSU , D3DTADDRESS_CLAMP);
	m_pD3DDev->SetTextureStageState(3, D3DTSS_ADDRESSV , D3DTADDRESS_CLAMP);

	m_pD3DDev->SetRenderState(D3DRS_ALPHAREF, (DWORD)127);
	m_pD3DDev->SetRenderState(D3DRS_ALPHATESTENABLE, TRUE); 
	m_pD3DDev->SetRenderState(D3DRS_ALPHAFUNC, D3DCMP_GREATER);
	m_pD3DDev->SetRenderState(D3DRS_ZFUNC, D3DCMP_LESSEQUAL); 
	m_pD3DDev->SetRenderState(D3DRS_ALPHABLENDENABLE, FALSE);

	/* First pass */ 

	if (UsePixelShader)
	{
		hr = m_pD3DDev->SetPixelShader(m_dwFlyPixLightShader);
		if (FAILED(hr))
			return hr;
	}
	else
	{
		m_pD3DDev->SetTextureStageState(0, D3DTSS_COLOROP , D3DTOP_SELECTARG1);
		m_pD3DDev->SetTextureStageState(0, D3DTSS_COLORARG1, D3DTA_TEXTURE ); 
		m_pD3DDev->SetTextureStageState(0, D3DTSS_ALPHAOP , D3DTOP_SELECTARG1);
		m_pD3DDev->SetTextureStageState(0, D3DTSS_ALPHAARG1, D3DTA_TEXTURE ); 
	}

	hr = m_pD3DDev->SetVertexShader(m_dwFlyVertLightShader);
	if (FAILED(hr))
		return hr;

	hr = m_pD3DDev->SetStreamSource(0, m_pVertexBuffer, sizeof(Fly2Vertex));
	if (FAILED(hr))
		return hr;


	if (m_bManySpheres)
		hr = m_pD3DDev->DrawPrimitive(D3DPT_TRIANGLELIST, 0, 2 * NumOfSpheres);
	else
		hr = m_pD3DDev->DrawPrimitive(D3DPT_TRIANGLELIST, 0, 2);


	/* Second pass */ 

	if (m_bTexture && UsePixelShader)
	{
		m_pD3DDev->SetRenderState(D3DRS_ALPHATESTENABLE, FALSE); 
		m_pD3DDev->SetRenderState(D3DRS_ZFUNC, D3DCMP_EQUAL); 
		m_pD3DDev->SetRenderState(D3DRS_ALPHABLENDENABLE, TRUE);

		m_pD3DDev->SetRenderState(D3DRS_DESTBLEND, D3DBLEND_ZERO); 
		m_pD3DDev->SetRenderState(D3DRS_SRCBLEND, D3DBLEND_DESTCOLOR); 

		hr = m_pD3DDev->SetPixelShader(m_dwFlyPixCubeShader);
		if (FAILED(hr))
			return hr;

		hr = m_pD3DDev->SetVertexShader(m_dwFlyVertCubeShader);
		if (FAILED(hr))
			return hr;

		if (m_bManySpheres)
			hr = m_pD3DDev->DrawPrimitive(D3DPT_TRIANGLELIST, 0, 2 * NumOfSpheres);
		else
			hr = m_pD3DDev->DrawPrimitive(D3DPT_TRIANGLELIST, 0, 2);
	}

	return (DrawLight(light));
}


void CShaderFlying2::MouseButton(HWND hWnd, eButtonID button, bool bDown, int x, int y)
{
	if(button == MOUSE_LEFTBUTTON)
	{
		if(bDown)
		{
			m_pUI->OnLButtonDown(x, y);
		}
		else
		{
			m_pUI->OnLButtonUp(x, y);
		}
	}
	return;
}

void CShaderFlying2::MouseMove(HWND hWnd, int x, int y)
{
	m_pUI->OnMouseMove(x, y);
	return;
}

void CShaderFlying2::Keyboard(DWORD dwKey, UINT nFlags, bool bDown)
{
	eEBKeyAction Action = TranslateEffectKey(dwKey, nFlags, bDown);
	
    switch ( Action )
    {
		case EB_HELP:
		{
			::MessageBoxEx( NULL, " Help : F1 - Help \n\n W - Wireframe Toggle \n\n Left Button & Mouse - Rotate Object\n\n Shift Left Button & Mouse - Pan Camera \n\n Ctrl Left Button & Mouse - Move Camera In & Out\n\n M - Change One\\Many Spheres\n\n  L- Light Toggle\n\n  T - Texture Toggle\n\n",
				"Help", MB_ICONINFORMATION | MB_TASKMODAL, MAKELANGID( LANG_NEUTRAL, SUBLANG_DEFAULT ) );
		}
		break;

		case EB_WIREFRAME:
        {
            m_bWireframe = !m_bWireframe;
			m_dwEffectDirtyFlags |= EBEFFECT_DIRTY_PUBLICSTATE;
            
        }
		break;

		case EB_RESET:
        {
            m_pUI->Reset();
            m_bWireframe = false;
			m_dwEffectDirtyFlags |= EBEFFECT_DIRTY_PUBLICSTATE;
            
        }
		break;
	
        default :
            break;
    }
	
		if (bDown)
		{
			switch (dwKey)
			{
				case 'M':
				{
					m_bManySpheres = !m_bManySpheres;
					m_dwEffectDirtyFlags |= EBEFFECT_DIRTY_PUBLICSTATE;

					break;
				}
				case 'L':
				{
					m_bLighting = !m_bLighting;
					m_dwEffectDirtyFlags |= EBEFFECT_DIRTY_PUBLICSTATE;

					break;
				}
				case 'T':
				{
					m_bTexture = !m_bTexture;
					m_dwEffectDirtyFlags |= EBEFFECT_DIRTY_PUBLICSTATE;

					break;
				}


				break;
			}
		}
}
