/*********************************************************************NVMH2****
Path:  E:\devrel\NV_SDK_4\DX8\NVEffectsBrowser\Effects\shader_Kosmo
File:  shader_Kosmo.cpp

Copyright (C) 1999, 2000 NVIDIA Corporation
This file is provided without support, instruction, or implied warranty of any
kind.  NVIDIA makes no guarantee of its fitness for a particular purpose and is
not liable under any circumstances for any damages or loss whatsoever arising
from the use or inability to use this file or items derived from it.

Comments:


******************************************************************************/

#include "eb_effect.h"
#include "nvinc.h"
#include "shader_Kosmo.h"
#include "ComPtr.h"
#include "Constants.h"

using namespace std;

DECLARE_EFFECT_MAIN()

extern "C"
{
	
	__declspec(dllexport) unsigned int GetNumEffects() { return 1; }
	
	__declspec(dllexport) EBEffect* CreateEffect(unsigned int EffectNum)
	{	
		switch(EffectNum)
		{
		case 0:
			return new CShaderKosmo();
			break;
		default:
			return NULL;
		}
	}
	
}

// ------------------------------------------------------------------------------
// CShaderKosmo Constructor
//
// Description: Nothing to it -- just straight-forward construction
// ------------------------------------------------------------------------------ 
CShaderKosmo::CShaderKosmo()
: m_pVertexBuffer(NULL), 
  m_pIndexBuffer(NULL),
  m_pVertexBuffer2(NULL), 
  m_pIndexBuffer2(NULL),
  m_bWireframe(false),
  m_pUI(NULL),
  m_fRadius(10.0f),									// eye distance
  m_vecCenter(0.0f, 0.0f, 0.0f),
  m_dwNumParticles(300),								// for 1D particles
  m_dwNumVerts(0),
  m_dwNumFaces(0),
  m_eDisplayOption(KOSMO1_DISPLAY),
  m_dwParticle1D_SShader(0),
  m_dwParticle1D_BShader(0),
  m_dwNormalShader(0),
  m_fTestConstant1(0.0f),
  m_fTestConstant2(10.0f),
  m_iScene(5),
  m_iCurrentTexture(0),
  m_fParticleLength(0.15f)
{
	m_strEffectLocation = "3rd Party\\Particles";
	m_strEffectName = "1D Particles";
	m_strEffectVertexShader = GetFilePath("Particle1D_s.nvv");
	m_strEffectPixelShader = "";
	m_strEffectVersion = "1.0";
	
	D3DXMatrixIdentity(&m_world);
	D3DXMatrixIdentity(&m_view);
	D3DXMatrixIdentity(&m_proj);

	for (DWORD i = 0; i < NUM_SHADETEXTURES; i++)
	{
		m_pShadeTexture[i] = NULL;
	}

}

// ------------------------------------------------------------------------------
// CShaderKosmo Destructor
//
// Description: Nothing to it -- just straight-forward destruction
// ------------------------------------------------------------------------------ 
CShaderKosmo::~CShaderKosmo()
{
	Free(); 
}

// ------------------------------------------------------------------------------
// CShaderKosmo::UpdateProperties
//
// Description: This adds options to the menu (click on "File" or right-click
// on the display.
// ------------------------------------------------------------------------------ 
#define STR_INCREASE_TEXTURE "next texture (+)"
#define STR_DECREASE_TEXTURE "previous Texture (-)"
#define STR_INCREASE_SCENE "next Scene  (SPACE)"

void CShaderKosmo::UpdateProperties()
{
	EBEffect::UpdateProperties();

	AddProperty(new EBTriggerProperty(STR_INCREASE_TEXTURE));
	AddProperty(new EBTriggerProperty(STR_DECREASE_TEXTURE));
	AddProperty(new EBTriggerProperty(STR_INCREASE_SCENE));

	EBEnumProperty* pEnumProp = new EBEnumProperty("Display Options", OBJECT_MEMBER(m_eDisplayOption), EBTYPE_DWORD_PROP);
	
	pEnumProp->AddEnumerant(new EBEnumValue(pEnumProp, "Kosmo1 Rendering", (DWORD)KOSMO1_DISPLAY, EBTYPE_DWORD_PROP));
	pEnumProp->AddEnumerant(new EBEnumValue(pEnumProp, "Kosmo2 Rendering", (DWORD)KOSMO2_DISPLAY, EBTYPE_DWORD_PROP));
	AddProperty(pEnumProp);
	
	AddProperty(new EBProperty("Wireframe", OBJECT_MEMBER(m_bWireframe), EBTYPE_BOOL_PROP));

	// Vertex shaders
	m_pVertexShaderEnum->AddEnumerant(new EBEnumValue(m_pVertexShaderEnum, "Particle 1D standard", GetFilePath("Particle1D_s.nvv"), EBTYPE_STRING_PROP));
	m_pVertexShaderEnum->AddEnumerant(new EBEnumValue(m_pVertexShaderEnum, "Particle 1D blended", GetFilePath("Particle1D_b.nvv"), EBTYPE_STRING_PROP));
//	m_pVertexShaderEnum->AddEnumerant(new EBEnumValue(m_pVertexShaderEnum, "Normal TnL", GetFilePath("NormalTransform.nvv"), EBTYPE_STRING_PROP));

    string aboutText = "file://";
    aboutText += GetFilePath("..\\..\\..\\NVEffectsExplained.htm");
    aboutText += "#particle1d";
	SetAboutInfo( NULL, _T("1D Particles"), _T(aboutText.c_str()));

}


// ------------------------------------------------------------------------------
// CShaderKosmo::ConfirmDevice
//
// Description: Performs caps-bit checking to make sure the selected device 
//		supports this demo.  In this particular case we check for nothing!
// ------------------------------------------------------------------------------ 
HRESULT CShaderKosmo::ConfirmDevice(D3DCAPS8* pCaps, DWORD dwBehavior, D3DFORMAT Format)
{
	return S_OK;
}




void PushLine(	PSystemVertex* &pBuff, 
				D3DXVECTOR3 inFromPos, D3DXVECTOR3 inToPos, 
				float inFromSize, float inToSize )
{
	D3DXVECTOR3 v=inFromPos-inToPos;
	float invlen=1.0/(sqrt(v.x*v.x+v.y*v.y+v.z*v.z)+inFromSize+inToSize);

	pBuff->input_v0 = inFromPos;
	pBuff->input_v1 = inToPos;
	pBuff->input_v2 = D3DXVECTOR4(inFromSize,-inFromSize,0,0);
    pBuff->input_v3 = D3DXVECTOR3(inFromSize,inToSize,invlen);
	++pBuff;
	pBuff->input_v0 = inToPos;
	pBuff->input_v1 = inFromPos;
	pBuff->input_v2 = D3DXVECTOR4(inToSize,inToSize,0.25f,0);
    pBuff->input_v3 = D3DXVECTOR3(inToSize,inFromSize,invlen);
	++pBuff;
	pBuff->input_v0 = inToPos;
	pBuff->input_v1 = inFromPos;
	pBuff->input_v2 = D3DXVECTOR4(inToSize,-inToSize,0.25f,0.25f);
    pBuff->input_v3 = D3DXVECTOR3(inToSize,inFromSize,invlen);
	++pBuff;
	pBuff->input_v0 = inFromPos;
	pBuff->input_v1 = inToPos;
	pBuff->input_v2 = D3DXVECTOR4(inFromSize,inFromSize,0,0.25f);
    pBuff->input_v3 = D3DXVECTOR3(inFromSize,inToSize,invlen);
	++pBuff;
}


HRESULT CShaderKosmo::FillVertexBuffer( void )
{
	HRESULT hr;

	





	PSystemVertex* pBuff;

    assert(m_pVertexBuffer != NULL);
	hr = m_pVertexBuffer->Lock(0, m_dwNumParticles *4* sizeof(PSystemVertex),(BYTE**)&pBuff, 0);
	if (FAILED(hr))
	{
		m_strLastError = "Couldn't lock buffer!";
		return hr;
	}

    float const     kRandMax  = static_cast<float>(RAND_MAX);

	int i=0;

	

	 // generate particle values
	switch(m_iScene)
	{
		case 0:
			{
				float z=0.0f;
				float k=6.0f;
				while(m_dwNumParticles-i>=12)
				{
					PushLine(pBuff,D3DXVECTOR3(1,1,z),D3DXVECTOR3(-1,1,z),0.9f,0.9f);
					PushLine(pBuff,D3DXVECTOR3(-1,1,z),D3DXVECTOR3(-1,-1,z),0.9f,0.9f);
					PushLine(pBuff,D3DXVECTOR3(-1,-1,z),D3DXVECTOR3(1,-1,z),0.9f,0.9f);
					PushLine(pBuff,D3DXVECTOR3(1,-1,z),D3DXVECTOR3(1,1,z),0.9f,0.9f);

					PushLine(pBuff,D3DXVECTOR3(k+1*0.3f,1*0.3f,z),D3DXVECTOR3(k-1*0.3f,1*0.3f,z),0.9f*0.3f,0.9f*0.3f);
					PushLine(pBuff,D3DXVECTOR3(k-1*0.3f,1*0.3f,z),D3DXVECTOR3(k-1*0.3f,-1*0.3f,z),0.9f*0.3f,0.9f*0.3f);
					PushLine(pBuff,D3DXVECTOR3(k-1*0.3f,-1*0.3f,z),D3DXVECTOR3(k+1*0.3f,-1*0.3f,z),0.9f*0.3f,0.9f*0.3f);
					PushLine(pBuff,D3DXVECTOR3(k+1*0.3f,-1*0.3f,z),D3DXVECTOR3(k+1*0.3f,1*0.3f,z),0.9f*0.3f,0.9f*0.3f);

					PushLine(pBuff,D3DXVECTOR3(-k+1,1,z),D3DXVECTOR3(-k-1,1,z),0.9f,0.9f);
					PushLine(pBuff,D3DXVECTOR3(-k-1,1,z),D3DXVECTOR3(-k-1,-1,z),0.9f,0.9f);
					PushLine(pBuff,D3DXVECTOR3(-k-1,-1,z),D3DXVECTOR3(-k+1,-1,z),0.9f,0.9f);
					PushLine(pBuff,D3DXVECTOR3(-k+1,-1,z),D3DXVECTOR3(-k+1,1,z),0.9f,0.9f);

					i+=12;z+=2.0f;
				}
			}
			break;

		case 1:
			{
				for(;i<m_dwNumParticles;i++)
				{
					D3DXVECTOR3 pos0,pos1;

					pos0=D3DXVECTOR3(static_cast<float>(rand())/kRandMax,static_cast<float>(rand())/kRandMax,static_cast<float>(rand())/kRandMax);
					pos1=D3DXVECTOR3(static_cast<float>(rand())/kRandMax,static_cast<float>(rand())/kRandMax,static_cast<float>(rand())/kRandMax);

					pos0=pos0*8.0f-D3DXVECTOR3(4.0f,4.0f,4.0f);

					pos1=pos0+pos1*0.5f-D3DXVECTOR3(0.25f,0.25f,0.25f);

					PushLine(pBuff,pos0,pos1,0.9f,0.9f);
				}
			}
			break;

		case 2:
			{
				D3DXVECTOR3 pos0(0,0,0);	
				float f=0.0f;

				for(;i<m_dwNumParticles;f+=0.2f)
				{
					D3DXVECTOR3 pos1;

					pos1=D3DXVECTOR3(sin(f)*4.0f,cos(f)*4.0f,f*0.7f);

					if(f!=0.0f)
					{
						PushLine(pBuff,pos0,pos1,0.9f,0.9f);

						i++;
					}

					pos0=pos1;
				}
			}
			break;

		case 3:
			{
				D3DXVECTOR3 pos0(0,0,0);	
				float f=0.0f;

				for(;i<m_dwNumParticles;f+=0.1f)
				{
					D3DXVECTOR3 pos1;

					pos1=2.0f*D3DXVECTOR3(sin(f)*cos(f*0.5f)*4.0f,
						             sin(f*1.7f)*cos(f)*4.0f,sin(f*1.8f)*cos(f*0.7f)*3.7f);

					if(f!=0.0f)
					{
						PushLine(pBuff,pos0,pos1,0.9f,0.9f);

						i++;
					}

					pos0=pos1;
				}
			}
			break;
		case 4:
			{
				D3DXVECTOR3 pos0(0,0,0);	
				float f=0.0f;
				float size0=0.0f,size1;

				for(;i<m_dwNumParticles;f+=0.1f)
				{
					D3DXVECTOR3 pos1;

					pos1=2.0f*D3DXVECTOR3(sin(f)*cos(f*0.5f)*4.0f,
						             sin(f*1.7f)*cos(f)*4.0f,sin(f*1.8f)*cos(f*0.7f)*3.7f);

					size1=1.4f*(1+sin(f*1.5f+0.3)*sin(f*2.7f));

					if(f!=0.0f)
					{
						PushLine(pBuff,pos0,pos1,size0,size1);

						i++;
					}

					pos0=pos1;size0=size1;
				}
			}
			break;
		case 5:
			{
				D3DXVECTOR3 pos0(0,0,0);	
				float f=0.004f*timeGetTime();
				float g=0.003f*timeGetTime();
				float size0=-1.0f,size1;

				for(;i<m_dwNumParticles;f+=0.1f,g+=0.06f)
				{
					D3DXVECTOR3 pos1;

					pos1=2.0f*D3DXVECTOR3(sin(f)*cos(g*0.5f)*4.0f,
						             sin(f*1.27f)*cos(g)*4.0f,sin(f*1.8f)*cos(g*2.7f)*3.7f);

					size1=1.4f*(1+sin(f*1.5f+0.3)*sin(g*2.7f));

					if(size0!=-1.0)
					{
						PushLine(pBuff,pos0,pos1,size0,size1);

						i++;
					}

					pos0=pos1;size0=size1;
				}
			}
			break;
		case 6:
			{
				D3DXVECTOR3 pos0(0,0,0);	
				float f=0.0f;
				float g=0.002f*timeGetTime();
				float size0=0.0f,size1;

				for(;i<m_dwNumParticles;f+=0.1f,g+=0.06f)
				{
					D3DXVECTOR3 pos1;

					pos1=2.0f*D3DXVECTOR3(sin(f)*cos(f*1.5f)*4.0f,
						             sin(f*2.7f)*cos(f)*4.0f,sin(f*1.4f)*cos(f*0.7f)*3.7f);

					size1=1.4f*(1+sin(g*1.5f+0.3)*sin(g*2.7f));

					if(f!=0.0f)
					{
						PushLine(pBuff,pos0,pos1,size0,size1);

						i++;
					}

					pos0=pos1;size0=size1;
				}
			}
			break;
		case 7:
			{
				D3DXVECTOR3 pos0(0,0,0);	
				float f=0.0f;
				float g=0.002f*timeGetTime();
				float size0=0.0f,size1;

				for(;i<m_dwNumParticles;f+=0.1f,g+=0.06f)
				{
					D3DXVECTOR3 pos1;

					pos1=2.0f*D3DXVECTOR3(sin(f)*cos(f*0.5f)*4.0f,
						             sin(f*1.7f)*cos(f)*4.0f,sin(f*1.8f)*cos(f*0.7f)*3.7f);

					size1=1.4f*(1+sin(g*1.5f+0.3)*sin(g*2.7f));

					if(f!=0.0f)
					{
						PushLine(pBuff,pos0,pos1,size0,size1);

						i++;
					}

//					pos0=pos1;
					size0=size1;
				}
			}
			break;
		case 8:
			{
				D3DXVECTOR3 pos0(0,0,0);	
				float gg=0.0005f*timeGetTime();
				float size1;

				for(;i<m_dwNumParticles;gg+=0.08f)
				{
					float size0=-1.0f;

					float g=sin(i)*10.0f+sin(i*2.4f)*40.0f+gg;

					for(float h=0;h<1.0f;h+=0.1f)
					{
						D3DXVECTOR3 pos1;

						pos1=(h+0.2f)*1.0f*D3DXVECTOR3(
										sin(g)*cos(h*0.5f)*4.0f+cos(gg+h*8.0f)*0.5f,
										sin(g*1.7f)*cos(h)*4.0f+sin(gg+h*10.0f)*0.5f,
										sin(h*1.8f)*cos(g*0.7f)*3.7f);

						size1=0.3f*(1+sin(g*1.5f+0.3)*sin(g*2.7f));

						if(size0!=-1.0f)
						{
							PushLine(pBuff,pos0,pos1,size0,size1);

							i++;if(i>=m_dwNumParticles)break;
						}

						pos0=pos1;
						size0=size1;
					}
				}
			}
			break;
	}

	m_pVertexBuffer->Unlock();

	return(S_OK);
}




// ------------------------------------------------------------------------------
// CShaderKosmo::Initialize
//
// Description: Initialize render-states, vertex shader and vertex buffers
// ------------------------------------------------------------------------------ 
HRESULT CShaderKosmo::Initialize(IDirect3DDevice8* pDev)
{
	HRESULT hr;
	
	// get the D3D device 
	m_pD3DDev = pDev;
	pDev->AddRef();
	
	//initialize mouse UI
	RECT rect;
	rect.left = rect.top = 0;
	D3DVIEWPORT8 viewport;
	m_pD3DDev->GetViewport(&viewport);
	rect.bottom = viewport.Height;
	rect.right  = viewport.Width;
	m_pUI = new MouseUI((const RECT)rect);
	
	vector<DWORD> Declaration;
	Declaration.push_back(D3DVSD_STREAM(0));					// PSystemVertex
	Declaration.push_back(D3DVSD_REG(0, D3DVSDT_FLOAT3));
	Declaration.push_back(D3DVSD_REG(1, D3DVSDT_FLOAT3));
	Declaration.push_back(D3DVSD_REG(2, D3DVSDT_FLOAT4));
	Declaration.push_back(D3DVSD_REG(3, D3DVSDT_FLOAT3));
	Declaration.push_back(D3DVSD_END());

	vector<DWORD> NormalDeclaration;
	NormalDeclaration.push_back(D3DVSD_STREAM(0));					// HVertex
	NormalDeclaration.push_back(D3DVSD_REG(0, D3DVSDT_FLOAT3));
	NormalDeclaration.push_back(D3DVSD_REG(1, D3DVSDT_FLOAT3));
	NormalDeclaration.push_back(D3DVSD_END());

	// Load the two vertex shaders
	m_dwParticle1D_SShader = m_dwParticle1D_BShader = m_dwNormalShader = 0;

	hr = LoadAndCreateShader(GetFilePath("NormalTransform.vso"), &NormalDeclaration[0], 0, SHADERTYPE_VERTEX, &m_dwNormalShader);
	if (FAILED(hr))
		return hr;


	hr = LoadAndCreateShader(GetFilePath("Particle1D_s.vso"), &Declaration[0], 0, SHADERTYPE_VERTEX, &m_dwParticle1D_SShader);
	if (FAILED(hr))
		return hr;

	hr = LoadAndCreateShader(GetFilePath("Particle1D_b.vso"), &Declaration[0], 0, SHADERTYPE_VERTEX, &m_dwParticle1D_BShader);
	if (FAILED(hr))
		return hr;
	
	// select the just created vertex shader as the thing to transform and light vertices
	hr = m_pD3DDev->SetVertexShader(m_dwParticle1D_SShader);
	if (FAILED(hr))
		return hr;

	DWORD tempFVF = D3DFVF_XYZ | D3DFVF_NORMAL;
	hr = LoadXFile(GetFilePath("teapot.x").c_str(), tempFVF);
	if(FAILED(hr))
		return hr;



  // create the particles
    hr = m_pD3DDev->CreateVertexBuffer( m_dwNumParticles *4* sizeof(PSystemVertex),
										D3DUSAGE_POINTS | D3DUSAGE_WRITEONLY, 0, D3DPOOL_MANAGED, &m_pVertexBuffer);
	if (FAILED(hr))
		return hr;


	FillVertexBuffer();


	hr = m_pD3DDev->CreateIndexBuffer(m_dwNumParticles *6* sizeof(WORD), D3DUSAGE_WRITEONLY, D3DFMT_INDEX16, D3DPOOL_MANAGED, &m_pIndexBuffer);
	if (FAILED(hr))
	{
		m_strLastError = "Could not create index buffer!";
		return hr;
	}
	
	WORD *pIndices;

	hr = m_pIndexBuffer->Lock(0, m_dwNumParticles *6* sizeof(WORD),(BYTE**)&pIndices, 0);
	if (FAILED(hr))
	{
		m_strLastError = "Could not lock vertex buffer!";
		return hr;
	}

	// 2 triangles make one 1D particle
    for(int i=0;i<m_dwNumParticles;i++)
    {
		*pIndices++=i*4+0;
		*pIndices++=i*4+1;
		*pIndices++=i*4+2;

   		*pIndices++=i*4+2;
		*pIndices++=i*4+3;
		*pIndices++=i*4+0;
	}


	m_pIndexBuffer->Unlock();















	// use this vertex buffer as our source for vertices
	hr = m_pD3DDev->SetStreamSource(0, m_pVertexBuffer, sizeof(PSystemVertex));
	if (FAILED(hr))
		return hr;
	
	//set index buffer
	m_pD3DDev->SetIndices(m_pIndexBuffer, 0);
	
	//set render states
//	m_pD3DDev->SetRenderState(D3DRS_ZENABLE, D3DZB_TRUE);
	m_pD3DDev->SetRenderState(D3DRS_ALPHABLENDENABLE, FALSE);
	m_pD3DDev->SetRenderState(D3DRS_LIGHTING, FALSE);
	m_pD3DDev->SetRenderState(D3DRS_CULLMODE, D3DCULL_NONE);

//	m_pD3DDev->SetRenderState(D3DRS_SRCBLEND, D3DBLEND_SRCALPHA);
//	m_pD3DDev->SetRenderState(D3DRS_DESTBLEND, D3DBLEND_INVSRCALPHA);

	m_pD3DDev->SetRenderState(D3DRS_DESTBLEND,D3DBLEND_ONE);
	m_pD3DDev->SetRenderState(D3DRS_SRCBLEND,D3DBLEND_ONE);
	m_pD3DDev->SetRenderState(D3DRS_ZWRITEENABLE,FALSE);
	m_pD3DDev->SetRenderState(D3DRS_ZENABLE,			0);


	D3DXCreateTextureFromFile(m_pD3DDev, GetFilePath("1D_inner.bmp").c_str(), &m_pShadeTexture[0]);
	D3DXCreateTextureFromFile(m_pD3DDev, GetFilePath("1D_inner2.bmp").c_str(), &m_pShadeTexture[1]);
	D3DXCreateTextureFromFile(m_pD3DDev, GetFilePath("1D_perfectadd3.bmp").c_str(), &m_pShadeTexture[2]);
	D3DXCreateTextureFromFile(m_pD3DDev, GetFilePath("1D_rauch2.bmp").c_str(), &m_pShadeTexture[3]);
	D3DXCreateTextureFromFile(m_pD3DDev, GetFilePath("1D_ring2.bmp").c_str(), &m_pShadeTexture[4]);
	D3DXCreateTextureFromFile(m_pD3DDev, GetFilePath("1D_spiral.bmp").c_str(), &m_pShadeTexture[5]);
	D3DXCreateTextureFromFile(m_pD3DDev, GetFilePath("1D_xenon.bmp").c_str(), &m_pShadeTexture[6]);
	D3DXCreateTextureFromFile(m_pD3DDev, GetFilePath("1D_glow1.bmp").c_str(), &m_pShadeTexture[7]);
	D3DXCreateTextureFromFile(m_pD3DDev, GetFilePath("1D_debug.bmp").c_str(), &m_pShadeTexture[8]);
	
//	m_pD3DDev->SetTexture(0, m_pShadeTexture);

	m_pD3DDev->SetTextureStageState(0, D3DTSS_TEXCOORDINDEX, 0);
	
	m_pD3DDev->SetTextureStageState(0, D3DTSS_ADDRESSU, D3DTADDRESS_CLAMP);
	m_pD3DDev->SetTextureStageState(0, D3DTSS_MINFILTER, D3DTEXF_LINEAR);
	m_pD3DDev->SetTextureStageState(0, D3DTSS_MAGFILTER, D3DTEXF_LINEAR);
	m_pD3DDev->SetTextureStageState(0, D3DTSS_MIPFILTER, D3DTEXF_NONE);

	m_pD3DDev->SetTextureStageState(0, D3DTSS_COLORARG1, D3DTA_DIFFUSE);
	m_pD3DDev->SetTextureStageState(0, D3DTSS_COLOROP, D3DTOP_SELECTARG1);
	m_pD3DDev->SetTextureStageState(0, D3DTSS_COLORARG2, D3DTA_TEXTURE);
	
	D3DXVECTOR4 lightDir(1.0f, 0.0f, -1.0f, 0.0f);
	D3DXVec4Normalize(&lightDir, &lightDir);

	m_pD3DDev->SetVertexShaderConstant(CV_LIGHT_DIRECTION, &lightDir, 1);


	{	
		D3DXVECTOR4 constNums(0.5f, 0.25f, 0.125f, 0.0625f);
		m_pD3DDev->SetVertexShaderConstant(CV_BINARYCONSTANTS, &constNums, 1);
	}

	{
		D3DXVECTOR4 constNums(1.0f, 2.0f, 4.0f, 64.0f);
		m_pD3DDev->SetVertexShaderConstant(CV_BINARYCONSTANTS2, &constNums, 1);
	}

//	D3DXVECTOR4 perturb(1.0f, 0.0f, -1.0f, 0.0f);
//	m_pD3DDev->SetVertexShaderConstant(30, &perturb, 1);
	
//	D3DXVECTOR4 constNums(0.0f, 0.2f, 1.0f, 30.0f);
//	m_pD3DDev->SetVertexShaderConstant(CV_CONSTANTS, &constNums, 1);
	
	//set view matrix depending on the bounding sphere
	D3DXVECTOR3 eye, lookAt, up;
	
	eye.x    = m_vecCenter.x; eye.y    = m_vecCenter.y; eye.z    = -m_fRadius*2;
	lookAt.x = m_vecCenter.x; lookAt.y = m_vecCenter.y; lookAt.z = m_vecCenter.z;
	up.x	 = 0.0f;          up.y     = 1.0f;          up.z     = 0.0f;
	
	m_pD3DDev->SetVertexShaderConstant(CV_EYE, &eye, 1);
	
	D3DXMatrixIdentity(&m_world);
	D3DXMatrixLookAtLH(&m_view, &eye, &lookAt, &up);
	
	D3DXMatrixPerspectiveFovLH(&m_proj,
		D3DXToRadian(60.0f),
		1,
		1.0f,
		100.0f);
	m_pD3DDev->SetTransform(D3DTS_PROJECTION, (D3DMATRIX*)&m_proj);
	
	//matrices
	SetVertexShaderMatrices();

	return S_OK;
}


HRESULT CShaderKosmo::SetVertexShaderMatrices()
{
	D3DXMATRIX worldViewProjMat, worldITMat, worldMat;
	D3DXMATRIX tempMat;
	
	D3DXMatrixMultiply(&tempMat, &m_pUI->GetRotationMatrix(), &m_pUI->GetTranslationMatrix());
	D3DXMatrixTranspose(&worldMat, &tempMat);
	
	D3DXMatrixIdentity(&worldViewProjMat);
	
	D3DXMatrixMultiply(&worldViewProjMat, &tempMat, &m_view);
	
	D3DXMatrixInverse(&worldITMat, NULL, &tempMat);
	
	D3DXMatrixMultiply(&worldViewProjMat, &worldViewProjMat, &m_proj);
	D3DXMatrixTranspose(&worldViewProjMat, &worldViewProjMat);
	
	m_pD3DDev->SetVertexShaderConstant(CV_WORLDVIEWPROJ_0, &worldViewProjMat, 4);
	
	m_pD3DDev->SetVertexShaderConstant(CV_WORLD_IT_0, &worldITMat, 4);
	m_pD3DDev->SetVertexShaderConstant(CV_WORLD_0, &worldMat, 4);
	



	D3DXMATRIX matWorldI;

	D3DXMatrixInverse(&matWorldI, NULL, &worldMat);
	


	D3DXVECTOR3 eye;
//	D3DXVECTOR4  eyeObjSpc;	   // eye and light positions in object space

	// Transform light and cam pos to object space and set vshader constants

	m_pD3DDev->GetVertexShaderConstant(CV_EYE, &eye, 1);
//	D3DXVec3Transform( &eyeObjSpc, &eye, &IworldMat );

	// Set vshader constants

	// made this a vector 4 so, we wouldn't have to set the w part in the shader
	D3DXVECTOR4 eyePos;
	// put eye into object space.
	D3DXMatrixTranspose(&matWorldI, &matWorldI);
	D3DXVec3TransformCoord((D3DXVECTOR3*)&eyePos, &eye, &matWorldI);
	eyePos.w = 1.0f;
	m_pD3DDev->SetVertexShaderConstant( CV_EYE_POS_OSPACE,   (void*)&eyePos.x,   1 );








	return S_OK;
}
// ------------------------------------------------------------------------------
// CShaderKosmo::Free
//
// Description: Called when we switch away from this demo.	Free up all the 
//		memory that is in use.
// ------------------------------------------------------------------------------ 
HRESULT CShaderKosmo::Free()
{
	SAFE_RELEASE(m_pVertexBuffer);
	SAFE_RELEASE(m_pIndexBuffer);
	
	SAFE_RELEASE(m_pVertexBuffer2);
	SAFE_RELEASE(m_pIndexBuffer2);

	for (DWORD i = 0; i < NUM_SHADETEXTURES; i++)
	{
		SAFE_RELEASE(m_pShadeTexture[i]);
	}
	
	if (m_pD3DDev)
	{
		m_pD3DDev->DeleteVertexShader(m_dwNormalShader);
		m_pD3DDev->DeleteVertexShader(m_dwParticle1D_BShader);
		m_pD3DDev->DeleteVertexShader(m_dwParticle1D_SShader);
		SAFE_RELEASE(m_pD3DDev);
	}

	SAFE_DELETE(m_pUI);
	
	return S_OK;
}


HRESULT CShaderKosmo::LoadXFile(const char* fileName, const DWORD dwFVF)
{
	ID3DXMesh* tempMesh, *tempMeshFVF;
	ID3DXBuffer* tempMaterials;
	
	HRESULT hr = D3DXLoadMeshFromX(const_cast<char*>(fileName), D3DXMESH_VB_SYSTEMMEM, m_pD3DDev, NULL, 
		&tempMaterials, &m_dwNumSections, &tempMesh);

	//comptr to ensure this object gets released
	ComPtr< ID3DXBuffer* > tempMaterialBuffer( tempMaterials, COMPTR_DONT_ADD_REF );
	ComPtr< ID3DXMesh* > tempMesh2( tempMesh, COMPTR_DONT_ADD_REF  );

	if(FAILED(hr))
		return hr;

/*	//compute bounding sphere
	HVertex* pBuff;
	hr = tempMesh->LockVertexBuffer(D3DLOCK_READONLY, (BYTE**)&pBuff);
	if(FAILED(hr))
		return hr;
	D3DXComputeBoundingSphere(pBuff, tempMesh->GetNumVertices(), tempMesh->GetFVF(), &m_vecCenter, &m_fRadius);
	tempMesh->UnlockVertexBuffer();
*/	
	//convert to our format, and also make writeonly
	hr = tempMesh->CloneMeshFVF(D3DXMESH_WRITEONLY, dwFVF, m_pD3DDev, &tempMeshFVF);
	if(FAILED(hr))
		return hr;

	//comptr to ensure this object gets released
	ComPtr< ID3DXMesh* > tempMesh4( tempMeshFVF, COMPTR_DONT_ADD_REF  );
	
	tempMeshFVF->GetVertexBuffer(&m_pVertexBuffer2);
	tempMeshFVF->GetIndexBuffer(&m_pIndexBuffer2);
	m_dwNumVerts = tempMeshFVF->GetNumVertices();
	m_dwNumFaces = tempMeshFVF->GetNumFaces();
	
	return S_OK;
}


// ------------------------------------------------------------------------------
// CShaderKosmo::Start
//
// Description: Called to reset
// ------------------------------------------------------------------------------ 
HRESULT CShaderKosmo::Start()
{
	return S_OK;
}


// ------------------------------------------------------------------------------
// CShaderKosmo::Tick
//
// Description: This function is called every frame.  Here is where we write
//		and update constants, clear the z- and back-buffer and render our 
//		primitives
// ------------------------------------------------------------------------------ 
HRESULT CShaderKosmo::Tick(EBTimer* pTimer)
{
	if(m_iScene>=5)
		FillVertexBuffer();



	HRESULT hr = S_OK;

	SetVertexShaderMatrices();

 /*
	float value = sin(D3DXToRadian(m_fAngle));
	value = fabs(value);
	m_fAngle += 2.0f;
	if(m_fAngle > 180.0f)
		m_fAngle = 0.0f;
// */


	m_pD3DDev->SetRenderState(D3DRS_FILLMODE, (m_bWireframe ? D3DFILL_WIREFRAME : D3DFILL_SOLID));

	D3DXVECTOR4 constNums(0.5f, 0.25f, 0.125f, 0.0625f);
	D3DXVECTOR4 const2Nums=constNums*m_fParticleLength;
	m_pD3DDev->SetVertexShaderConstant(CV_LENGTHCONSTANTS, &const2Nums, 1);


	D3DXVECTOR4 const8Nums(m_fTestConstant1,m_fTestConstant2,0,0);
	m_pD3DDev->SetVertexShaderConstant(CV_TESTCONSTANTS, &const8Nums, 1);

	if(m_eDisplayOption == KOSMO1_DISPLAY)
	{
		m_pD3DDev->Clear(0, NULL, D3DCLEAR_TARGET | D3DCLEAR_ZBUFFER, 0x000000, 1.0f, 0L);

		// use this vertex buffer as our source for vertices
		hr = m_pD3DDev->SetStreamSource(0, m_pVertexBuffer2, sizeof(HVertex));
		if (FAILED(hr))
			return hr;
		//set index buffer
		m_pD3DDev->SetIndices(m_pIndexBuffer2, 0);


		//pass one
		m_pD3DDev->SetRenderState(D3DRS_ZENABLE, D3DZB_TRUE);
		m_pD3DDev->SetRenderState(D3DRS_ALPHABLENDENABLE, FALSE);
		m_pD3DDev->SetRenderState(D3DRS_LIGHTING, FALSE);
		m_pD3DDev->SetRenderState(D3DRS_CULLMODE, D3DCULL_NONE);

		m_pD3DDev->SetRenderState(D3DRS_SRCBLEND, D3DBLEND_SRCALPHA);
		m_pD3DDev->SetRenderState(D3DRS_DESTBLEND, D3DBLEND_INVSRCALPHA);

		m_pD3DDev->SetRenderState(D3DRS_ZWRITEENABLE,TRUE);



		m_pD3DDev->SetTextureStageState(0, D3DTSS_COLOROP, D3DTOP_SELECTARG1);
		m_pD3DDev->SetVertexShader(m_dwNormalShader);
		m_pD3DDev->SetRenderState(D3DRS_ALPHABLENDENABLE, FALSE);
		m_pD3DDev->SetRenderState(D3DRS_ZWRITEENABLE, TRUE);
		m_pD3DDev->DrawIndexedPrimitive(D3DPT_TRIANGLELIST, 0, m_dwNumVerts, 0, m_dwNumFaces);

		// use this vertex buffer as our source for vertices
		hr = m_pD3DDev->SetStreamSource(0, m_pVertexBuffer, sizeof(PSystemVertex));
		if (FAILED(hr))
			return hr;
		//set index buffer
		m_pD3DDev->SetIndices(m_pIndexBuffer, 0);

		//pass two
		m_pD3DDev->SetRenderState(D3DRS_ALPHABLENDENABLE, FALSE);
		m_pD3DDev->SetRenderState(D3DRS_LIGHTING, FALSE);
		m_pD3DDev->SetRenderState(D3DRS_CULLMODE, D3DCULL_NONE);

		m_pD3DDev->SetRenderState(D3DRS_DESTBLEND,D3DBLEND_ONE);
		m_pD3DDev->SetRenderState(D3DRS_SRCBLEND,D3DBLEND_ONE);
		m_pD3DDev->SetRenderState(D3DRS_ZWRITEENABLE,FALSE);
		m_pD3DDev->SetRenderState(D3DRS_ZENABLE,			TRUE);

		//arg1 is the texture
		m_pD3DDev->SetTexture(0, m_pShadeTexture[m_iCurrentTexture]);
//		m_pD3DDev->SetTextureStageState(0, D3DTSS_COLOROP, D3DTOP_SELECTARG1);
		m_pD3DDev->SetTextureStageState(0, D3DTSS_COLOROP, D3DTOP_SELECTARG2);

		m_pD3DDev->SetRenderState(D3DRS_ALPHABLENDENABLE, TRUE);
		m_pD3DDev->SetRenderState(D3DRS_ZWRITEENABLE, FALSE);

		m_pD3DDev->SetVertexShader(m_dwParticle1D_SShader);
//		D3DXVECTOR4 color(0.9f, 0.9f, 0.0f, 0.0f);
//		m_pD3DDev->SetVertexShaderConstant(CV_COLOR, &color, 1);
		D3DXVECTOR4 const2Nums((sin(0.004f*timeGetTime())+1.0f)*8.0f,0,0,0);
		m_pD3DDev->SetVertexShaderConstant(CV_DATACONSTANTS, &const2Nums, 1);
		
		m_pD3DDev->DrawIndexedPrimitive(D3DPT_TRIANGLELIST, 0, m_dwNumParticles*4, 0, m_dwNumParticles*2);

		//pass two
//		m_pD3DDev->SetVertexShader(m_dwNormalShader);
//		m_pD3DDev->SetRenderState(D3DRS_ALPHABLENDENABLE, FALSE);
//		m_pD3DDev->SetRenderState(D3DRS_ZWRITEENABLE, TRUE);
//		m_pD3DDev->DrawIndexedPrimitive(D3DPT_TRIANGLELIST, 0, m_dwNumParticles*4, 0, m_dwNumParticles*2);
	}
	else
	{
		m_pD3DDev->Clear(0, NULL, D3DCLEAR_TARGET | D3DCLEAR_ZBUFFER, 0x000000, 1.0f, 0L);

		// use this vertex buffer as our source for vertices
		hr = m_pD3DDev->SetStreamSource(0, m_pVertexBuffer2, sizeof(HVertex));
		if (FAILED(hr))
			return hr;
		//set index buffer
		m_pD3DDev->SetIndices(m_pIndexBuffer2, 0);


		//pass one
		m_pD3DDev->SetRenderState(D3DRS_ZENABLE, D3DZB_TRUE);
		m_pD3DDev->SetRenderState(D3DRS_ALPHABLENDENABLE, FALSE);
		m_pD3DDev->SetRenderState(D3DRS_LIGHTING, FALSE);
		m_pD3DDev->SetRenderState(D3DRS_CULLMODE, D3DCULL_NONE);

		m_pD3DDev->SetRenderState(D3DRS_SRCBLEND, D3DBLEND_SRCALPHA);
		m_pD3DDev->SetRenderState(D3DRS_DESTBLEND, D3DBLEND_INVSRCALPHA);

		m_pD3DDev->SetRenderState(D3DRS_ZWRITEENABLE,TRUE);



		m_pD3DDev->SetTextureStageState(0, D3DTSS_COLOROP, D3DTOP_SELECTARG1);
		m_pD3DDev->SetVertexShader(m_dwNormalShader);
		m_pD3DDev->SetRenderState(D3DRS_ALPHABLENDENABLE, FALSE);
		m_pD3DDev->SetRenderState(D3DRS_ZWRITEENABLE, TRUE);
		m_pD3DDev->DrawIndexedPrimitive(D3DPT_TRIANGLELIST, 0, m_dwNumVerts, 0, m_dwNumFaces);

		// use this vertex buffer as our source for vertices
		hr = m_pD3DDev->SetStreamSource(0, m_pVertexBuffer, sizeof(PSystemVertex));
		if (FAILED(hr))
			return hr;
		//set index buffer
		m_pD3DDev->SetIndices(m_pIndexBuffer, 0);

		//pass two
		m_pD3DDev->SetRenderState(D3DRS_ALPHABLENDENABLE, FALSE);
		m_pD3DDev->SetRenderState(D3DRS_LIGHTING, FALSE);
		m_pD3DDev->SetRenderState(D3DRS_CULLMODE, D3DCULL_NONE);

		m_pD3DDev->SetRenderState(D3DRS_DESTBLEND,D3DBLEND_ONE);
		m_pD3DDev->SetRenderState(D3DRS_SRCBLEND,D3DBLEND_ONE);
		m_pD3DDev->SetRenderState(D3DRS_ZWRITEENABLE,FALSE);
		m_pD3DDev->SetRenderState(D3DRS_ZENABLE,			TRUE);

		//arg1 is the texture
		m_pD3DDev->SetTexture(0, m_pShadeTexture[m_iCurrentTexture]);
//		m_pD3DDev->SetTextureStageState(0, D3DTSS_COLOROP, D3DTOP_SELECTARG1);
		m_pD3DDev->SetTextureStageState(0, D3DTSS_COLOROP, D3DTOP_SELECTARG2);

		m_pD3DDev->SetRenderState(D3DRS_ALPHABLENDENABLE, TRUE);
		m_pD3DDev->SetRenderState(D3DRS_ZWRITEENABLE, FALSE);

		m_pD3DDev->SetVertexShader(m_dwParticle1D_BShader);
//		D3DXVECTOR4 color(0.9f, 0.9f, 0.0f, 0.0f);
//		m_pD3DDev->SetVertexShaderConstant(CV_COLOR, &color, 1);
		D3DXVECTOR4 const2Nums((sin(0.004f*timeGetTime())+1.0f)*8.0f,0,0,0);
		m_pD3DDev->SetVertexShaderConstant(CV_DATACONSTANTS, &const2Nums, 1);
		
		m_pD3DDev->DrawIndexedPrimitive(D3DPT_TRIANGLELIST, 0, m_dwNumParticles*4, 0, m_dwNumParticles*2);

		//pass two
//		m_pD3DDev->SetVertexShader(m_dwNormalShader);
//		m_pD3DDev->SetRenderState(D3DRS_ALPHABLENDENABLE, FALSE);
//		m_pD3DDev->SetRenderState(D3DRS_ZWRITEENABLE, TRUE);
//		m_pD3DDev->DrawIndexedPrimitive(D3DPT_TRIANGLELIST, 0, m_dwNumParticles*4, 0, m_dwNumParticles*2);




/*		m_pD3DDev->Clear(0, NULL, D3DCLEAR_TARGET | D3DCLEAR_ZBUFFER, 0x00BFBFBF, 1.0f, 0L);
		
		//pass one
		m_pD3DDev->SetTexture(0, m_pShadeTexture[m_iCurrentTexture]);
//		m_pD3DDev->SetTextureStageState(0, D3DTSS_COLOROP, D3DTOP_SELECTARG1);
		m_pD3DDev->SetTextureStageState(0, D3DTSS_COLOROP, D3DTOP_SELECTARG2);

		m_pD3DDev->SetRenderState(D3DRS_ALPHABLENDENABLE, FALSE);
		m_pD3DDev->SetRenderState(D3DRS_ZWRITEENABLE, FALSE);

		m_pD3DDev->SetVertexShader(m_dwParticle1D_BShader);
//		D3DXVECTOR4 color(0.0f, 0.0f, 0.0f, 0.0f);
//		m_pD3DDev->SetVertexShaderConstant(CV_COLOR, &color, 1);
		
		m_pD3DDev->DrawIndexedPrimitive(D3DPT_TRIANGLELIST, 0, m_dwNumParticles*4, 0, m_dwNumParticles*2);

		//pass two

		//arg2 is the texture
//		m_pD3DDev->SetTextureStageState(0, D3DTSS_COLOROP, D3DTOP_SELECTARG2);
//		m_pD3DDev->SetVertexShader(m_dwParticle1D_SShader);
//		m_pD3DDev->SetRenderState(D3DRS_ZWRITEENABLE, TRUE);
//		m_pD3DDev->DrawIndexedPrimitive(D3DPT_TRIANGLELIST, 0, m_dwNumParticles*4, 0, m_dwNumParticles*2);
*/
	}


	return hr;
}


void CShaderKosmo::MouseButton(HWND hWnd, eButtonID button, bool bDown, int x, int y)
{
	if(button == MOUSE_LEFTBUTTON)
	{
		if(bDown)
		{
			m_pUI->OnLButtonDown(x, y);
		}
		else
		{
			m_pUI->OnLButtonUp(x, y);
		}
	}
	return;
}


void CShaderKosmo::MouseMove(HWND hWnd, int x, int y)
{
	m_pUI->OnMouseMove(x, y);
	return;
}

void CShaderKosmo::Keyboard(DWORD dwKey, UINT nFlags, bool bDown)
{
	if(bDown)
	{
		switch(dwKey)
		{
			case 'W' :
            case 'w' :
            {
                m_bWireframe = !m_bWireframe;
				m_dwEffectDirtyFlags |= EBEFFECT_DIRTY_PUBLICSTATE;
                break;
            }

            case 'H' :
            case VK_F1 :
            {
		        ::MessageBoxEx( NULL, " Help : F1 - Help \n\n Home - Reset To Defaults \n\n W - Wireframe Toggle \n\n Space\\Pause - Toggle Pause/Resume \n\n Left Button & Mouse - Rotate Object\n\n Shift Left Button & Mouse - Pan Camera \n\n Ctrl Left Button & Mouse - Move Camera In & Out\n\n + increase silhouette width\n\n - decrease silhouette width",
					   "Help", MB_ICONINFORMATION | MB_TASKMODAL, MAKELANGID( LANG_NEUTRAL, SUBLANG_DEFAULT ) );
                break;
            }

			case ' ':
			{
				m_iScene++;
				if(m_iScene>8)m_iScene=0;
				FillVertexBuffer();return;
			}

            case VK_HOME :
            case VK_END :
			case VK_NUMPAD7 :
			case VK_NUMPAD1 :
            case '7' :
            case '1' :
            {
                m_pUI->Reset();
                m_bWireframe = false;
				m_dwEffectDirtyFlags |= EBEFFECT_DIRTY_PUBLICSTATE;
                break;
            }

			case 'Q':
			{
				m_fParticleLength+=0.01f;
				break;
			}
			case 'A':
			{
				m_fParticleLength-=0.01f;
				if(m_fParticleLength < 0.01f)
					m_fParticleLength = 0.01f;
				break;
			}
/*
			case 'W':
			{
				m_fTestConstant1+=0.01f;
				break;
			}
			case 'S':
			{
				m_fTestConstant1-=0.01f;
//				if(m_fTestConstant < 0.01f)
//					m_fTestConstant = 0.01f;
				break;
			}

			case 'E':
			{
				m_fTestConstant2+=0.01f;
				break;
			}
			case 'D':
			{
				m_fTestConstant2-=0.01f;
//				if(m_fTestConstant < 0.01f)
//					m_fTestConstant = 0.01f;
				break;
			}
*/
			case VK_ADD:
			case 0xBB:
			case '+':
			case '=':
			{
				m_iCurrentTexture++;
				if(m_iCurrentTexture > 8)
					m_iCurrentTexture--;
				break;
			}
			case VK_SUBTRACT:
			case 0xBD:
			case '-':
			case '_':
			{
				m_iCurrentTexture--;
				if(m_iCurrentTexture < 0)
					m_iCurrentTexture = 0;
				break;
			}

            default :
            {
                break;
            }
        }

	}
	SetVertexShaderMatrices();
	return;
}

void CShaderKosmo::PropertyUpdateCallback(const EBProperty* pProperty, bool bWritten)
{
	if (!bWritten)
		return;

	EBString name;

	if (pProperty->IsKindOf(EBTYPE_TRIGGER_PROP))
	{

		name = pProperty->GetPropertyName();
		
		if(name == EBString(STR_INCREASE_TEXTURE))
		{
			Keyboard(VK_ADD, 0, true);
		}
		else if (name == EBString(STR_DECREASE_TEXTURE))
		{
			Keyboard(VK_SUBTRACT, 0, true);
		}
		else if (name == EBString(STR_INCREASE_SCENE))
		{
			Keyboard(VK_SPACE, 0, true);
		}
	}
}



