/*********************************************************************NVMH2****
Path:  E:\devrel\NV_SDK_4\DX8\NVEffectsBrowser\Effects\shader_Kosmo
File:  shader_Kosmo.h

Copyright (C) 1999, 2000 NVIDIA Corporation
This file is provided without support, instruction, or implied warranty of any
kind.  NVIDIA makes no guarantee of its fitness for a particular purpose and is
not liable under any circumstances for any damages or loss whatsoever arising
from the use or inability to use this file or items derived from it.

Comments:


******************************************************************************/


#ifndef SHADER_KOSMO_H
#define SHADER_KOSMO_H
#include "MouseUI.h"

//Kosmo Vertex
struct HVertex
{
	float x, y, z;
	float nx, ny, nz;
};


typedef struct tagPSystemVertex
{
	D3DXVECTOR3 input_v0;
	D3DXVECTOR3 input_v1;
	D3DXVECTOR4 input_v2;
	D3DXVECTOR3 input_v3;
} PSystemVertex;

static const DWORD NUM_SHADETEXTURES = 9;

typedef enum tageKosmoDisplayOptions
{
	KOSMO1_DISPLAY = 0,
	KOSMO2_DISPLAY = 1,
} eKosmoDisplayOptions;

class CShaderKosmo : public EBEffect
{
public:	
	CShaderKosmo();
	~CShaderKosmo();
	
	virtual void    UpdateProperties();
	virtual HRESULT Initialize(IDirect3DDevice8* pDev);
	virtual HRESULT Free();
	virtual HRESULT Start();
	virtual HRESULT Tick(EBTimer* pTimer);
	virtual HRESULT ConfirmDevice(D3DCAPS8* pCaps, DWORD dwBehavior, D3DFORMAT Format);
	virtual void MouseMove(HWND hWnd, int x, int y);
	virtual void MouseButton(HWND hWnd, eButtonID Button, bool bDown, int x, int y);
	virtual void Keyboard(DWORD dwKey, UINT nFlags, bool bDown);
	virtual void PropertyUpdateCallback(const EBProperty* pProperty, bool bWritten);
	

	HRESULT		FillVertexBuffer( void );

private:
	LPDIRECT3DVERTEXBUFFER8 m_pVertexBuffer;		// Particles
	LPDIRECT3DINDEXBUFFER8 m_pIndexBuffer;			// Particles

	LPDIRECT3DVERTEXBUFFER8 m_pVertexBuffer2;		// Mesh
	LPDIRECT3DINDEXBUFFER8 m_pIndexBuffer2;			// Mesh

	//our matrices
	D3DXMATRIX m_world;
	D3DXMATRIX m_view;
	D3DXMATRIX m_proj;
	
	//number of sections in the mesh
	DWORD m_dwNumSections;
	
	DWORD m_dwParticle1D_SShader, m_dwParticle1D_BShader,m_dwNormalShader;

	DWORD m_dwNumParticles;
	DWORD m_dwNumVerts, m_dwNumFaces;
	
	bool m_bWireframe;
	
	MouseUI* m_pUI;

	//bounding sphere attributes
	float m_fRadius;
	D3DXVECTOR3 m_vecCenter;

	int m_iScene;

	// 1D particles
	float m_fParticleLength;

	float m_fTestConstant1;
	float m_fTestConstant2;

	// Textures
	LPDIRECT3DTEXTURE8 m_pShadeTexture[9];
	int	m_iCurrentTexture;	

	//display options
	eKosmoDisplayOptions m_eDisplayOption;
	
	HRESULT LoadXFile(const char* fileName, const DWORD dwFVF);
	HRESULT SetVertexShaderMatrices();
};

#endif