/******************************************************************************

 Particle System Effect Plugin
 Greg Snook    
      
        
******************************************************************************/

#ifndef __PARTICLE_SYSTEM2_H
#define __PARTICLE_SYSTEM2_H
#include "MouseUI.h"
#include "ParticleDialog\common.h"

typedef struct tagParticleVertex2
{
	D3DXVECTOR3 LaunchVector;	// Normalized direction the particle was launched in
	float Data;					// whole number is the vertex number (0,1,2), fractional part is the birth offset (0-.999)
	D3DXVECTOR3 Origin;			// location of particle birth
	float pad;					// pad out to 32 bytes
} ParticleVertex2;

struct BaseModelVertex
{
	float x, y, z;
	float nx, ny, nz;
//	float u,v;
	enum FVF
	{
		FVF_Flags = D3DFVF_XYZ | D3DFVF_NORMAL// | D3DFVF_TEX0
	};
};

class CParticleSystemShader2 : public EBEffect
{
public:
	CParticleSystemShader2();
	~CParticleSystemShader2();
	virtual void UpdateProperties();
	virtual HRESULT Initialize(LPDIRECT3DDEVICE8 pDev);
	virtual HRESULT Free();
	virtual HRESULT Start();
	virtual HRESULT Tick(EBTimer* pTimer);
	virtual HRESULT ConfirmDevice(D3DCAPS8* pCaps, DWORD dwBehavior, D3DFORMAT Format);
	virtual void Keyboard(DWORD dwKey, UINT nFlags, bool bDown);
	virtual void MouseButton(HWND hWnd, eButtonID button, bool bDown, int x, int y);
	virtual void MouseMove(HWND hWnd, int x, int y);
	virtual void PropertyUpdateCallback(const EBProperty* pProperty, bool bWritten);
	
private:
    void WriteVertexShaderConstants() const;
    void SetViewFromKeyboardInput(EBTimer* pTimer, D3DXMATRIX &matView);
	HRESULT LoadBaseModel(const char* fileName);

    enum {
        kStartParticles =  2400,
        kNumParticles   = 10000,
        kNumRndVectors  =    40,
        kMaxVKey        =   256,
    };

    bool                    m_bKey[kMaxVKey];
	DWORD m_iTris,m_iVerts;
	DWORD m_dwCurrentShader;
	LPDIRECT3DVERTEXBUFFER8 m_pVertexBuffer;
	LPDIRECT3DTEXTURE8 m_pLogoTexture;
	bool m_bWireframe;
	MouseUI* m_pUI;

	HMODULE m_hDialog;
	OptionsDialogFunc m_DialogFunc;
	DialogSettings m_ParticleSettings;

	float m_ElapsedTime;

	D3DXVECTOR4 m_Origin;	// origin position
	D3DXVECTOR4 m_Magnet;	// magnet position
	D3DXVECTOR4 m_Gravity;	// gravity vector
	D3DXVECTOR4 m_Wind;		// wind vector

	D3DXVECTOR4 m_StartColor;
	D3DXVECTOR4 m_EndColor;

	D3DXVECTOR4 m_SineCosineTable[32];
	D3DXVECTOR4 m_VertexData[3];
	D3DXVECTOR4 m_UVTable[12];

	D3DXVECTOR4 m_ConstantValues;

	LPDIRECT3DVERTEXBUFFER8   m_pBaseVertexBuffer;
	LPDIRECT3DINDEXBUFFER8    m_pBaseIndexBuffer;
	DWORD m_dwNumSections;
	int m_NumVerts;
	int m_NumFaces;

	D3DLIGHT8 m_Light;

};

#endif