/**

Shining flare effect

This is a quickie for the NVidia shader competition.  Rotate the flare to change the
orientation

Displays a texture mapped triangle fan that is radially distorted.  Vertex shader
is used for interpolation.  Radial distortion and flare gradients are loaded from files.

The NVidia framework code is copied from Quad example.


---

  used file resources:
  lc.dds (gradient)
  kasvu.raw (radial pattern)
  flaba.vso (vertex shader)


  **/

#include "eb_effect.h"
#include "shader_Flaba.h"
#include "nvinc.h"
#include <NVFileDialog.h>
#include <stdio.h>
#include "nvstatemanager.h"


using namespace std;
using namespace nv_objects;

static NVStateManager* g_pStateManager = NULL;

DECLARE_EFFECT_MAIN()


#define VERTICES (257*33)

extern "C"
{

__declspec(dllexport) unsigned int GetNumEffects() { return 1; }

__declspec(dllexport) EBEffect* CreateEffect(unsigned int EffectNum)
{	
	switch(EffectNum)
	{
		case 0:
			return new CShaderFlaba();
			break;
		default:
			return NULL;
	}
}

}

CShaderFlaba::CShaderFlaba()
	: m_pVertexBuffer(NULL),
	m_pLogoTexture(NULL),
	m_dwCurrentShader(0),
	m_bWireframe(false),
	m_pUI(NULL)
{
	m_strEffectLocation = "3rd Party\\Deformation";
	m_strEffectName = "Shining Flare";
	m_strEffectVertexShader = GetFilePath("Flaba.nvv");
	m_strEffectPixelShader = "";
	m_strEffectVersion = "1.0";
}

void CShaderFlaba::UpdateProperties()
{
	EBEffect::UpdateProperties();

	m_pVertexShaderEnum->AddEnumerant(new EBEnumValue(m_pVertexShaderEnum, "simple", GetFilePath("Flaba.nvv"), EBTYPE_STRING_PROP));
	AddProperty(new EBProperty("Wireframe", OBJECT_MEMBER(m_bWireframe), EBTYPE_BOOL_PROP));

    string aboutText = "file://";
    aboutText += GetFilePath("..\\..\\..\\NVEffectsExplained.htm");
    aboutText += "#shinningflare";
	SetAboutInfo( NULL, _T("Shinning Flare"), _T(aboutText.c_str()));

}

CShaderFlaba::~CShaderFlaba()
{
	Free();	
}

HRESULT CShaderFlaba::Initialize(LPDIRECT3DDEVICE8 pDev)
{
	HRESULT hr;

	m_pD3DDev = pDev;
	pDev->AddRef();

	g_pStateManager = new NVStateManager(m_pD3DDev);

	//initialize mouse UI
	RECT rect;
	rect.left = rect.top = 0;
	D3DVIEWPORT8 viewport;
	m_pD3DDev->GetViewport(&viewport);
	rect.bottom = viewport.Height;
	rect.right  = viewport.Width;
	m_pUI = new MouseUI((const RECT)rect);

	OutputDebugString("stream declaration\n");

	vector<DWORD> Declaration;
	Declaration.push_back(D3DVSD_STREAM(0));
	Declaration.push_back(D3DVSD_REG(0, D3DVSDT_FLOAT3));
	Declaration.push_back(D3DVSD_REG(1, D3DVSDT_FLOAT2));
	Declaration.push_back(D3DVSD_REG(2, D3DVSDT_FLOAT2));
	//D3DVSDT_FLOAT1 
	Declaration.push_back(D3DVSD_END());

	OutputDebugString("LoadAndCreateShader\n");
	hr = LoadAndCreateShader(GetFilePath("Flaba.vso"), &Declaration[0], 0, SHADERTYPE_VERTEX, &m_dwCurrentShader);
	if (FAILED(hr)) return hr;

	OutputDebugString("CreateVertexBuffer 2\n");
	// triangle fan
    hr = m_pD3DDev->CreateVertexBuffer(VERTICES*sizeof(FlabaVertex), D3DUSAGE_WRITEONLY, 0, D3DPOOL_MANAGED, &m_pVertexBuffer);
	if (FAILED(hr)) return hr;

	NVSTATEMANAGER.SetStreamSource(0, m_pVertexBuffer, sizeof(FlabaVertex));
	NVSTATEMANAGER.SetVertexShader(m_dwCurrentShader);
	
	FlabaVertex* pBuff;
	if (m_pVertexBuffer)
	{
		float fScale = 1.0f;
		BYTE buf[256*32];

		FILE *stream;
		stream=fopen(GetFilePath("kasvu.raw").c_str(), "rb" );
		fseek(stream, 0x320, SEEK_SET);
		fread(buf, 1, sizeof(buf), stream);
		fclose(stream);

		hr = m_pVertexBuffer->Lock(0, VERTICES*sizeof(FlabaVertex),(BYTE**)&pBuff, 0);
		if (FAILED(hr))
		{
			m_strLastError = "Couldn't lock buffer!";
			return hr;
		}

		for(int j=0;j<32;j++)
		{
			pBuff->Position = D3DXVECTOR3(0.0f, 0.0f, 0.1f);
			pBuff->Texture = D3DXVECTOR2(0.0f, 0.0f);
			pBuff->Distortion = D3DXVECTOR2(0.f, 0.f);
			pBuff++;

			for(int i=0;i<256;i++)
			{
				float a=2*3.14159265358979f/255.0f;
				//f=(f+(rand()&255)/255.f)*.5f;

				float f0=buf[i+j*256]/255.f;
				float f1=buf[i+((j+1)&31)*256]/255.f;
				pBuff->Position = D3DXVECTOR3(sin(i*a)*5, 5*cos(i*a), 0);
				pBuff->Texture = D3DXVECTOR2(1, 0);
				pBuff->Distortion = D3DXVECTOR2(f0, f1);
				pBuff++;
			}
		}

		m_pVertexBuffer->Unlock();
	}

	// Create the logo texture
	hr = D3DXCreateTextureFromFileEx(m_pD3DDev, 
		GetFilePath("lc.dds").c_str(),
		D3DX_DEFAULT,
		D3DX_DEFAULT,
		0,
		0,
		D3DFMT_UNKNOWN,
		D3DPOOL_MANAGED,
		D3DX_FILTER_LINEAR,
		D3DX_FILTER_LINEAR,
		0,
		NULL,
		NULL,
		&m_pLogoTexture);
	if (FAILED(hr))
	{
		m_strLastError = "Could not create nvlogo_only.dds";
		return hr;
	}

	NVSTATEMANAGER.SetRenderState(D3DRS_CULLMODE, D3DCULL_NONE);

	NVSTATEMANAGER.SetTexture(0, m_pLogoTexture);
	NVSTATEMANAGER.SetTextureStageState(0, D3DTSS_COLOROP, D3DTOP_MODULATE);
	NVSTATEMANAGER.SetTextureStageState(0, D3DTSS_COLORARG1, D3DTA_TEXTURE);
	NVSTATEMANAGER.SetTextureStageState(0, D3DTSS_COLORARG2, D3DTA_DIFFUSE);

	NVSTATEMANAGER.SetTextureStageState(0, D3DTSS_ALPHAOP, D3DTOP_ADD);
	NVSTATEMANAGER.SetTextureStageState(0, D3DTSS_ALPHAARG1, D3DTA_TEXTURE);
	NVSTATEMANAGER.SetTextureStageState(0, D3DTSS_ALPHAARG2, D3DTA_DIFFUSE);

	NVSTATEMANAGER.SetTextureStageState(1, D3DTSS_COLOROP, D3DTOP_DISABLE);
	NVSTATEMANAGER.SetTextureStageState(1, D3DTSS_ALPHAOP, D3DTOP_DISABLE);

	NVSTATEMANAGER.SetTextureStageState(0, D3DTSS_MINFILTER, D3DTEXF_LINEAR);
    NVSTATEMANAGER.SetTextureStageState(0, D3DTSS_MAGFILTER, D3DTEXF_LINEAR);
	NVSTATEMANAGER.SetTextureStageState(0, D3DTSS_MIPFILTER, D3DTEXF_NONE);

	NVSTATEMANAGER.SetTextureStageState(0, D3DTSS_ADDRESSU, D3DTADDRESS_CLAMP);

	return S_OK;

}

HRESULT CShaderFlaba::Free()
{
	SAFE_RELEASE(m_pLogoTexture);
	SAFE_RELEASE(m_pVertexBuffer);
	SAFE_DELETE(m_pUI);

	if (m_pD3DDev)
	{
		if (m_dwCurrentShader)
			m_pD3DDev->DeleteVertexShader(m_dwCurrentShader);
		SAFE_RELEASE(m_pD3DDev);
	}
	SAFE_DELETE(g_pStateManager);
	
	return S_OK;
}

HRESULT CShaderFlaba::Start()
{
	return S_OK;
}

HRESULT CShaderFlaba::Tick(EBTimer* pTimer)
{
	HRESULT hr = S_OK;
	D3DXMATRIX matWorld;
    D3DXMATRIX matView;
	D3DXMATRIX matProj;
	D3DXMATRIX matTemp;
	D3DXMATRIX matWorldViewProj;
	D3DXMATRIX matWorldView;
	D3DXMATRIX matWorldI;

	hr = m_pD3DDev->Clear(0, NULL, D3DCLEAR_TARGET|D3DCLEAR_ZBUFFER, 0, 1.0, 0);

	D3DXVECTOR3 vEyePt = D3DXVECTOR3( 0.0f, 0.0f, -2.0f );
	D3DXVECTOR3 vLookatPt = D3DXVECTOR3( 0.0f, 0.0f, 0.0f );
	D3DXVECTOR3 vUp = D3DXVECTOR3( 0.0f, 1.0f, 0.0f );

	// Setup the world, view and projection matrices
	D3DXMatrixMultiply( &matWorld, &m_pUI->GetRotationMatrix(), &m_pUI->GetTranslationMatrix() );

	D3DXMatrixLookAtLH(&matView, &vEyePt, &vLookatPt, &vUp);
	D3DXMatrixPerspectiveFovLH(&matProj, D3DXToRadian(60.0f), 1.0f, 1.0f, 50.0f);
		
	// Setup the composite matrix to get vertices to projection space
	D3DXMatrixMultiply(&matTemp, &matWorld, &matView);
	D3DXMatrixMultiply(&matWorldViewProj, &matTemp, &matProj);
	D3DXMatrixMultiply(&matWorldView, &matWorld, &matView);
	
	// Create a 3x3 inverse of the worldview for the light transformation back to model space
	D3DXMatrixInverse(&matWorldI, NULL, &matWorld);
	D3DXMatrixTranspose(&matWorldViewProj, &matWorldViewProj);

	// Setup constants
	NVSTATEMANAGER.SetVertexShaderConstant(0, &matWorldViewProj(0, 0), 4);
	
	float time=pTimer->GetDuration()*10;
	int foo=(int)(time);


	D3DXVECTOR4 frac(time-foo,1-(time-foo),0,1.0f);
	NVSTATEMANAGER.SetVertexShaderConstant(4, &frac, 1);


	NVSTATEMANAGER.SetRenderState(D3DRS_CULLMODE, D3DCULL_NONE);
	NVSTATEMANAGER.SetRenderState(D3DRS_FILLMODE, m_bWireframe ? D3DFILL_WIREFRAME : D3DFILL_SOLID);
	NVSTATEMANAGER.SetRenderState(D3DRS_ALPHABLENDENABLE, TRUE);

	NVSTATEMANAGER.DrawPrimitive(D3DPT_TRIANGLEFAN, 257*(foo&31), 257);

	return hr;
}

HRESULT CShaderFlaba::ConfirmDevice(D3DCAPS8* pCaps, DWORD dwBehavior, D3DFORMAT Format)
{
	return S_OK;
}

void CShaderFlaba::MouseButton(HWND hWnd, eButtonID button, bool bDown, int x, int y)
{
	if(button == MOUSE_LEFTBUTTON)
	{
		if(bDown)
		{
			m_pUI->OnLButtonDown(x, y);
		}
		else
		{
			m_pUI->OnLButtonUp(x, y);
		}
	}
	return;
}

void CShaderFlaba::MouseMove(HWND hWnd, int x, int y)
{
	m_pUI->OnMouseMove(x, y);
	return;
}

void CShaderFlaba::Keyboard(DWORD dwKey, UINT nFlags, bool bDown)
{
	eEBKeyAction Action = TranslateEffectKey(dwKey, nFlags, bDown);
	
    switch ( Action )
    {
		case EB_HELP:
		{
			::MessageBoxEx( NULL, " Help : F1 - Help \n\n Home - Reset To Defaults \n\n W - Wireframe Toggle \n\n Left Button & Mouse - Rotate Object\n\n Shift Left Button & Mouse - Pan Camera \n\n Ctrl Left Button & Mouse - Move Camera In & Out\n\n",
				   "Help", MB_ICONINFORMATION | MB_TASKMODAL, MAKELANGID( LANG_NEUTRAL, SUBLANG_DEFAULT ) );
		}
		break;

		case EB_WIREFRAME:
        {
            m_bWireframe = !m_bWireframe;
			m_dwEffectDirtyFlags |= EBEFFECT_DIRTY_PUBLICSTATE;
            
        }
		break;

		case EB_RESET:
        {
            m_pUI->Reset();
            m_bWireframe = false;
			m_dwEffectDirtyFlags |= EBEFFECT_DIRTY_PUBLICSTATE;
            
        }
		break;

        default :
            break;
    }
}
