/*********************************************************************NVMH2****
Path:  \NVEffectsBrowser\Effects\Sketch
File:  Sketch.h

Copyright (C) 2001 David E. Tin Nyo
http://www.meme-x.com

Derived from shader_ToonShade.h:
	Copyright (C) 1999, 2000 NVIDIA Corporation
	This file is provided without support, instruction, or implied warranty of any
	kind.  NVIDIA makes no guarantee of its fitness for a particular purpose and is
	not liable under any circumstances for any damages or loss whatsoever arising
	from the use or inability to use this file or items derived from it.

Comments:


******************************************************************************/


#ifndef SHADER_SKETCH_H
#define SHADER_SKETCH_H
#include "MouseUI.h"

//SketchVertex
struct SkVertex
{
	float x, y, z;
	float nx, ny, nz;
	float u0,v0;         //texcoord for stage 0, crosshatch pattern1
	float u1,v1;         //texcoord for stage 1, crosshatch pattern2
	float u2;			 //texcoord for stage 2, shading greyscale 1
	float u3;			 //texcoord for stage 3, shading greyscale 2
};

typedef enum tageSketchDisplayOptions
{
	SKETCHDISPLAY_LIGHTPOS0 = 0,   // offset
	SKETCHDISPLAY_LIGHTPOS1 = 1,   // opposite
	SKETCHDISPLAY_LIGHTPOS2 = 2,   // same
} eSketchDisplayOptions;

class CShaderSketch : public EBEffect
{
public:	
	CShaderSketch();
	~CShaderSketch();
	
	virtual void    UpdateProperties();
	virtual HRESULT Initialize(IDirect3DDevice8* pDev);
	virtual HRESULT Free();
	virtual HRESULT Start();
	virtual HRESULT Tick(EBTimer* pTimer);
	virtual HRESULT ConfirmDevice(D3DCAPS8* pCaps, DWORD dwBehavior, D3DFORMAT Format);
	virtual void MouseMove(HWND hWnd, int x, int y);
	virtual void MouseButton(HWND hWnd, eButtonID Button, bool bDown, int x, int y);
	virtual void Keyboard(DWORD dwKey, UINT nFlags, bool bDown);
	
private:
	LPDIRECT3DVERTEXBUFFER8	  m_pVertexBuffer;
	LPDIRECT3DINDEXBUFFER8    m_pIndexBuffer;
	
	LPDIRECT3DTEXTURE8 m_pCross1Texture;
	LPDIRECT3DTEXTURE8 m_pCross2Texture;
	LPDIRECT3DTEXTURE8 m_pShade1Texture;
	LPDIRECT3DTEXTURE8 m_pShade2Texture;
	
	D3DXATTRIBUTERANGE*  m_pAttributes;
	
	//our matrices
	D3DXMATRIX m_world;
	D3DXMATRIX m_view;
	D3DXMATRIX m_proj;
	
	//number of sections in the mesh
	DWORD m_dwNumSections;
	
	DWORD m_dwCurrentVShader;
	DWORD m_dwCurrentPShader;

	bool m_bAnisotropic;
		
	MouseUI* m_pUI;

	//bounding sphere attributes
	float m_fRadius;
	D3DXVECTOR3 m_vecCenter;

	eSketchDisplayOptions m_eDisplayOption;
	
	HRESULT LoadXFile(const char* fileName, const DWORD dwFVF);
	HRESULT SetVertexShaderMatrices();
	HRESULT SetLightPositions();
	
};

#endif