/******************************************************************************

  Based on code Copyright (C) 1999, 2000 NVIDIA Corporation
        
******************************************************************************/

#include "eb_effect.h"
#include "shader_Spyrorama.h"
#include "nvinc.h"

using namespace std;

DECLARE_EFFECT_MAIN()

extern "C"
{

__declspec(dllexport) unsigned int GetNumEffects() { return 1; }

__declspec(dllexport) EBEffect* CreateEffect(unsigned int EffectNum)
{	
	switch(EffectNum)
	{
		case 0:
			return new CShaderSpyrorama();
			break;
		default:
			return NULL;
	}
}

}

CShaderSpyrorama::CShaderSpyrorama()
	: 
	m_dwCurrentShader(0),
	m_bWireframe(false),
	m_pObjVB(NULL),
	m_pObjIB(NULL),
	m_fLastShift(0.0f),
	m_pUI(NULL)
{
	m_strEffectLocation = "3rd Party\\Procedural";
	m_strEffectName = "Spyrorama!!";
	m_strEffectVertexShader = GetFilePath("spyrorama.nvv");
	m_strEffectPixelShader = "";
	m_strEffectVersion = "1.0";
	srand(timeGetTime());
}

void CShaderSpyrorama::UpdateProperties()
{
	EBEffect::UpdateProperties();

	m_pVertexShaderEnum->AddEnumerant(new EBEnumValue(m_pVertexShaderEnum, "Polar Coords to Colors", GetFilePath("Spyrorama.nvv"), EBTYPE_STRING_PROP));
	AddProperty(new EBProperty("Wireframe", OBJECT_MEMBER(m_bWireframe), EBTYPE_BOOL_PROP));

    string aboutText = "file://";
    aboutText += GetFilePath("..\\..\\..\\..\\NVEffectsExplained.htm");
    aboutText += "#spyrorama";
	SetAboutInfo( NULL, _T("Spyrorama!"), _T(aboutText.c_str()));
}

CShaderSpyrorama::~CShaderSpyrorama()
{
	Free();	
}


static void ComputePolarCoordinates(FLOAT x, FLOAT y, D3DXVECTOR2* coords);

static void ComputePolarCoordinates(FLOAT x, FLOAT y, D3DXVECTOR2* coords)
{
	FLOAT	r = sqrtf(x*x+y*y),a;
	if (r == 0.0)
		a = 0.0f;
	else {
		if (y > 0)
			a =  (D3DX_PI*2.0f)-acosf(x/r);
		else
			a = acosf(x/r);
	}
	coords->x = r;
	coords->y = a;
}


HRESULT CShaderSpyrorama::Initialize(LPDIRECT3DDEVICE8 pDev)
{
	HRESULT hr;

    // get and store the d3d device
	m_pD3DDev = pDev;
	pDev->AddRef();

	//initialize mouse UI
	RECT rect;
	rect.left = rect.top = 0;
	D3DVIEWPORT8 viewport;
	m_pD3DDev->GetViewport(&viewport);
	rect.bottom = viewport.Height;
	rect.right  = viewport.Width;
	m_pUI = new MouseUI((const RECT)rect);


    // here we declare what the vertex format is for this vertex shader: 
    // the vertex shader will expect 3 floats (position) in v0, 3 floats (normal)
    // in v1, and 2 floats (uv) in v2.
	vector<DWORD> Declaration;
	Declaration.push_back(D3DVSD_STREAM(0));
	Declaration.push_back(D3DVSD_REG(0, D3DVSDT_FLOAT3));
	Declaration.push_back(D3DVSD_REG(1, D3DVSDT_FLOAT3));
	Declaration.push_back(D3DVSD_REG(2, D3DVSDT_FLOAT2));
	Declaration.push_back(D3DVSD_END());

    // here we instantiate the vertex shader by loading it's object file and setting the declaration
	hr = LoadAndCreateShader(GetFilePath("Spyrorama.vso"), &Declaration[0], 0, SHADERTYPE_VERTEX, &m_dwCurrentShader);
	if (FAILED(hr))
		return hr;

    // and use it for all vertices that are rendered
	hr = m_pD3DDev->SetVertexShader(m_dwCurrentShader);
	if (FAILED(hr))
		return hr;

	m_pD3DDev->SetRenderState(D3DRS_CULLMODE, D3DCULL_NONE);

	m_pD3DDev->SetTextureStageState(0, D3DTSS_COLOROP, D3DTOP_MODULATE);
	m_pD3DDev->SetTextureStageState(0, D3DTSS_COLORARG1, D3DTA_TEXTURE);
	m_pD3DDev->SetTextureStageState(0, D3DTSS_COLORARG2, D3DTA_DIFFUSE);

	m_pD3DDev->SetTextureStageState(0, D3DTSS_ALPHAOP, D3DTOP_ADD);
	m_pD3DDev->SetTextureStageState(0, D3DTSS_ALPHAARG1, D3DTA_TEXTURE);
	m_pD3DDev->SetTextureStageState(0, D3DTSS_ALPHAARG2, D3DTA_DIFFUSE);

	m_pD3DDev->SetTextureStageState(0, D3DTSS_MINFILTER, D3DTEXF_LINEAR);
    m_pD3DDev->SetTextureStageState(0, D3DTSS_MAGFILTER, D3DTEXF_LINEAR);
	m_pD3DDev->SetTextureStageState(0, D3DTSS_MIPFILTER, D3DTEXF_NONE);

    // turn off the second stage
	m_pD3DDev->SetTextureStageState(1, D3DTSS_COLOROP, D3DTOP_DISABLE);
	m_pD3DDev->SetTextureStageState(1, D3DTSS_ALPHAOP, D3DTOP_DISABLE);

	LPD3DXMESH tempMesh;
	LPD3DXMESH m_pObj;

	hr = D3DXCreateSphere( m_pD3DDev, 1.0f, 64, 64, &tempMesh, NULL);

	tempMesh->CloneMeshFVF( D3DXMESH_SYSTEMMEM, SPYRO_D3DVERTEX, m_pD3DDev, &m_pObj);
	SAFE_RELEASE(tempMesh);
  D3DXComputeNormals( m_pObj );

	m_NumObjVertices = m_pObj->GetNumVertices();
	m_NumObjFaces = m_pObj->GetNumFaces();
  m_pD3DDev->CreateVertexBuffer( m_NumObjVertices * sizeof(SpyroVertex),
                                    D3DUSAGE_WRITEONLY, 0, D3DPOOL_MANAGED,
                                    &m_pObjVB );
  m_pD3DDev->CreateIndexBuffer( m_NumObjFaces * 3 * sizeof(WORD),
                                    D3DUSAGE_WRITEONLY,
                                    D3DFMT_INDEX16, D3DPOOL_MANAGED,
                                    &m_pObjIB );

	// Copy Vertices/Indexes from Meshes
	 LPDIRECT3DVERTEXBUFFER8 pMeshSourceVB;
	 LPDIRECT3DINDEXBUFFER8  pMeshSourceIB;
	 SpyroVertex              *pSrc, *pDst, *sp;

	 m_pObj->GetVertexBuffer( &pMeshSourceVB );
	 m_pObjVB->Lock( 0, 0, (BYTE**)&pDst, 0 );
	 pMeshSourceVB->Lock( 0, 0, (BYTE**)&pSrc, 0 );
	 memcpy( pDst, pSrc, m_NumObjVertices * sizeof(SpyroVertex) );

	 sp = pDst;
	 for (int n = 0; n < m_NumObjVertices; ++n, ++sp)
	 {
			D3DXVECTOR2 pc1;
			FLOAT				lon1,lat1,lon2,lat2;

			lon1 = 0.0f;
			lat1 = D3DX_PI/2;
			ComputePolarCoordinates(sp->pos.x,sp->pos.y,&pc1);
			lat2 = asinf(sp->pos.z);
			lon2 = pc1.y;
			// Recompute Distance as distance along XZ arc

			FLOAT a,d;
			a = powf(sin((lat2-lat1)/2),2.0f) + 
				  cosf(lat1) * cosf(lat2) * powf(sinf((lon2-lon1)/2),2.0f);
			d = 2 * asinf(min(1.0f,sqrtf(a)));
			sp->texture.x = d;
			sp->texture.y = pc1.y;
	 }

	 m_pObjVB->Unlock();
	 pMeshSourceVB->Unlock();
	 pMeshSourceVB->Release();

	 m_pObj->GetIndexBuffer( &pMeshSourceIB );
	 m_pObjIB->Lock( 0, 0, (BYTE**)&pDst, 0 );
	 pMeshSourceIB->Lock( 0, 0, (BYTE**)&pSrc, 0 );
	 memcpy( pDst, pSrc, 3 * m_NumObjFaces * sizeof(WORD) );
	 m_pObjIB->Unlock();
	 pMeshSourceIB->Unlock();
	 pMeshSourceIB->Release();

	m_pObj->Release();

  m_pD3DDev->SetStreamSource( 0, m_pObjVB, sizeof(SpyroVertex) );
  m_pD3DDev->SetIndices( m_pObjIB, 0 );



	// The material color
	D3DXVECTOR4 matColor(1.0f, 1.0f, 1.0f, 0.0f);
	m_pD3DDev->SetVertexShaderConstant(   5, &matColor, 1);

  // Various useful constants
	D3DXVECTOR4 vZero( 0.0f, 0.0f, 0.0f, 0.0f );
  D3DXVECTOR4 vOne( 1.0f, 0.5f, 0.2f, 0.05f );
	D3DXVECTOR4 vPI(D3DX_PI, 1.f/2.f, 2.f*D3DX_PI, 1.f/(2.f*D3DX_PI) );
	D3DXVECTOR4 vCosStuff( -1.0f, 1.f/2.f, -1.f/24.f, 1.f/720.f );
	D3DXVECTOR4 vSinStuff( 1.0f, -1.f/6.f, 1.f/120.f, -1.f/5040.f );

  m_pD3DDev->SetVertexShaderConstant(  20, &vZero,     1 );
  m_pD3DDev->SetVertexShaderConstant(  21, &vOne,      1 );
  m_pD3DDev->SetVertexShaderConstant(  22, &vPI,       1 );
  m_pD3DDev->SetVertexShaderConstant(  30, &vCosStuff, 1 ); 
  m_pD3DDev->SetVertexShaderConstant(  31, &vSinStuff, 1 ); 

	ShiftPattern();

	return S_OK;
}

HRESULT CShaderSpyrorama::Free()
{
    // free all the memory we were using
	SAFE_DELETE(m_pUI);
	SAFE_RELEASE(m_pObjVB);
	SAFE_RELEASE(m_pObjIB);

	if (m_pD3DDev)
	{
		m_pD3DDev->DeleteVertexShader(m_dwCurrentShader);
		SAFE_RELEASE(m_pD3DDev);
	}
	
	return S_OK;
}

HRESULT CShaderSpyrorama::Start()
{
	return S_OK;
}


// This function is used to randomize the pattern every 4 seconds
// Ooh!  Ah!
//
#define IRAND(x)	((int) (rand()*(float)x/RAND_MAX))
void CShaderSpyrorama::ShiftPattern()
{
	m_vRotateDir.x = IRAND(2) < 1? -1 : 1;
	m_vRotateDir.y = IRAND(2) < 1? -1 : 1;
	m_vRotateDir.z = IRAND(2) < 1? -1 : 1;
  m_pD3DDev->SetVertexShaderConstant(  11, &m_vRotateDir, 1 );
	if (IRAND(2) < 1) {
		FLOAT fPleats = IRAND(3);
		m_vPleats = D3DXVECTOR4(fPleats, fPleats, fPleats, 0.0f);
	}
	else {
		m_vPleats.x = IRAND(4);
		m_vPleats.y = IRAND(4);
		m_vPleats.z = IRAND(4);
	}
  m_pD3DDev->SetVertexShaderConstant(  13, &m_vPleats, 1 );
	switch (IRAND(6)) {
	case 0:	m_vPhaseShift = D3DXVECTOR4(1.0f,2.0f,4.0f,0.0f);	break;
	case 1:	m_vPhaseShift = D3DXVECTOR4(1.0f,4.0f,2.0f,0.0f);	break;
	case 2:	m_vPhaseShift = D3DXVECTOR4(2.0f,1.0f,4.0f,0.0f); break;
	case 3:	m_vPhaseShift = D3DXVECTOR4(2.0f,4.0f,1.0f,0.0f); break;
	case 4:	m_vPhaseShift = D3DXVECTOR4(4.0f,1.0f,2.0f,0.0f); break;
	case 5:	m_vPhaseShift = D3DXVECTOR4(4.0f,2.0f,1.0f,0.0f); break;
	}
}


HRESULT CShaderSpyrorama::Tick(EBTimer* pTimer)
{
	HRESULT hr = S_OK;
	D3DXMATRIX matWorld;
  D3DXMATRIX matView;
	D3DXMATRIX matProj;
	D3DXMATRIX matTemp;
	D3DXMATRIX matWorldViewProj;
	D3DXMATRIX matWorldView;
	D3DXMATRIX matWorldI;


	FLOAT	fTime = pTimer->GetDuration();

	if (fabs(fTime - m_fLastShift) > 4) {
		m_fLastShift = fTime;
		ShiftPattern();
	}

#define kMaxTwist 32.0f
	FLOAT sTime = sinf(fTime/8) * kMaxTwist;

	D3DXVECTOR4 vTwist( sTime, sTime, sTime, 0.0f);
	D3DXVECTOR4 vPhase( fTime*m_vPhaseShift.x, fTime*m_vPhaseShift.y, fTime*m_vPhaseShift.z, 0.0f);

	m_pD3DDev->SetVertexShaderConstant(  10, &vTwist, 1 );
  m_pD3DDev->SetVertexShaderConstant(  12, &vPhase, 1 );


	hr = m_pD3DDev->Clear(0, NULL, D3DCLEAR_TARGET|D3DCLEAR_ZBUFFER, D3DCOLOR_XRGB( 0xAA, 0xAA, 0xAA ), 1.0, 0);

	m_pD3DDev->SetRenderState(D3DRS_FILLMODE, m_bWireframe ? D3DFILL_WIREFRAME : D3DFILL_SOLID);

	D3DXVECTOR3 vEyePt = D3DXVECTOR3( 0.0f, 0.0f, -5.0f );
	D3DXVECTOR3 vLookatPt = D3DXVECTOR3( 0.0f, 0.0f, 0.0f );
	D3DXVECTOR3 vUp = D3DXVECTOR3( 0.0f, 1.0f, 0.0f );

	// Setup the world, view and projection matrices
	D3DXMatrixMultiply( &matWorld, &m_pUI->GetRotationMatrix(), &m_pUI->GetTranslationMatrix() );

	D3DXMatrixLookAtLH(&matView, &vEyePt, &vLookatPt, &vUp);
	D3DXMatrixPerspectiveFovLH(&matProj, D3DXToRadian(60.0f), 1.0f, 1.0f, 50.0f);
		
	// Setup the composite matrix to get vertices to projection space
	D3DXMatrixMultiply(&matTemp, &matWorld, &matView);
	D3DXMatrixMultiply(&matWorldViewProj, &matTemp, &matProj);
	D3DXMatrixMultiply(&matWorldView, &matWorld, &matView);
	
	// Create a 3x3 inverse of the worldview for the light transformation back to model space
	D3DXMatrixInverse(&matWorldI, NULL, &matWorld);
	D3DXMatrixTranspose(&matWorldViewProj, &matWorldViewProj);

	// Setup constants
	m_pD3DDev->SetVertexShaderConstant(0, &matWorldViewProj(0, 0), 4);
	
	// Transform from world space to model space and point it away from the vertex towards the light
	// Light is pointing into the world
	D3DXVECTOR3 vecLight(0.0f, 0.0f, 1.0f);
	D3DXVec3TransformNormal(&vecLight, &vecLight, &matWorldI);
	D3DXVec3Normalize((D3DXVECTOR3*)&vecLight, (D3DXVECTOR3*)&vecLight);
	vecLight = -vecLight;
	
	m_pD3DDev->SetVertexShaderConstant(4, &vecLight.x, 1);

  m_pD3DDev->DrawIndexedPrimitive( D3DPT_TRIANGLELIST,
                                   0, m_NumObjVertices,
                                   0, m_NumObjFaces );

	return hr;
}

HRESULT CShaderSpyrorama::ConfirmDevice(D3DCAPS8* pCaps, DWORD dwBehavior, D3DFORMAT Format)
{
	return S_OK;
}

void CShaderSpyrorama::MouseButton(HWND hWnd, eButtonID button, bool bDown, int x, int y)
{
	if(button == MOUSE_LEFTBUTTON)
	{
		if(bDown)
		{
			m_pUI->OnLButtonDown(x, y);
		}
		else
		{
			m_pUI->OnLButtonUp(x, y);
		}
	}
	return;
}

void CShaderSpyrorama::MouseMove(HWND hWnd, int x, int y)
{
	m_pUI->OnMouseMove(x, y);
	return;
}

void CShaderSpyrorama::Keyboard(DWORD dwKey, UINT nFlags, bool bDown)
{
	eEBKeyAction Action = TranslateEffectKey(dwKey, nFlags, bDown);
	
    switch ( Action )
    {
		case EB_HELP:
		{
			::MessageBoxEx( NULL, " Help : F1 - Help \n\n Home - Reset To Defaults \n\n W - Wireframe Toggle \n\n Left Button & Mouse - Rotate Object\n\n Shift Left Button & Mouse - Pan Camera \n\n Ctrl Left Button & Mouse - Move Camera In & Out\n\n",
				   "Help", MB_ICONINFORMATION | MB_TASKMODAL, MAKELANGID( LANG_NEUTRAL, SUBLANG_DEFAULT ) );
		}
		break;

		case EB_WIREFRAME:
        {
            m_bWireframe = !m_bWireframe;
			m_dwEffectDirtyFlags |= EBEFFECT_DIRTY_PUBLICSTATE;
            
        }
		break;

		case EB_RESET:
        {
            m_pUI->Reset();
            m_bWireframe = false;
			m_dwEffectDirtyFlags |= EBEFFECT_DIRTY_PUBLICSTATE;
            
        }
		break;

        default :
            break;
    }
}
