#include "eb_effect.h"
#include "nvinc.h"
#include "bezier.h"

using namespace std;

DECLARE_EFFECT_MAIN()

extern "C"
{

__declspec(dllexport) unsigned int GetNumEffects() { return 1; }

__declspec(dllexport) EBEffect* CreateEffect(unsigned int EffectNum)
{
	return new Bezier();
}

}

Bezier::Bezier()
: ib(NULL),
	vb(NULL),
	tex(NULL),
	vshaderbox(0),
	vshader(0),
	bTexture(true)
{
	mpMouseUI=NULL;

	m_strEffectName = "Bezir Patches";
	m_strEffectLocation = "3rd Party\\Deformation";
	m_strEffectPixelShader = "";
	m_strEffectVersion = "1.0";

	//Set the default detail of the patch bezir
	res=32;
	//Set default value of Wireframe mode
	bWire=false;
	//Set default value of Animation mode
	bPause=false;
	//Set default position, rotation
	pos=D3DXVECTOR3(0,0,0);
	rot=D3DXVECTOR3(0,0,0);
	n  =D3DXVECTOR3(0,1,0);
	//Set to NULL vertex buffer of patch and index buffer of patch
	vb=NULL;
	ib=NULL;
	//Set to NULL vertex buffer of opened box
	vbz[0]=NULL;
	vbz[1]=NULL;
	vbz[2]=NULL;
	//Set up control point
	int c=0;
	for(int x=0;x<4;x++)
	{
		for(int z=0;z<4;z++)
		{
				controlpoint[c]=D3DXVECTOR4(x,0,z,3);
				c++;
		}
	}
	controlpoint[5].y=3;
	controlpoint[6].y=3;	
	controlpoint[9].y=3;
	controlpoint[10].y=3;
	//Set up to default the color of light and position
    lightDiffuse=D3DXVECTOR4( 0.25f,0.75f,0.25f,1.0f);
	lightSpecular=D3DXVECTOR4(0.75f,0.75f,0.25f,1.0f);
	lightPosition=D3DXVECTOR4(0.0f,1.0f,0.0f,0.0f);

	for (int i = 0; i < 3; i++)
	{
		vbz[i] = NULL;
	}
}

Bezier::~Bezier()
{
	//Free the vertex buffers
	Free();
	SAFE_DELETE(mpMouseUI);
}

void Bezier::UpdateProperties()
{
	EBEffect::UpdateProperties();
	//Ad to popup menu the wireframe on/off option
    AddProperty(new EBProperty("Wireframe",    OBJECT_MEMBER(bWire), EBTYPE_BOOL_PROP));
	//Ad to popup menu the animation on/off option
    AddProperty(new EBProperty("Stop ChangeShading",        OBJECT_MEMBER(bPause), EBTYPE_BOOL_PROP));
	//Ad to popup menu the texture on/off option
	AddProperty(new EBProperty("Texture ",  OBJECT_MEMBER(bTexture), EBTYPE_BOOL_PROP));

	m_pVertexShaderEnum->AddEnumerant(new EBEnumValue(m_pVertexShaderEnum, "3D SURFACE IN VERTEX SHADERS", GetFilePath("bezier.nvv"), EBTYPE_STRING_PROP));

    string aboutText = "file://";
    aboutText += GetFilePath("..\\..\\..\\NVEffectsExplained.htm");
    aboutText += "#bezierpatch";
	SetAboutInfo( NULL, _T("Bezier Patch"), _T(aboutText.c_str()));

}

void Bezier::CreateBuffers()
{
	//Calculate level of detail and numeber of verticies
	nvb=res*res;
	nib=(res-1)*(res-1)*6;
	//Create the vertex buffer
	m_pD3DDev->CreateVertexBuffer(nvb*sizeof(TwinUV),
			                      0, 0,
						          D3DPOOL_DEFAULT, &vb );
			
	//Set up the vertex buffer, and creating the u,v data
	//for the bezir patch calculation
	TwinUV *v;
	vb->Lock(0,nvb*sizeof(TwinUV),(BYTE**)&v, 0 );
	float c=1.0f;
	c/=res;
	for(int x=0;x<res;x++)
    {
		for(int z=0;z<res;z++)
        {
			//u,v 
			v->t0.x=z*c;
			v->t0.y=x*c;
			//1-u,1-v
			v->t1.x=1-z*c;
			v->t1.y=1-x*c;
			v++;
		}
    }
	vb->Unlock();
	//Create index buffer
	m_pD3DDev->CreateIndexBuffer( nib*sizeof(WORD),
								  0, D3DFMT_INDEX16,
								  D3DPOOL_DEFAULT, &ib);
	//Set up the index buffer to create indicies a patch
	//triangle list
	WORD *tmpword;
	ib->Lock( 0, nib*sizeof(WORD), (BYTE**)&tmpword, 0 );
    
	for(float y=1;y<res;y++)
    {
		for(float x=1;x<res;x++)
        {
			*tmpword++=(WORD)((y-1)*res+(x-1));
            *tmpword++=(WORD)((y-0)*res+(x-1));
            *tmpword++=(WORD)((y-1)*res+(x-0));

            *tmpword++=(WORD)((y-1)*res+(x-0));
            *tmpword++=(WORD)((y-0)*res+(x-1));
            *tmpword++=(WORD)((y-0)*res+(x-0));
		}
	}
	ib->Unlock();
};

void Bezier::CreateBox(float d)
{
	//Create the opened box vertex buffer

	//First box buffer
	m_pD3DDev->CreateVertexBuffer(4*sizeof(SimpleVertex),
			                      0, 0,
						          D3DPOOL_DEFAULT, &vbz[0] );
			
	//Second box buffer
	m_pD3DDev->CreateVertexBuffer(4*sizeof(SimpleVertex),
			                      0, 0,
						          D3DPOOL_DEFAULT, &vbz[1] );

	//Third box buffer
	m_pD3DDev->CreateVertexBuffer(4*sizeof(SimpleVertex),
			                      0, 0,
						          D3DPOOL_DEFAULT, &vbz[2] );

	//Set up the vertex in the first vertex buffer
	SimpleVertex *v;
	vbz[0]->Lock(0,4*sizeof(SimpleVertex),(BYTE**)&v, 0 );
	v[0].pos=D3DXVECTOR3(d,0,0);
	v[1].pos=D3DXVECTOR3(0,0,0);
	v[2].pos=D3DXVECTOR3(d,0,d);
	v[3].pos=D3DXVECTOR3(0,0,d);
	vbz[0]->Unlock();
	v=NULL;
	//Set up the vertex in the second vertex buffer
	vbz[1]->Lock(0,4*sizeof(SimpleVertex),(BYTE**)&v, 0 );
	v[0].pos=D3DXVECTOR3(0,d,0);
	v[1].pos=D3DXVECTOR3(0,0,0);
	v[3].pos=D3DXVECTOR3(d,0,0);
	v[2].pos=D3DXVECTOR3(d,d,0);
	vbz[1]->Unlock();
	v=NULL;
	//Set up the vertex in the third vertex buffer
	vbz[2]->Lock(0,4*sizeof(SimpleVertex),(BYTE**)&v, 0 );
	v[0].pos=D3DXVECTOR3(0,d,0);
	v[1].pos=D3DXVECTOR3(0,0,0);
	v[3].pos=D3DXVECTOR3(0,0,d);
	v[2].pos=D3DXVECTOR3(0,d,d);
	vbz[2]->Unlock();
	v=NULL;
};

HRESULT Bezier::Initialize(IDirect3DDevice8* pDev)
{
	m_pD3DDev = pDev;
	pDev->AddRef();
    //Inizialize the Mouse UI
	D3DVIEWPORT8    viewport;
	RECT            rect;
    m_pD3DDev->GetViewport(&viewport);
	rect.left   = rect.top = 0;
	rect.bottom = viewport.Height;
	rect.right  = viewport.Width;
	mpMouseUI   = new MouseUI(rect);
    mpMouseUI->SetRotationalSensitivityFactor(1.0f);
	//Inizialize the patch buffers
	CreateBuffers();
	//Load vertex shader of the patch from file bezir.vso
	vector<DWORD> vs;
	vs.push_back(D3DVSD_STREAM(0));
	vs.push_back(D3DVSD_REG(6, D3DVSDT_FLOAT2));
	vs.push_back(D3DVSD_REG(9, D3DVSDT_FLOAT2));
	vs.push_back(D3DVSD_END());
	LoadAndCreateShader(GetFilePath("bezier.vso"), &vs[0], 0, SHADERTYPE_VERTEX, &vshader);
	//Inizialize the box buffers
	CreateBox(1);
	//Load vertex shader of the opened box from file box.vso
	vector<DWORD> vs2;
	vs2.push_back(D3DVSD_STREAM(0));
	vs2.push_back(D3DVSD_REG(0, D3DVSDT_FLOAT3));
	vs2.push_back(D3DVSD_END());
	LoadAndCreateShader(GetFilePath("box.vso"), &vs2[0], 0, SHADERTYPE_VERTEX, &vshaderbox);
	//Set up the colors of the box
	Green=D3DXVECTOR4(0,1,0,1);
	Red=D3DXVECTOR4(1,0,0,1);
	Blue=D3DXVECTOR4(0,0,1,1);
	//Set up the device render state: the cull mode and the zbuffer
	m_pD3DDev->SetRenderState(D3DRS_CULLMODE, D3DCULL_NONE);
    m_pD3DDev->SetRenderState(D3DRS_ZWRITEENABLE, TRUE);
	//Set up the camera poisition and target
	cpos=D3DXVECTOR3(2,2,2);
	tpos=D3DXVECTOR3(0,0,0);
	//Load the texture patch from file texture.bmp
	D3DXCreateTextureFromFileEx(m_pD3DDev, 
			GetFilePath("texture.bmp").c_str(),
			D3DX_DEFAULT,
			D3DX_DEFAULT,
			1,
			0,
			D3DFMT_A8R8G8B8,
			D3DPOOL_MANAGED,
			D3DX_FILTER_LINEAR,
			D3DX_FILTER_LINEAR,
			0,
			NULL,
			NULL,
			&tex);

	return S_OK;
}

HRESULT Bezier::Free()
{
	//Release all the buffers from memory
	SAFE_RELEASE(ib);
	SAFE_RELEASE(vb);
	SAFE_RELEASE(vbz[0]);
	SAFE_RELEASE(vbz[1]);
	SAFE_RELEASE(vbz[2]);
	SAFE_RELEASE(tex);

	if (m_pD3DDev)
	{
		m_pD3DDev->DeleteVertexShader(vshaderbox);
		m_pD3DDev->DeleteVertexShader(vshader);
		vshaderbox = 0;
		vshader = 0;
	}

	SAFE_RELEASE(m_pD3DDev);
	return S_OK;
}

HRESULT Bezier::Start()
{
	return S_OK;
}

HRESULT Bezier::Tick(EBTimer* pTimer)
{
	//Check if the application works in wire mode or doesn't
	if(bWire) m_pD3DDev->SetRenderState(D3DRS_FILLMODE,  D3DFILL_WIREFRAME );
	else m_pD3DDev->SetRenderState(D3DRS_FILLMODE,  D3DFILL_SOLID);
	//Clear Zbuffer and the target surface where is rendered the scene
	m_pD3DDev->Clear(0, NULL, D3DCLEAR_TARGET|D3DCLEAR_ZBUFFER, D3DCOLOR_XRGB(0,0,125), 1.0, 0);
	//Set the 16 bezir control point as vertex shader constants
	m_pD3DDev->SetVertexShaderConstant(0,   &controlpoint[0],  1);
	m_pD3DDev->SetVertexShaderConstant(1,   &controlpoint[1],  1);
	m_pD3DDev->SetVertexShaderConstant(2,   &controlpoint[2],  1);
	m_pD3DDev->SetVertexShaderConstant(3,   &controlpoint[3],  1);
	m_pD3DDev->SetVertexShaderConstant(4,   &controlpoint[4],  1);
	m_pD3DDev->SetVertexShaderConstant(5,   &controlpoint[5],  1);
	m_pD3DDev->SetVertexShaderConstant(6,   &controlpoint[6],  1);
	m_pD3DDev->SetVertexShaderConstant(7,   &controlpoint[7],  1);
	m_pD3DDev->SetVertexShaderConstant(8,   &controlpoint[8],  1);
	m_pD3DDev->SetVertexShaderConstant(9,   &controlpoint[9],  1);
	m_pD3DDev->SetVertexShaderConstant(10,  &controlpoint[10], 1);
	m_pD3DDev->SetVertexShaderConstant(11,  &controlpoint[11], 1);
	m_pD3DDev->SetVertexShaderConstant(12,  &controlpoint[12], 1);
	m_pD3DDev->SetVertexShaderConstant(13,  &controlpoint[13], 1);
	m_pD3DDev->SetVertexShaderConstant(14,  &controlpoint[14], 1);
	m_pD3DDev->SetVertexShaderConstant(15,  &controlpoint[15], 1);
	//Set the lighiting data as vertex shader constants
	if(!bPause)
	{
		rot.x+=float(0.001);
		lightPosition.y=float(sin(rot.x));
		lightPosition.x=float(cos(rot.x));
		lightPosition.z=lightPosition.y*0.5f;
	}
	m_pD3DDev->SetVertexShaderConstant(20,  &lightDiffuse,     1);
	m_pD3DDev->SetVertexShaderConstant(21,  &lightSpecular,    1);
	m_pD3DDev->SetVertexShaderConstant(22,  &lightPosition,    1);
	//Calculate the world view projection matrix
	//First calculate the projection matrix
	D3DXMATRIX project;
	D3DXMatrixPerspectiveFovLH(&project, PI*0.25f, 1.0f, 0.01f, 100.0f);
	//Then the view matrix
	D3DXMATRIX view;
	D3DXMatrixLookAtLH(&view, &cpos,&tpos,&n );
	//Last the world matrix
	D3DXMATRIX world,WorldViewProj;
	D3DXMatrixTranslation(&world,pos.x,pos.y,pos.z);
	//Assembly the world, view, projection matrix in worldviewprojection matrix
	D3DXMatrixMultiply( &WorldViewProj, &world, &view);
	D3DXMatrixMultiply( &WorldViewProj, &WorldViewProj, &project);	
	D3DXMatrixTranspose(&WorldViewProj, &WorldViewProj);
	m_pD3DDev->SetVertexShaderConstant(16, &WorldViewProj, 4);
	//Render the bezir patch using its shader
	m_pD3DDev->SetVertexShader( vshader);
	m_pD3DDev->SetStreamSource( 0, vb, sizeof(TwinUV) );
	m_pD3DDev->SetIndices( ib, 0 );
	if(bTexture)
	{
		m_pD3DDev->SetTextureStageState( 0, D3DTSS_COLOROP,   D3DTOP_MODULATE );
		m_pD3DDev->SetTextureStageState( 0, D3DTSS_COLORARG1, D3DTA_TEXTURE );
		m_pD3DDev->SetTextureStageState( 0, D3DTSS_COLORARG2, D3DTA_DIFFUSE );
		m_pD3DDev->SetTexture(0, tex);

	}
	else m_pD3DDev->SetTexture(0, NULL);
	m_pD3DDev->DrawIndexedPrimitive( D3DPT_TRIANGLELIST,0,nvb,
									 0, nib/3 );
	//Render the opened box using its shader
	m_pD3DDev->SetTexture(0, NULL);
	m_pD3DDev->SetVertexShader(vshaderbox);
	//First box triangles
	m_pD3DDev->SetVertexShaderConstant(20,  &Red,     1);
	m_pD3DDev->SetStreamSource( 0, vbz[0], sizeof(SimpleVertex) );
	m_pD3DDev->DrawPrimitive(D3DPT_TRIANGLESTRIP, 0,2);
	//Second box triangles
	m_pD3DDev->SetVertexShaderConstant(20,  &Green,     1);
	m_pD3DDev->SetStreamSource( 0, vbz[1], sizeof(SimpleVertex) );
	m_pD3DDev->DrawPrimitive(D3DPT_TRIANGLESTRIP, 0,2);
	m_pD3DDev->SetVertexShaderConstant(20,  &Blue,     1);
	//Third box triangles
	m_pD3DDev->SetStreamSource( 0, vbz[2], sizeof(SimpleVertex) );
	m_pD3DDev->DrawPrimitive(D3DPT_TRIANGLESTRIP, 0,2);
	return S_OK;
}

void Bezier::Keyboard(DWORD vKey, UINT nFlags, bool bDown)
{
	if(bDown)
	{
		switch (vKey)
        {
			case VK_ADD:
            {
				//Increase the patch detail level
				res++;
				SAFE_RELEASE(ib);
				SAFE_RELEASE(vb);
				CreateBuffers();
                break;
            }
            case VK_SUBTRACT :
            {
				//Decrease the patch detail level
				if(res>2) res--;
				SAFE_RELEASE(ib);
				SAFE_RELEASE(vb);
				CreateBuffers();
                break;
            }
			case VK_UP:
			{
				//Zoom In
				cpos-=D3DXVECTOR3(0.1f,0.1f,0.1f);
				break;
			}

			case VK_DOWN:
			{
				//Zoom Out
				cpos+=D3DXVECTOR3(0.1f,0.1f,0.1f);
				break;
			}
       
			case 'B' :
            {		
				//Create random control points
				int c=0;	
				for(int x=0;x<4;x++)
				{
					for(int z=0;z<4;z++)
					{
						controlpoint[c].x=x+(rand()%100)*0.02f;
						controlpoint[c].y=rand()%3+(rand()%100)*0.02f;
						controlpoint[c].z=z+(rand()%100)*0.02f;
						c++;
					}
				}
                break;
            }
		};
	}
}


HRESULT Bezier::ConfirmDevice(D3DCAPS8* pCaps, DWORD dwBehavior, D3DFORMAT Format)
{
    // Check if the device supports the Vertex Shaders 1.0
    if(D3DSHADER_VERSION_MAJOR(pCaps->VertexShaderVersion) < 1)
	{
		m_strLastError = "Device doesn't support Dx8 Vertex Shader version 1.0";
		return E_FAIL;
	}
    // Check if the device supports the Vertex Shaders 1.0
    if(D3DSHADER_VERSION_MINOR(pCaps->VertexShaderVersion) < 1)
	{
		m_strLastError = "Device doesn't support Dx8 Vertex Shader version 1.1";
		return E_FAIL;
	}
	return S_OK;
}

void Bezier::MouseMove(HWND hWnd, int x, int y)
{
    mpMouseUI->OnMouseMove(x, y);

}

void Bezier::MouseButton(HWND hWnd, eButtonID button, bool bDown, int x, int y)
{
	if(button == MOUSE_LEFTBUTTON)
	{
		if(bDown)
			mpMouseUI->OnLButtonDown(x, y);
		else
			mpMouseUI->OnLButtonUp(x, y);
	}
}