#ifndef __SHADER_Bezier_H
#define __SHADER_Bezier_H

#include <sys/types.h>
#include <sys/timeb.h>

#include "MouseUI.h"


struct TwinUV
{
	D3DXVECTOR2 t0;
	D3DXVECTOR2 t1;
};

//A vertex structure that reaches only the position
struct SimpleVertex
{
	D3DXVECTOR3 pos;
};

const FLOAT PI=3.14159265358979323846f;

class Bezier : public EBEffect
{
public:

	Bezier();
	~Bezier();
	virtual void    UpdateProperties();
	virtual HRESULT Initialize(IDirect3DDevice8* pDev);
	virtual HRESULT Free();
	virtual HRESULT Start();
	virtual HRESULT Tick(EBTimer* pTimer);
	virtual HRESULT ConfirmDevice(D3DCAPS8* pCaps, DWORD dwBehavior, D3DFORMAT Format);
    virtual void    Keyboard(DWORD dwKey, UINT nFlags, bool bDown);
    virtual void    MouseMove(HWND hWnd, int x, int y);
	virtual void    MouseButton(HWND hWnd, eButtonID Button, bool bDown, int x, int y);
		
private:
    
	//Input Data variables
	bool                    mpKey[256];
    MouseUI					*mpMouseUI;
	
	///////////////////////////////////////
	//Camera data (position, target,
	//and up vector)
	///////////////////////////////////////
	D3DXVECTOR3				cpos,tpos,n;

	///////////////////////////////////////
	//LIGHTING DATA
	///////////////////////////////////////
	D3DXVECTOR4				lightDiffuse;
	D3DXVECTOR4				lightSpecular;
	D3DXVECTOR4				lightPosition;

	///////////////////////////////////////
	//World data
	///////////////////////////////////////
	//Position
	D3DXVECTOR3				pos;
	//Rotation
	D3DXVECTOR3				rot;

	///////////////////////////////////////
	//Patch Bezir data
	///////////////////////////////////////
	//Vertex buffer
	LPDIRECT3DVERTEXBUFFER8 vb;
	//Index buffer
	LPDIRECT3DINDEXBUFFER8  ib;
	//Texture
	LPDIRECT3DTEXTURE8		tex;
	//Number of verticies
	int						nvb;
	//Detail level
	int						res;
	//Number of indicies
	int						nib;
	//Control point
	D3DXVECTOR4				controlpoint[16];
	//Vertex shader
	DWORD					vshader;

	///////////////////////////////////////
	//Opened box data
	///////////////////////////////////////
	//Vertex buffer
	LPDIRECT3DVERTEXBUFFER8	vbz[3];
	//Vertex buffer
	DWORD					vshaderbox;
	//The three colors of the opened box
	D3DXVECTOR4				Green,Red,Blue;
	
	///////////////////////////////////////
	//OPTION DATA
	///////////////////////////////////////
	//Wireframe state
	bool					bWire;
	//Pause state
	bool					bPause;
	//Texture state
	bool					bTexture;

	///////////////////////////////////////
	//Procedure to create buffer for the 
	//patch and the opened box
	///////////////////////////////////////
	//Create patch bezir buffer
	void					CreateBuffers();
	//Create opened box buffer with an input 
	//measure
	void					CreateBox(float d);

};

#endif