#pragma once

#include <memory>
#include "Basic.h"
#include "ParameterSurface.h"

// hair vertex data, used by the vertex shader
struct HairVertex {
	HairVertex(const D3DXVECTOR3 &p, const D3DXVECTOR3 &tx, const D3DXVECTOR3 &ty, const D3DXVECTOR2 &tex, const D3DXVECTOR4 &parm): p(p), tx(tx), ty(ty), tex(tex), parm(parm) {}
	D3DXVECTOR3 p;	// position
	D3DXVECTOR3 tx, ty; // tangents
	D3DXVECTOR2 tex; // texture coordinate
	D3DXVECTOR4 parm; // .x = u parameter value, .y = radius, .z = length
};


// creates a hair patch
struct HairPatch {
	enum ShaderType {
		kUseVertexData,
		kUseHairData,
		kNumberOfTypes
	};

	HairPatch(IDirect3DDevice8 *pD3DDev, ParameterSurface *pSurface, ShaderType shaderType = kUseVertexData);
	~HairPatch();
	
	void setLength(float l) { length = l; }
	float getLength() const { return length; }
	void setRadius(float r) { radius = r; }
	float getRadius() const { return radius; }
	void setLight(const D3DXVECTOR3 &l) { lightDirection = l; }
	const D3DXVECTOR3 &getLight() const { return lightDirection; }

	void setBend(const D3DXVECTOR4 &b) { bend = b; }
	void setGravity(const D3DXVECTOR4 &g) { gravity = g; }
	void setColor(const D3DXVECTOR4 &c) { color = c; }
	void setParameterLimits(const D3DXVECTOR2 &begin, const D3DXVECTOR2 &end) { parameterBegin = begin; parameterEnd = end; }

	int getNHairs() const { return nHairs; }
	int getNSegments() const { return nSegments; }

	void create(int nHairs, int nSegments);
	void free();
	void setTransform(const D3DXMATRIX &matWorld, const D3DXMATRIX &matView, const D3DXMATRIX &matProjection);
	void draw();


private:
	std::auto_ptr<ParameterSurface> pSurface;
	ShaderType shaderType;

	IDirect3DVertexBuffer8 *pVertexBuffer;
	IDirect3DDevice8 *pD3DDev;
	DWORD hairShader;	
	D3DXVECTOR4 color, bend, gravity;
	D3DXVECTOR3 lightDirection;

	int nHairs, nSegments;
	float length, radius;
	D3DXVECTOR2 parameterBegin, parameterEnd;

	D3DXMATRIX matWorldLast;
	bool useLast;
public:
	struct SortData {
		D3DXVECTOR3 pos;
		float depth;
		int vertexIndex;
	};
private:
	std::vector<SortData> sortList;
};