enum SurfaceType{ kSphere, kTorus };
struct ParameterSurface { 
	// s, t : 0..1
	virtual D3DXVECTOR3 pos(float s, float t) = 0;
	virtual D3DXVECTOR3 ds(float s, float t) = 0;
	virtual D3DXVECTOR3 dt(float s, float t) = 0;
};

struct SphereSurface: public ParameterSurface {
	void scale(float &s, float &t) { s = s*D3DX_PI*2 - D3DX_PI; t = t*D3DX_PI - D3DX_PI/2; }
	virtual D3DXVECTOR3 pos(float s, float t) { scale(s, t); return D3DXVECTOR3(cos(s)*cos(t), sin(s)*cos(t), sin(t)); }
	virtual D3DXVECTOR3 ds(float s, float t)  { scale(s, t); return D3DXVECTOR3(-sin(s)*cos(t), cos(s)*cos(t), 0); }
	virtual D3DXVECTOR3 dt(float s, float t)  { scale(s, t); return D3DXVECTOR3(-cos(s)*sin(t), -sin(s)*sin(t), cos(t)); }
};

struct TorusSurface: public ParameterSurface {
	TorusSurface(float d, float r): d(d), r(r) {}
	void scale(float &s, float &t) { s = s*D3DX_PI*2 - D3DX_PI; t = t*D3DX_PI*2 - D3DX_PI; }
	virtual D3DXVECTOR3 pos(float s, float t) { scale(s, t); return -D3DXVECTOR3((d - r*cos(t))*sin(s), r*sin(t), (d - r*cos(t))*cos(s)); }
	virtual D3DXVECTOR3 ds(float s, float t)  { scale(s, t); return -D3DXVECTOR3((d - r*cos(t))*cos(s), 0, -(d - r*cos(t))*sin(s)); }
	virtual D3DXVECTOR3 dt(float s, float t)  { scale(s, t); return -D3DXVECTOR3((r*sin(t))*sin(s), r*cos(t), (r*sin(t))*cos(s)); }
	float d, r;
};

