// -------------------------------------------------------------
// Hair shader effect
// -------------------------------------------------------------

#include "eb_effect.h"
#include "Basic.h"
#include "shaderHair.h"
#include "nvinc.h"
#include "ShaderConstants.h"
#include <time.h>
#include <strstream>

using namespace std;
using namespace nv_objects;

DECLARE_EFFECT_MAIN()

extern "C" {
	__declspec(dllexport) unsigned int GetNumEffects() { return 1; }
	__declspec(dllexport) EBEffect* CreateEffect(unsigned int EffectNum) { return new CShaderHair(kSphere); }
}

CShaderHair::CShaderHair(SurfaceType surfaceType):
	m_pLogoTexture(NULL),
	m_pSphere(NULL),
	m_pTorus(NULL),
	m_bWireframe(false),
	m_bUseBend(true),
	m_bShowTorus(false),
	pHairPatch(NULL),
	m_pUI(NULL),
	m_pNVDevice(NULL),
	surfaceType(surfaceType),
	lightDirection(1, 1, 1)
{
	m_strEffectLocation = "3rd Party\\Stylistic";
	m_strEffectName = "DynamicHair";
	m_strEffectVertexShader = GetFilePath("Hair.nvv");
	m_strEffectPixelShader = "";
	m_strEffectVersion = "1.0";
	D3DXVec3Normalize(&lightDirection, &lightDirection);
}

void CShaderHair::UpdateProperties()
{
	EBEffect::UpdateProperties();
	
	m_pVertexShaderEnum->AddEnumerant(new EBEnumValue(m_pVertexShaderEnum, "Hair", GetFilePath("Hair.nvv"), EBTYPE_STRING_PROP));
	AddProperty(new EBProperty("Wireframe", OBJECT_MEMBER(m_bWireframe), EBTYPE_BOOL_PROP));
	AddProperty(new EBProperty("Animate", OBJECT_MEMBER(m_bUseBend), EBTYPE_BOOL_PROP));
	
	AddProperty(new EBProperty("TriangleCount", OBJECT_MEMBER(m_iTris), EBTYPE_DWORD_PROP));
	AddProperty(new EBProperty("VertexCount", OBJECT_MEMBER(m_iVerts), EBTYPE_DWORD_PROP));

    string aboutText = "file://";
    aboutText += GetFilePath("..\\3rdparty.htm");
    aboutText += "#hair";
	SetAboutInfo( NULL, _T("Hair"), _T(aboutText.c_str()));
}

CShaderHair::~CShaderHair()
{
	Free();	
}


CShaderHair::createHair(int nHairs, int nSegments, float length, float radius) {
	ParameterSurface *pSurface;
	if ( surfaceType == kSphere ) 
		pSurface = new SphereSurface;
	else
		pSurface = new TorusSurface(2, 0.5);
	pHairPatch = auto_ptr<HairPatch>(new HairPatch(m_pD3DDev, pSurface));
	pHairPatch->setColor(D3DXVECTOR4(0, 0, 0, 0.6));
	pHairPatch->setLight(lightDirection);

	pHairPatch->setLength(length);
	pHairPatch->setRadius(radius);
	pHairPatch->create(nHairs, nSegments);

	m_iVerts = (nSegments*2 + 1)*nHairs;
	m_iTris  = ((nSegments*2 + 1) - 2)*nHairs;
}


HRESULT CShaderHair::Initialize(LPDIRECT3DDEVICE8 pDev)
{
	HRESULT hr;
	
	m_pD3DDev = pDev;
	pDev->AddRef();
	m_pNVDevice = new NVDevice(pDev);
	
	//initialize mouse UI
	RECT rect;
	rect.left = rect.top = 0;
	D3DVIEWPORT8 viewport;
	m_pD3DDev->GetViewport(&viewport);
	rect.bottom = viewport.Height;
	rect.right  = viewport.Width;
	m_pUI = new MouseUI((const RECT)rect);
	
	vector<DWORD> Declaration;
	Declaration.push_back(D3DVSD_STREAM(0));
	Declaration.push_back(D3DVSD_REG(0, D3DVSDT_FLOAT3));
	Declaration.push_back(D3DVSD_REG(1, D3DVSDT_FLOAT3));
	Declaration.push_back(D3DVSD_REG(2, D3DVSDT_FLOAT2));
	Declaration.push_back(D3DVSD_END());
	
	
	// Create the logo texture
	hr = D3DXCreateTextureFromFileEx(m_pD3DDev, 
		GetFilePath("b.dds").c_str(),
		D3DX_DEFAULT,
		D3DX_DEFAULT,
		0,
		0,
		D3DFMT_UNKNOWN,
		D3DPOOL_MANAGED,
		D3DX_FILTER_LINEAR,
		D3DX_FILTER_LINEAR,
		0,
		NULL,
		NULL,
		&m_pLogoTexture);
	if (FAILED(hr)) {
		m_strLastError = "Could not create b.dds";
		return hr;
	}
	
	m_pD3DDev->SetRenderState(D3DRS_CULLMODE, D3DCULL_NONE);
	
	m_pD3DDev->SetTexture(0, m_pLogoTexture);
	
    m_pD3DDev->SetRenderState(D3DRS_ALPHABLENDENABLE, TRUE);
    m_pD3DDev->SetRenderState(D3DRS_SRCBLEND, D3DBLEND_SRCALPHA);
    m_pD3DDev->SetRenderState(D3DRS_DESTBLEND, D3DBLEND_INVSRCALPHA);
	
	m_pD3DDev->SetRenderState(D3DRS_ZENABLE, TRUE);
	m_pD3DDev->SetRenderState(D3DRS_ZWRITEENABLE, TRUE);


	{ // set light
		D3DLIGHT8 light;
		ZeroMemory(&light, sizeof(D3DLIGHT8));
		light.Type = D3DLIGHT_DIRECTIONAL;
		light.Diffuse = D3DXCOLOR(1, 1, 1, 1);
		light.Ambient = D3DXCOLOR(.5, .5, .5, 1);
		light.Direction = lightDirection;
		light.Range = 10000;
		m_pD3DDev->SetLight(0, &light);
		m_pD3DDev->LightEnable(0, TRUE);
		m_pD3DDev->SetRenderState(D3DRS_LIGHTING, TRUE);
	}	

	{ // load xfile
		m_pSphere = new NVMesh();
		hr = m_pSphere->Create(m_pNVDevice, GetFilePath("sp.x"));
		if (FAILED(hr)) {
			m_strLastError = "Could not load xfile";
			return hr;
		}
		
		m_pSphere->SetFVF(m_pNVDevice, D3DFVF_XYZ | D3DFVF_NORMAL | D3DFVF_TEX1);
		
		m_pSphere->SetTexture(m_pLogoTexture);
		m_pSphere->RestoreDeviceObjects(m_pNVDevice);
	}
	
	{ // load xfile
		m_pTorus = new NVMesh();
		hr = m_pTorus->Create(m_pNVDevice, GetFilePath("torus.x"));
		if (FAILED(hr)) {
			m_strLastError = "Could not load xfile";
			return hr;
		}
		
		m_pTorus->SetFVF(m_pNVDevice, D3DFVF_XYZ | D3DFVF_NORMAL | D3DFVF_TEX1);
		
		m_pTorus->SetTexture(m_pLogoTexture);
		m_pTorus->RestoreDeviceObjects(m_pNVDevice);
	}
	
	
	createHair(5000, 6, 1.5, 0.08);
	
	return S_OK;
	
}

HRESULT CShaderHair::Free()
{
	SAFE_DELETE(m_pSphere);
	SAFE_DELETE(m_pTorus);
	SAFE_DELETE(m_pNVDevice);
	
	SAFE_RELEASE(m_pLogoTexture);
	SAFE_DELETE(m_pUI);

	SAFE_RELEASE(m_pD3DDev);
	
	return S_OK;
}

HRESULT CShaderHair::Start()
{
	return S_OK;
}

HRESULT CShaderHair::Tick(EBTimer* pTimer)
{
	HRESULT hr = S_OK;
	
	hr = m_pD3DDev->Clear(0, NULL, D3DCLEAR_TARGET|D3DCLEAR_ZBUFFER, D3DCOLOR_XRGB( 0x0, 0x0, 0x0 ), 1.0, 0);
	
	m_pD3DDev->SetRenderState(D3DRS_FILLMODE, m_bWireframe ? D3DFILL_WIREFRAME : D3DFILL_SOLID);
	
	
	// -------------------------------------------------------------
	// Setup transformations
	// -------------------------------------------------------------
	
	D3DXMATRIX matWorld;
    D3DXMATRIX matView;
	D3DXMATRIX matProj;
	{ // Setup the world, view and projection matrices
		D3DXMatrixMultiply( &matWorld, &m_pUI->GetRotationMatrix(), &m_pUI->GetTranslationMatrix() );
		
		D3DXVECTOR3 vEyePt = D3DXVECTOR3( 0.0f, 0.0f, -10.0f );
		D3DXVECTOR3 vLookatPt = D3DXVECTOR3( 0.0f, 0.0f, 0.0f );
		D3DXVECTOR3 vUp = D3DXVECTOR3( 0.0f, 1.0f, 0.0f );
		D3DXMatrixLookAtLH(&matView, &vEyePt, &vLookatPt, &vUp);
		D3DXMatrixPerspectiveFovLH(&matProj, D3DXToRadian(30.0f), 1.0f, 1.0f, 50.0f);
	}
	
	
	{ // draw 
		m_pD3DDev->SetTextureStageState(0, D3DTSS_COLOROP, D3DTOP_MODULATE);
		m_pD3DDev->SetTextureStageState(0, D3DTSS_COLORARG1, D3DTA_TEXTURE);
		m_pD3DDev->SetTextureStageState(0, D3DTSS_COLORARG2, D3DTA_DIFFUSE);
		
		m_pD3DDev->SetTextureStageState(0, D3DTSS_ALPHAOP, D3DTOP_DISABLE);
		m_pD3DDev->SetTextureStageState(0, D3DTSS_ALPHAARG1, D3DTA_TEXTURE);
		m_pD3DDev->SetTextureStageState(0, D3DTSS_ALPHAARG2, D3DTA_DIFFUSE);

		m_pD3DDev->SetRenderState(D3DRS_CULLMODE, D3DCULL_CCW); 

		D3DXMATRIX matScale;
		if ( surfaceType == kSphere ) {
			D3DXMatrixScaling(&matScale, .2, .2, .2);
			D3DXMatrixMultiply(&matScale, &matScale, &matWorld);
			m_pD3DDev->SetTransform(D3DTS_WORLD, &matScale);
		} else {
			m_pD3DDev->SetTransform(D3DTS_WORLD, &matWorld);
		}
		m_pD3DDev->SetTransform(D3DTS_VIEW, &matView);
		m_pD3DDev->SetTransform(D3DTS_PROJECTION, &matProj);
		m_pD3DDev->SetRenderState(D3DRS_NORMALIZENORMALS, TRUE);
		m_pSphere->SetTexture(m_pLogoTexture);
		if ( surfaceType == kSphere ) { // draw sphere
			m_pSphere->Render(m_pNVDevice);
		} else if ( surfaceType == kTorus ) { // draw torus
			m_pTorus->Render(m_pNVDevice);
		}
	}

	{ // draw hairs

		static double a;
		a = float(clock())/CLOCKS_PER_SEC;
		{ // bend 
			static D3DXVECTOR4 bend(0, 0, 0, 0);

			if ( m_bUseBend ) {
				bend.x = cos(a);
				bend.y = sin(a)*cos(a/5);
				bend.z = cos(a)*sin(a);
			}
			pHairPatch->setBend(bend);
		}


		{ // gravity
			static D3DXVECTOR4 gravity(0, 0, 0, 0.01);
			gravity.y = -10;
			gravity.w = 0.001;
			pHairPatch->setGravity(gravity);
		}

		pHairPatch->setTransform(matWorld, matView, matProj);
		pHairPatch->draw();
	}
	return hr;
}

HRESULT CShaderHair::ConfirmDevice(D3DCAPS8* pCaps, DWORD dwBehavior, D3DFORMAT Format)
{
	return S_OK;
}

void CShaderHair::MouseButton(HWND hWnd, eButtonID button, bool bDown, int x, int y)
{
	if ( button == MOUSE_LEFTBUTTON ) {
		if(bDown)
			m_pUI->OnLButtonDown(x, y);
		else
			m_pUI->OnLButtonUp(x, y);
	}
	return;
}

void CShaderHair::MouseMove(HWND hWnd, int x, int y)
{
	m_pUI->OnMouseMove(x, y);
	return;
}

void CShaderHair::Keyboard(DWORD dwKey, UINT nFlags, bool bDown)
{
	eEBKeyAction Action = TranslateEffectKey(dwKey, nFlags, bDown);

	float factor = 1;
	bool create = false;
	int nHairs = pHairPatch->getNHairs(); 
	int nSegments = pHairPatch->getNSegments(); 
	float length = pHairPatch->getLength(); 
	float radius = pHairPatch->getRadius(); 

    switch ( Action ) {
		case EB_HELP: {
			std::strstream help; 
            help << "Home - \tReset To Defaults \n\n ";
            help << "W -    \tWireframe Toggle \n";
            help << "PgUP/PgDown - Change number of hairs\n";
            help << "+/-    \tChange length of hairs\n";
			help << "R,E    \tChange radius\n";
            help << "*//    \tChange number of segments\n";
            help << "P	    \tChange model\n\n";
            help << "Left Button & Mouse - Rotate Object\n";
            help << "Shift Left Button & Mouse - Pan Camera \n";
			help << "Ctrl Left Button & Mouse - Move Camera In & Out\n\n";
			help << "nTris: " << m_iTris << " nVerts: " << m_iVerts << endl;
			help << "nHairs: " << nHairs << " nSegments: " << nSegments << endl;
			help << "length: " << length << " radius: " << radius << ends;

			::MessageBoxEx( NULL, help.str(), "Help", MB_ICONINFORMATION | MB_TASKMODAL, MAKELANGID( LANG_NEUTRAL, SUBLANG_DEFAULT ) );
		}
		break;
		
		case EB_POINTS: {
            m_bShowTorus = !m_bShowTorus;
			surfaceType = m_bShowTorus ? kTorus : kSphere;
			create = true;
        }
		break;
		case EB_WIREFRAME: {
            m_bWireframe = !m_bWireframe;
			m_dwEffectDirtyFlags |= EBEFFECT_DIRTY_PUBLICSTATE;
        }
		break;
		case EB_PAGEUP: 
			factor = 4;
		case EB_PAGEDOWN: {
			factor /= 2;
			nHairs = maximum(int(factor*nHairs), 1); 
			create = true;
		} 
		break;
		case EB_MULTIPLY:
			factor = 4;
		case EB_DIVIDE: {
			factor /= 2;
			nSegments = maximum(int(factor*nSegments), 1); 
			create = true;
		} 
		break;
		case EB_ADD:
			factor = 4;
		case EB_SUBTRACT: {
			factor /= 2;
			length = length*((factor-1)*0.5+1);
			create = true;
		}
		break;
		case EB_RESET: {
			m_pUI->Reset();
			m_bWireframe = false;
			m_dwEffectDirtyFlags |= EBEFFECT_DIRTY_PUBLICSTATE;
		}
		break;
		case EB_UNKNOWN: 
			switch ( dwKey ) {
				case 'R':
					factor = 4;
				case 'E': {
					factor /= 2;
					radius = maximum(radius*((factor-1)*0.1+1), 0.01);
					create = true;
				}
				break;
			}
		break;						 

	}
	if ( create ) {
		createHair(nHairs,nSegments, length, radius);
		m_dwEffectDirtyFlags |= EBEFFECT_DIRTY_PUBLICSTATE;
	}
}

#if defined(_DEBUG) && 0
	struct ConsoleWindow {
		PROCESS_INFORMATION    pi;
		
		ConsoleWindow() { 
			STARTUPINFO si = {0};            // Initialize all members to zero
			si.cb = sizeof(STARTUPINFO);     // Set byte count
			
			AllocConsole();                  // Allocate console window
			freopen("CONOUT$", "a", stderr); // Redirect stderr to console
			freopen("CONOUT$", "a", stdout);
			
			// Display user message in console window.
			CreateProcess(NULL, "cmd.exe", NULL, NULL, TRUE, CREATE_SUSPENDED, NULL, NULL, &si, &pi);              // address of PROCESS_INFORMATION
		}
		
		~ConsoleWindow() {
			TerminateProcess(pi.hProcess, 0); 
			CloseHandle(pi.hProcess); 
		}
		
	};
	ConsoleWindow con;
#endif
