/******************************************************************************

  Copyright (C) 1999, 2000 NVIDIA Corporation
  This file is provided without support, instruction, or implied warranty of any
  kind.  NVIDIA makes no guarantee of its fitness for a particular purpose and is
  not liable under any circumstances for any damages or loss whatsoever arising
  from the use or inability to use this file or items derived from it.
  
    Comments:
    
      
        
******************************************************************************/


#include "eb_effect.h"
#include "nvinc.h"

#include "voxel.hpp"



using namespace std;

DECLARE_EFFECT_MAIN()

// enable testmodes
const TestModeOn = false;

// maximum is limited by the precision of matrix multiply in the pixelshader (testet for 2048 in ReferenceDevice) 
const TargetSizeX = 256;

// maximum is 256 for 8Bit per color target resolution
const TargetSizeY = 256;

// height of a single stack line, maybe 1, but works not ever correct
const LineSizeY = 2;

extern "C"
{
	__declspec(dllexport) unsigned int GetNumEffects() { return (TestModeOn ? 3 : 1); }

	__declspec(dllexport) EBEffect* CreateEffect(unsigned int EffectNum)
	{	
		switch(EffectNum)
		{
			case 0:
				return new CVoxel(true, 10);
				break;
			case 1:
				return new CVoxel(false, 1);
				break;
			case 2:
				return new CVoxel(true, 1);
				break;
			default:
				return NULL;
		}
	}

}


// get untransformed heightfield texture coordinates
float CVoxel::GetHeightfieldLine(int _l)
{
	return((float)(1.0f - 2.0f * _l / TargetSizeY ) );
}

// get coordinates for a line
float CVoxel::GetTriangleLineTop(int _l, bool _invert)
{
	if (_invert)
		return( (float)(-1.0 + 2.0 * (_l+0.0) / (TargetSizeY) ) );
	else
		return( (float)(+1.0 - 2.0 * (_l+0.0) / (TargetSizeY) ) );
}

// get coordinates for a line
float CVoxel::GetTriangleLineBottom(int _l, bool _invert)
{
	if (_invert)
		return( (float)(-1.0 + 2.0 * (_l+1.0) / (TargetSizeY) ) );
	else
		return( (float)(+1.0 - 2.0 * (_l+1.0) / (TargetSizeY) ) );
}

// get texture-coordinates for a line
float CVoxel::GetLine(int _l)
{
	return( (float)((_l - 0.75f) / (TargetSizeY-1)) );
}

// get left texture coordinate
float CVoxel::GetMinPosition()
{
	return(-1.0f - 0.5f/ (float)TargetSizeX);
}

// get right texture coordinate
float CVoxel::GetMaxPosition()
{
	return(+1.0f - 0.5f/ (float)TargetSizeX);
}

// set vertices for copy a line
void CVoxel::SetLineCopy(BlittingVertex* _vertices, int _line, D3DXVECTOR4 _diffuse, bool _invert)
{
	_vertices[0].Position = D3DXVECTOR2(GetMinPosition(), GetTriangleLineTop(_line, _invert));
	_vertices[0].Texture = 0.0f;
	_vertices[0].Diffuse = _diffuse;

	_vertices[1].Position = D3DXVECTOR2(GetMaxPosition(), GetTriangleLineTop(_line, _invert));
	_vertices[1].Texture = 1.0f;
	_vertices[1].Diffuse = _diffuse;

	_vertices[2].Position = D3DXVECTOR2(GetMaxPosition(), GetTriangleLineBottom(_line, _invert));
	_vertices[2].Texture = 1.0f;
	_vertices[2].Diffuse = _diffuse;

	_vertices[3].Position = D3DXVECTOR2(GetMinPosition(), GetTriangleLineBottom(_line, _invert));
	_vertices[3].Texture = 0.0f;
	_vertices[3].Diffuse = _diffuse;

	_vertices[4] = _vertices[0];
	_vertices[5] = _vertices[2];
}


// set vertices for process a line (first step)
void CVoxel::SetProzessLine1(VoxelVertex* _vertices, int _line)
{
	_vertices[0].DestPosition = D3DXVECTOR2(GetMinPosition(), -1.0);
	_vertices[0].StackPointer = 0.0f;
	_vertices[0].IteratedData = D3DXVECTOR4(0.0, 1.0, GetLine(_line-1), GetLine(_line));	
	_vertices[0].HighFieldTexture = D3DXVECTOR2(-1.0, GetHeightfieldLine(_line));

	_vertices[1].DestPosition = D3DXVECTOR2(GetMaxPosition(), -1.0);
	_vertices[1].StackPointer = 1.0f;
	_vertices[1].IteratedData = D3DXVECTOR4(0.0, 1.0, GetLine(_line-1), GetLine(_line));
	_vertices[1].HighFieldTexture = D3DXVECTOR2(+1.0, GetHeightfieldLine(_line));

	_vertices[2].DestPosition = D3DXVECTOR2(GetMaxPosition(), +1.0);
	_vertices[2].StackPointer = 1.0f;
	_vertices[2].IteratedData = D3DXVECTOR4(0.0, 1.0, GetLine(_line-1), GetLine(_line));
	_vertices[2].HighFieldTexture = D3DXVECTOR2(+1.0, GetHeightfieldLine(_line));

	_vertices[3].DestPosition = D3DXVECTOR2(GetMinPosition(), +1.0);
	_vertices[3].StackPointer = 0.0f;
	_vertices[3].IteratedData = D3DXVECTOR4(0.0, 1.0, GetLine(_line-1), GetLine(_line));	
	_vertices[3].HighFieldTexture = D3DXVECTOR2(-1.0, GetHeightfieldLine(_line));

	_vertices[4] = _vertices[0];
	_vertices[5] = _vertices[2];

}

// set vertices for process a line (second step)
void CVoxel::SetProzessLine2(VoxelVertex* _vertices, int _line)
{
	_vertices[0].DestPosition = D3DXVECTOR2(GetMinPosition(), -1.0);
	_vertices[0].StackPointer = 0.0f;
	_vertices[0].IteratedData = D3DXVECTOR4(0.0f, 0.0f, GetLine(_line), GetLine(_line));

	_vertices[1].DestPosition = D3DXVECTOR2(GetMaxPosition(), -1.0);
	_vertices[1].StackPointer = 1.0f;
	_vertices[1].IteratedData = D3DXVECTOR4(0.0f, 1.0f, GetLine(_line), GetLine(_line));

	_vertices[2].DestPosition = D3DXVECTOR2(GetMaxPosition(), +1.0);
	_vertices[2].StackPointer = 1.0f;
	_vertices[2].IteratedData = D3DXVECTOR4(0.0f, 1.0f, GetLine(_line), GetLine(_line));

	_vertices[3].DestPosition = D3DXVECTOR2(GetMinPosition(), +1.0);
	_vertices[3].StackPointer = 0.0f;
	_vertices[3].IteratedData = D3DXVECTOR4(0.0f, 0.0f, GetLine(_line), GetLine(_line));

	_vertices[4] = _vertices[0];
	_vertices[5] = _vertices[2];

}




CVoxel::CVoxel(bool _UsePixelShader, int _trace_steps)
	: m_pVertexBufferVoxel1(NULL),
	m_pVertexBufferVoxel2(NULL),
	m_dwVoxelVertShader1(NULL),
	m_dwVoxelVertShader2(NULL),
	m_dwBlittingVertShader(NULL),
	m_HighField(NULL),
	m_TargetTexture1(NULL),
	m_dwVoxelPixShader1(NULL),
	m_dwVoxelPixShader2(NULL),
	m_pVertexBufferCopyLine1(NULL),
	m_pVertexBufferCopyLine1_1(NULL),
	m_pVertexBufferCopyLine2(NULL),
	m_LeftMouseDown(false),
	m_UsePixelShader(_UsePixelShader),
	m_LineTexture1(NULL),
	m_LineTexture2(NULL),
	m_trace_steps(_trace_steps)

{

	m_strEffectLocation = "3rd Party\\Imposters";


	if (m_UsePixelShader)
	{
		if (m_trace_steps==1)
			m_strEffectName = "HeightField-Step1";
		else 
			m_strEffectName = "HeightField";
	}
	else
	{
		m_strEffectName = "HeightField-Test";
	}
	m_strEffectVersion = "1.0";
	
}

void CVoxel::UpdateProperties()
{
	EBEffect::UpdateProperties();

	m_pVertexShaderEnum->AddEnumerant(new EBEnumValue(m_pVertexShaderEnum, "Voxel", GetFilePath("VoxelVert.nvv"), EBTYPE_STRING_PROP));

	
	if (m_UsePixelShader)
	{
		m_pVertexShaderEnum->AddEnumerant(new EBEnumValue(m_pVertexShaderEnum, "Voxel2", GetFilePath("VoxelVert2.nvv"), EBTYPE_STRING_PROP));

		m_pPixelShaderEnum->AddEnumerant(new EBEnumValue(m_pPixelShaderEnum, "Voxel", GetFilePath("VoxelPix.nvp"), EBTYPE_STRING_PROP));
		m_pPixelShaderEnum->AddEnumerant(new EBEnumValue(m_pPixelShaderEnum, "Voxel2", GetFilePath("VoxelPix2.nvp"), EBTYPE_STRING_PROP));

	}
    string aboutText = "file://";
    aboutText += GetFilePath("..\\..\\..\\NVEffectsExplained.htm");
    aboutText += "#heightfield";
	SetAboutInfo( NULL, _T("Height Field"), _T(aboutText.c_str()));


}

HRESULT CVoxel::ConfirmDevice(D3DCAPS8* pCaps, DWORD dwBehavior, D3DFORMAT Format)
{
	if (m_UsePixelShader)
	{
		if(D3DSHADER_VERSION_MAJOR(pCaps->PixelShaderVersion) < 1.0)
		{
			m_strLastError = "Device does not support pixel shaders!";
			return E_FAIL;
		}
	}



	return S_OK;
}

CVoxel::~CVoxel()
{
	Free();	
}


inline DWORD F2DW( FLOAT f ) { return *((DWORD*)&f); }


// creates all we need
HRESULT CVoxel::Initialize(LPDIRECT3DDEVICE8 pDev)
{
	HRESULT hr;

	m_pD3DDev = pDev;
	pDev->AddRef();

	m_alpha = -0.4f;
	m_height = 0.2f;
	m_scaling = 0.63f;
	m_centerx = 0.0f;
	m_centery = 0.0f;

	vector<DWORD> DeclarationVoxel;
	DeclarationVoxel.push_back(D3DVSD_STREAM(0));
	DeclarationVoxel.push_back(D3DVSD_REG(0, D3DVSDT_FLOAT2));
	DeclarationVoxel.push_back(D3DVSD_REG(1, D3DVSDT_FLOAT1));
	DeclarationVoxel.push_back(D3DVSD_REG(2, D3DVSDT_FLOAT4));
	DeclarationVoxel.push_back(D3DVSD_REG(3, D3DVSDT_FLOAT2));
	DeclarationVoxel.push_back(D3DVSD_END());

	vector<DWORD> DeclarationBlitting;
	DeclarationBlitting.push_back(D3DVSD_STREAM(0));
	DeclarationBlitting.push_back(D3DVSD_REG(0, D3DVSDT_FLOAT2));
	DeclarationBlitting.push_back(D3DVSD_REG(1, D3DVSDT_FLOAT1));
	DeclarationBlitting.push_back(D3DVSD_REG(2, D3DVSDT_FLOAT4));
	DeclarationBlitting.push_back(D3DVSD_END());

	hr = D3DXCreateTextureFromFileEx(m_pD3DDev, 
		GetFilePath("highField.tga").c_str(),
		D3DX_DEFAULT,
		D3DX_DEFAULT,
		0,
		0,
		D3DFMT_UNKNOWN,
		D3DPOOL_MANAGED,
		D3DX_FILTER_LINEAR,
		D3DX_FILTER_LINEAR,
		0,
		NULL,
		NULL,
		&m_HighField);

	if (FAILED(hr))
	{
		m_strLastError = "Could not create highField.tga";
		return hr;
	}

	

	hr = LoadAndCreateShader(GetFilePath("VoxelVert.vso"), &DeclarationVoxel[0], 0, SHADERTYPE_VERTEX, &m_dwVoxelVertShader1);
	if (FAILED(hr))
		return hr;

	hr = LoadAndCreateShader(GetFilePath("VoxelVert2.vso"), &DeclarationVoxel[0], 0, SHADERTYPE_VERTEX, &m_dwVoxelVertShader2);
	if (FAILED(hr))
		return hr;


	hr = LoadAndCreateShader(GetFilePath("CopyVert.vso"), &DeclarationBlitting[0], 0, SHADERTYPE_VERTEX, &m_dwBlittingVertShader);
	if (FAILED(hr))
		return hr;


	if (m_UsePixelShader)
	{
		hr = LoadAndCreateShader(GetFilePath("VoxelPix.pso"), NULL, 0, SHADERTYPE_PIXEL, &m_dwVoxelPixShader1);
		if (FAILED(hr))
			return hr;

		hr = LoadAndCreateShader(GetFilePath("VoxelPix2.pso"), NULL, 0, SHADERTYPE_PIXEL, &m_dwVoxelPixShader2);
		if (FAILED(hr))
			return hr;
	}


	hr = m_pD3DDev->CreateVertexBuffer( 6 * TargetSizeY * sizeof(VoxelVertex), D3DUSAGE_POINTS | D3DUSAGE_WRITEONLY, 0, D3DPOOL_DEFAULT, &m_pVertexBufferVoxel1);
	if (FAILED(hr))
		return hr;

	hr = m_pD3DDev->CreateVertexBuffer( 6 * TargetSizeY * sizeof(BlittingVertex), D3DUSAGE_POINTS | D3DUSAGE_WRITEONLY, 0, D3DPOOL_DEFAULT, &m_pVertexBufferCopyLine1);
	if (FAILED(hr))
		return hr;

	hr = m_pD3DDev->CreateVertexBuffer( 6 * TargetSizeY * sizeof(BlittingVertex), D3DUSAGE_POINTS | D3DUSAGE_WRITEONLY, 0, D3DPOOL_DEFAULT, &m_pVertexBufferCopyLine1_1);
	if (FAILED(hr))
		return hr;

	
	VoxelVertex* pBuff0;
	hr = m_pVertexBufferVoxel1->Lock(0, 6 * TargetSizeY * sizeof(VoxelVertex),(BYTE**)&pBuff0, 0);
	if (FAILED(hr))
	{
		m_strLastError = "Couldn't lock buffer!";
		return hr;
	}

	BlittingVertex* pBuffCopy0;
	hr = m_pVertexBufferCopyLine1->Lock(0, 6 * TargetSizeY * sizeof(BlittingVertex),(BYTE**)&pBuffCopy0, 0);
	if (FAILED(hr))
	{
		m_strLastError = "Couldn't lock buffer!";
		return hr;
	}

	BlittingVertex* pBuffCopy1;
	hr = m_pVertexBufferCopyLine1_1->Lock(0, 6 * TargetSizeY * sizeof(BlittingVertex),(BYTE**)&pBuffCopy1, 0);
	if (FAILED(hr))
	{
		m_strLastError = "Couldn't lock buffer!";
		return hr;
	}



	for (int i=0; i<TargetSizeY; i++)
	{
		SetProzessLine1(pBuff0 + i*6, i);
		SetLineCopy(pBuffCopy0 + i*6, i, D3DXVECTOR4(1.0, 1.0, 1.0, 1.0));
		SetLineCopy(pBuffCopy1 + i*6, i, D3DXVECTOR4(1.0, 0.0, 0.0, 0.0), true);

	}


	m_pVertexBufferVoxel1->Unlock();
	m_pVertexBufferCopyLine1->Unlock();
	m_pVertexBufferCopyLine1_1->Unlock();





	hr = m_pD3DDev->CreateVertexBuffer( 6 * TargetSizeY * sizeof(VoxelVertex), D3DUSAGE_POINTS | D3DUSAGE_WRITEONLY, 0, D3DPOOL_DEFAULT, &m_pVertexBufferVoxel2);
	if (FAILED(hr))
		return hr;

	hr = m_pD3DDev->CreateVertexBuffer( 6 * TargetSizeY * sizeof(BlittingVertex), D3DUSAGE_POINTS | D3DUSAGE_WRITEONLY, 0, D3DPOOL_DEFAULT, &m_pVertexBufferCopyLine2);
	if (FAILED(hr))
		return hr;



	VoxelVertex* pBuff2;
	hr = m_pVertexBufferVoxel2->Lock(0, 6 * TargetSizeY * sizeof(VoxelVertex),(BYTE**)&pBuff2, 0);
	if (FAILED(hr))
	{
		m_strLastError = "Couldn't lock buffer!";
		return hr;
	}

	BlittingVertex* pBuffCopy2;
	hr = m_pVertexBufferCopyLine2->Lock(0, 6 * TargetSizeY * sizeof(BlittingVertex),(BYTE**)&pBuffCopy2, 0);
	if (FAILED(hr))
	{
		m_strLastError = "Couldn't lock buffer!";
		return hr;
	}


	for (i=0; i<TargetSizeY; i++)
	{
		int j= TargetSizeY-1-i;

		SetProzessLine2(pBuff2 + i*6, j);
		SetLineCopy(pBuffCopy2 + i*6, j, D3DXVECTOR4(1.0, 0.0, 0.0, 0.0), true);
	}
			
	m_pVertexBufferVoxel2->Unlock();
	m_pVertexBufferCopyLine2->Unlock();





	

	hr = m_pD3DDev->CreateTexture(TargetSizeX, TargetSizeY, 1, D3DUSAGE_RENDERTARGET , D3DFMT_A8R8G8B8, D3DPOOL_DEFAULT, &m_TargetTexture1); 
	if (FAILED(hr))
		return hr;

	
	hr = m_pD3DDev->CreateTexture(TargetSizeX, LineSizeY, 1, D3DUSAGE_RENDERTARGET , D3DFMT_A8R8G8B8, D3DPOOL_DEFAULT, &m_LineTexture1); 
	if (FAILED(hr))
		return hr;

	hr = m_pD3DDev->CreateTexture(TargetSizeX, LineSizeY, 1, D3DUSAGE_RENDERTARGET , D3DFMT_A8R8G8B8, D3DPOOL_DEFAULT, &m_LineTexture2); 
	if (FAILED(hr))
		return hr;

	return S_OK;

}

// release all resocurces
HRESULT CVoxel::Free()
{

	SAFE_RELEASE(m_pVertexBufferVoxel1);
	SAFE_RELEASE(m_pVertexBufferVoxel2);

	SAFE_RELEASE(m_pVertexBufferCopyLine1);
	SAFE_RELEASE(m_pVertexBufferCopyLine1_1);
	SAFE_RELEASE(m_pVertexBufferCopyLine2);

	SAFE_RELEASE(m_HighField);
	SAFE_RELEASE(m_TargetTexture1);
	SAFE_RELEASE(m_LineTexture2);
	SAFE_RELEASE(m_LineTexture1);
	

	if (m_pD3DDev)
	{
		if (m_dwVoxelVertShader1)
			m_pD3DDev->DeleteVertexShader(m_dwVoxelVertShader1);
	
		if (m_dwVoxelVertShader2)
			m_pD3DDev->DeleteVertexShader(m_dwVoxelVertShader2);

		if (m_dwVoxelPixShader1)
			m_pD3DDev->DeletePixelShader(m_dwVoxelPixShader1);

		if (m_dwVoxelPixShader2)
			m_pD3DDev->DeletePixelShader(m_dwVoxelPixShader2);


		if (m_dwBlittingVertShader)
			m_pD3DDev->DeleteVertexShader(m_dwBlittingVertShader);

		SAFE_RELEASE(m_pD3DDev);
	}

	return S_OK;
}

HRESULT CVoxel::Start()
{
	return S_OK;
}

// copy a line (2 triangles) from one buffer to another
HRESULT CVoxel::CopyLine
(
	IDirect3DSurface8*			_pColor, 
	IDirect3DSurface8*			_pDepth, 
	LPDIRECT3DTEXTURE8			_texture, 
	DWORD						_VertexShader,
	LPDIRECT3DVERTEXBUFFER8		_vertexbuffer,
	int							_index,
	int							_vertexSize,
	CopyModes					_modes
)
{
	HRESULT hr = S_OK;

	hr = m_pD3DDev->SetRenderTarget(_pColor, _pDepth);
	if (FAILED(hr))
		return hr;
	

	m_pD3DDev->SetTexture(0, _texture);
	m_pD3DDev->SetTexture(1, NULL);
	m_pD3DDev->SetTexture(2, NULL);
	m_pD3DDev->SetTexture(3, NULL);

	if (_modes & DIFFUSE)
	{
		m_pD3DDev->SetTextureStageState(0, D3DTSS_COLOROP ,  D3DTOP_MODULATE);
		m_pD3DDev->SetTextureStageState(0, D3DTSS_COLORARG2, D3DTA_TEXTURE ); 
		m_pD3DDev->SetTextureStageState(0, D3DTSS_COLORARG1, D3DTA_DIFFUSE ); 
		m_pD3DDev->SetTextureStageState(0, D3DTSS_ALPHAOP ,  D3DTOP_MODULATE);
		m_pD3DDev->SetTextureStageState(0, D3DTSS_ALPHAARG2, D3DTA_TEXTURE ); 
		m_pD3DDev->SetTextureStageState(0, D3DTSS_ALPHAARG1, D3DTA_DIFFUSE ); 
	}
	else
	{
		m_pD3DDev->SetTextureStageState(0, D3DTSS_COLOROP ,  D3DTOP_SELECTARG1);
		m_pD3DDev->SetTextureStageState(0, D3DTSS_COLORARG1, D3DTA_TEXTURE ); 
		m_pD3DDev->SetTextureStageState(0, D3DTSS_ALPHAOP ,  D3DTOP_SELECTARG1);
		m_pD3DDev->SetTextureStageState(0, D3DTSS_ALPHAARG1, D3DTA_TEXTURE ); 
	}

	if (_modes & FILTER)
	{
		m_pD3DDev->SetTextureStageState(0, D3DTSS_MINFILTER, D3DTEXF_LINEAR);
		m_pD3DDev->SetTextureStageState(0, D3DTSS_MAGFILTER, D3DTEXF_LINEAR);
		m_pD3DDev->SetTextureStageState(0, D3DTSS_MIPFILTER, D3DTEXF_NONE);
	}
	else
	{
		m_pD3DDev->SetTextureStageState(0, D3DTSS_MINFILTER, D3DTEXF_POINT);
		m_pD3DDev->SetTextureStageState(0, D3DTSS_MAGFILTER, D3DTEXF_POINT);
		m_pD3DDev->SetTextureStageState(0, D3DTSS_MIPFILTER, D3DTEXF_NONE);
	}

	if (_modes & CLAMP)
	{
		m_pD3DDev->SetTextureStageState(0, D3DTSS_ADDRESSU , D3DTADDRESS_CLAMP);
		m_pD3DDev->SetTextureStageState(0, D3DTSS_ADDRESSV , D3DTADDRESS_CLAMP);
	}
	else
	{
		m_pD3DDev->SetTextureStageState(0, D3DTSS_ADDRESSU , D3DTADDRESS_WRAP);
		m_pD3DDev->SetTextureStageState(0, D3DTSS_ADDRESSV , D3DTADDRESS_WRAP);
	}

	m_pD3DDev->SetTextureStageState(1, D3DTSS_COLOROP , D3DTOP_DISABLE);


	m_pD3DDev->SetPixelShader(NULL);

	m_pD3DDev->SetVertexShader(_VertexShader);

	hr = m_pD3DDev->SetStreamSource(0, _vertexbuffer, _vertexSize);
	if (FAILED(hr))
		return hr;

	hr = m_pD3DDev->DrawPrimitive(D3DPT_TRIANGLELIST, _index*6, 2);
	if (FAILED(hr))
		return hr;

	return(hr);

}


// process a line (2 triangles)
HRESULT CVoxel::ProzessLine
(
	IDirect3DSurface8*			_pColor, 
	LPDIRECT3DTEXTURE8			_texture0, 
	LPDIRECT3DTEXTURE8			_texture1, 
	LPDIRECT3DTEXTURE8			_texture2, 
	LPDIRECT3DTEXTURE8			_texture3, 
	DWORD						_VertexShader, 
	DWORD						_Pixelshader,
	LPDIRECT3DVERTEXBUFFER8		_vertexbuffer,
	int							_index,
	CopyModes					_Texture0Modes
)
{
	HRESULT hr = S_OK;

	hr = m_pD3DDev->SetRenderTarget(_pColor, NULL);
	if (FAILED(hr))
		return hr;
	

	m_pD3DDev->SetTexture(0, _texture0);
	m_pD3DDev->SetTexture(1, _texture1);
	m_pD3DDev->SetTexture(2, _texture2);
	m_pD3DDev->SetTexture(3, _texture3);

	if (_Texture0Modes & FILTER)
	{
		m_pD3DDev->SetTextureStageState(0, D3DTSS_MINFILTER, D3DTEXF_LINEAR);
		m_pD3DDev->SetTextureStageState(0, D3DTSS_MAGFILTER, D3DTEXF_LINEAR);
		m_pD3DDev->SetTextureStageState(0, D3DTSS_MIPFILTER, D3DTEXF_NONE);
	}
	else
	{
		m_pD3DDev->SetTextureStageState(0, D3DTSS_MINFILTER, D3DTEXF_POINT);
		m_pD3DDev->SetTextureStageState(0, D3DTSS_MAGFILTER, D3DTEXF_POINT);
		m_pD3DDev->SetTextureStageState(0, D3DTSS_MIPFILTER, D3DTEXF_NONE);
	}

	if (_Texture0Modes & CLAMP)
	{
		m_pD3DDev->SetTextureStageState(0, D3DTSS_ADDRESSU , D3DTADDRESS_CLAMP);
		m_pD3DDev->SetTextureStageState(0, D3DTSS_ADDRESSV , D3DTADDRESS_CLAMP);
	}
	else
	{
		m_pD3DDev->SetTextureStageState(0, D3DTSS_ADDRESSU , D3DTADDRESS_WRAP);
		m_pD3DDev->SetTextureStageState(0, D3DTSS_ADDRESSV , D3DTADDRESS_WRAP);
	}

	m_pD3DDev->SetTextureStageState(1, D3DTSS_MINFILTER, D3DTEXF_POINT);
	m_pD3DDev->SetTextureStageState(1, D3DTSS_MAGFILTER, D3DTEXF_POINT);
	m_pD3DDev->SetTextureStageState(1, D3DTSS_MIPFILTER, D3DTEXF_NONE);
	m_pD3DDev->SetTextureStageState(1, D3DTSS_ADDRESSU , D3DTADDRESS_CLAMP);
	m_pD3DDev->SetTextureStageState(1, D3DTSS_ADDRESSV , D3DTADDRESS_CLAMP);

	m_pD3DDev->SetTextureStageState(2, D3DTSS_MINFILTER, D3DTEXF_POINT);
	m_pD3DDev->SetTextureStageState(2, D3DTSS_MAGFILTER, D3DTEXF_POINT);
	m_pD3DDev->SetTextureStageState(2, D3DTSS_MIPFILTER, D3DTEXF_NONE);
	m_pD3DDev->SetTextureStageState(2, D3DTSS_ADDRESSU , D3DTADDRESS_CLAMP);
	m_pD3DDev->SetTextureStageState(2, D3DTSS_ADDRESSV , D3DTADDRESS_CLAMP);

	m_pD3DDev->SetTextureStageState(3, D3DTSS_MINFILTER, D3DTEXF_POINT);
	m_pD3DDev->SetTextureStageState(3, D3DTSS_MAGFILTER, D3DTEXF_POINT);
	m_pD3DDev->SetTextureStageState(3, D3DTSS_MIPFILTER, D3DTEXF_NONE);
	m_pD3DDev->SetTextureStageState(3, D3DTSS_ADDRESSU , D3DTADDRESS_CLAMP);
	m_pD3DDev->SetTextureStageState(3, D3DTSS_ADDRESSV , D3DTADDRESS_CLAMP);


	m_pD3DDev->SetPixelShader(_Pixelshader);

	m_pD3DDev->SetVertexShader(_VertexShader);

	hr = m_pD3DDev->SetStreamSource(0, _vertexbuffer, sizeof(VoxelVertex));
	if (FAILED(hr))
		return hr;

	hr = m_pD3DDev->DrawPrimitive(D3DPT_TRIANGLELIST, _index*6, 2);
	if (FAILED(hr))
		return hr;

	return(hr);

}



HRESULT CVoxel::Tick(EBTimer* pTimer)
{
	HRESULT hr = S_OK;
	D3DXMATRIX rot;
	D3DXMATRIX trans;
	D3DXMATRIX scale;
	D3DXMATRIX texture;
	D3DXMATRIX matProj;
	D3DXVECTOR4	temp_vec;
	IDirect3DSurface8* pTargetSurface1;
	IDirect3DSurface8* pLineSurface1;
	IDirect3DSurface8* pLineSurface2;
	IDirect3DSurface8* pBackbufferColor; 
    IDirect3DSurface8* pBackbufferDepth; 



	// get surfaces

	hr = m_TargetTexture1->GetSurfaceLevel(0, &pTargetSurface1);
	if (FAILED(hr))
		return hr;

	hr = m_LineTexture1->GetSurfaceLevel(0, &pLineSurface1);
	if (FAILED(hr))
		return hr;

	hr = m_LineTexture2->GetSurfaceLevel(0, &pLineSurface2);
	if (FAILED(hr))
		return hr;

	// get a pointer to the current back-buffer (so we can restore it later)
	m_pD3DDev->GetRenderTarget(        &pBackbufferColor );
	m_pD3DDev->GetDepthStencilSurface( &pBackbufferDepth );




	// set matrix for highfield texture
	D3DXMatrixRotationZ(&rot, m_alpha); 
	D3DXMatrixScaling(&scale, m_scaling, 2.0f * m_scaling, 1.0); 
	D3DXMatrixTranslation(&trans, m_centerx, m_centery, 0.0); 
	D3DXMatrixMultiply(&texture, &scale, &rot);
	D3DXMatrixMultiply(&texture, &texture, &trans);
	D3DXMatrixTranspose(&texture, &texture);
	m_pD3DDev->SetVertexShaderConstant(0, &texture(0, 0), 4);

	// set matrix for generate stack lookpup
	temp_vec = D3DXVECTOR4(0.0, 1.0, 0.0, 0.0);
	m_pD3DDev->SetVertexShaderConstant(10, &temp_vec, 1);
	temp_vec = D3DXVECTOR4(0.0, 0.0, 1.0, 0.0);
	m_pD3DDev->SetVertexShaderConstant(11, &temp_vec, 1);


	// set height factor and color mask
	temp_vec = D3DXVECTOR4(1.0, 0.0, 0.0, m_height);
	m_pD3DDev->SetPixelShaderConstant(0, &temp_vec, 1);


	// set general render states
	m_pD3DDev->SetRenderState(D3DRS_FILLMODE, D3DFILL_SOLID);
	m_pD3DDev->SetRenderState(D3DRS_CULLMODE, D3DCULL_NONE);
	m_pD3DDev->SetRenderState(D3DRS_ZFUNC, D3DCMP_ALWAYS); 
	m_pD3DDev->SetRenderState(D3DRS_ALPHABLENDENABLE, FALSE);
	m_pD3DDev->SetRenderState(D3DRS_ALPHATESTENABLE, FALSE);


	// clear stack texture
	hr = m_pD3DDev->SetRenderTarget(pLineSurface2, NULL);
	if (FAILED(hr))
		return hr;
	hr = m_pD3DDev->Clear(0, NULL, D3DCLEAR_TARGET, 0x0, 0.0, 0);


	// process first step
	for (int i=0; i<TargetSizeY; i++)
	{
		// if no pixel shader copy only
		if (m_UsePixelShader==false)
		{
			if ((i&1)==0)
				CopyLine(pLineSurface1, NULL, m_HighField, m_dwVoxelVertShader1, m_pVertexBufferVoxel1, i, sizeof(VoxelVertex), FILTER);
			else
				CopyLine(pLineSurface2, NULL, m_HighField, m_dwVoxelVertShader1, m_pVertexBufferVoxel1, i, sizeof(VoxelVertex), FILTER);
		}
		else
		{
			// process a line with last rendered line as input
			if ((i&1)==0)
			{
				ProzessLine(pLineSurface1, m_HighField, m_LineTexture2, NULL, NULL,
							m_dwVoxelVertShader1, m_dwVoxelPixShader1, m_pVertexBufferVoxel1, i, FILTER);
			}
			else
			{
				ProzessLine(pLineSurface2, m_HighField, m_LineTexture1, NULL, NULL,
							m_dwVoxelVertShader1, m_dwVoxelPixShader1, m_pVertexBufferVoxel1, i, FILTER);
			}	
		}


		// copy rendered line to target
		if (m_trace_steps > 1)
		{
			if ((i&1)==0)
				CopyLine(pTargetSurface1, NULL, m_LineTexture1, m_dwBlittingVertShader, m_pVertexBufferCopyLine1, i, sizeof(BlittingVertex));
			else
				CopyLine(pTargetSurface1, NULL, m_LineTexture2, m_dwBlittingVertShader, m_pVertexBufferCopyLine1, i, sizeof(BlittingVertex));
		}
		else
		{
			if ((i&1)==0)
				CopyLine(pBackbufferColor, pBackbufferDepth, m_LineTexture1, m_dwBlittingVertShader, m_pVertexBufferCopyLine1_1, i, sizeof(BlittingVertex));
			else
				CopyLine(pBackbufferColor, pBackbufferDepth, m_LineTexture2, m_dwBlittingVertShader, m_pVertexBufferCopyLine1_1, i, sizeof(BlittingVertex));

		}

	}



	if ((m_trace_steps <= 1) || (m_UsePixelShader==false))
		goto ende;

	// clear stack texture
	hr = m_pD3DDev->SetRenderTarget(pLineSurface1, NULL);
	if (FAILED(hr))
		return hr;
	hr = m_pD3DDev->Clear(0, NULL, D3DCLEAR_TARGET, D3DCOLOR_RGBA(255,255,255,255), 0.0, 0);

	// process second step
	for (i=0; i<TargetSizeY; i++)
	{
		// process a line with last rendered line as input
		if ((i&1)==0)
		{
			ProzessLine(pLineSurface1, m_LineTexture2, m_LineTexture2, NULL, m_TargetTexture1,
						m_dwVoxelVertShader2, m_dwVoxelPixShader2, m_pVertexBufferVoxel2, i, CLAMP);
		}
		else
		{
			ProzessLine(pLineSurface2, m_LineTexture1, m_LineTexture1, NULL, m_TargetTexture1,
						m_dwVoxelVertShader2, m_dwVoxelPixShader2, m_pVertexBufferVoxel2, i, CLAMP);
		}	

		// copy rendered line to target
		if ((i&1)==0)
			CopyLine(pBackbufferColor, pBackbufferDepth, m_LineTexture1, m_dwBlittingVertShader, m_pVertexBufferCopyLine2, i, sizeof(BlittingVertex));
		else
			CopyLine(pBackbufferColor, pBackbufferDepth, m_LineTexture2, m_dwBlittingVertShader, m_pVertexBufferCopyLine2, i, sizeof(BlittingVertex));
	}

	// clear allocated resources
ende:
	SAFE_RELEASE(pTargetSurface1);
	SAFE_RELEASE(pLineSurface1);
	SAFE_RELEASE(pLineSurface2);
	SAFE_RELEASE(pBackbufferColor);
	SAFE_RELEASE(pBackbufferDepth);



	return (hr);
}




void CVoxel::MouseButton(HWND hWnd, eButtonID button, bool bDown, int x, int y)
{
	if(button == MOUSE_LEFTBUTTON)
	{
		if(bDown)
			m_LeftMouseDown = true;
		else
			m_LeftMouseDown = false;
	}
	return;
}

void CVoxel::MouseMove(HWND hWnd, int x, int y)
{
float	dx,dy;

	dx = x - m_LastMouseX;
	dy = y - m_LastMouseY;

	m_LastMouseX = x;
	m_LastMouseY = y;

	if ( m_LeftMouseDown )
	{ 
		bool bShift = ( ( GetAsyncKeyState( VK_LSHIFT )   < 0 ) || ( GetAsyncKeyState( VK_SHIFT    ) < 0 ) || ( GetAsyncKeyState( VK_RSHIFT )  < 0 ) );
		bool bAlt =   ( ( GetAsyncKeyState( VK_LMENU )    < 0 ) || ( GetAsyncKeyState( VK_RMENU    ) < 0 ) || ( GetAsyncKeyState( VK_MENU )    < 0 ) );
		bool bCtl =   ( ( GetAsyncKeyState( VK_LCONTROL ) < 0 ) || ( GetAsyncKeyState( VK_RCONTROL ) < 0 ) || ( GetAsyncKeyState( VK_CONTROL ) < 0 ) );

		if (bShift)
		{
			m_height -= dy * 0.001f;
			if (m_height <= 0.0f)
				m_height = 0.0f;
			if (m_height >= 1.0f)
				m_height = 1.0f;
		}
		else if (bCtl)
		{
			m_scaling *= exp( dy * 0.003f);
		}
		else if (bAlt)
		{
			float s = sin(m_alpha);
			float c = cos(m_alpha);
			m_centerx -= (dx*c - dy*s) * m_scaling * 0.003;
			m_centery -= (dx*s + dy*c) * m_scaling * 0.003;
		}
		else
		{
			m_alpha += dx*0.01f;
		}
	}
	
}

void CVoxel::Keyboard(DWORD dwKey, UINT nFlags, bool bDown)
{
	eEBKeyAction Action = TranslateEffectKey(dwKey, nFlags, bDown);
	
    switch ( Action )
    {
		case EB_HELP:
		{
			::MessageBoxEx( NULL, " Help : F1 - Help \n\nLeft Button & Mouse - Rotate Object\n\nShift Left Button & Mouse - change high-scaling\n\n Ctrl Left Button & Mouse - change landscape scaling\n\n Alt Left Button & Mouse - move landscape center\n\n",
				"Help", MB_ICONINFORMATION | MB_TASKMODAL, MAKELANGID( LANG_NEUTRAL, SUBLANG_DEFAULT ) );
		}
		break;


        default :
            break;
    }
	
}


