/******************************************************************************

  Copyright (C) 1999, 2000 NVIDIA Corporation
  This file is provided without support, instruction, or implied warranty of any
  kind.  NVIDIA makes no guarantee of its fitness for a particular purpose and is
  not liable under any circumstances for any damages or loss whatsoever arising
  from the use or inability to use this file or items derived from it.
  
    Comments:
    
      
        
******************************************************************************/

#ifndef __SHADER_FLYING_H
#define __SHADER_FLYING_H
#include "MouseUI.h"
#include <string>


struct VoxelVertex
{
	D3DXVECTOR2	DestPosition;
	FLOAT		StackPointer;
	D3DXVECTOR4	IteratedData;
	D3DXVECTOR2	HighFieldTexture;
};


struct BlittingVertex
{
	D3DXVECTOR2		Position;
	FLOAT			Texture;
	D3DXVECTOR4		Diffuse;
};


class CVoxel : public EBEffect
{
public:
	CVoxel(bool _UsePixelShader, int _trace_steps);
	~CVoxel();
	virtual void UpdateProperties();
	virtual HRESULT Initialize(LPDIRECT3DDEVICE8 pDev);
	virtual HRESULT Free();
	virtual HRESULT Start();
	virtual HRESULT Tick(EBTimer* pTimer);
	virtual HRESULT ConfirmDevice(D3DCAPS8* pCaps, DWORD dwBehavior, D3DFORMAT Format);
	virtual void Keyboard(DWORD dwKey, UINT nFlags, bool bDown);
	virtual void MouseButton(HWND hWnd, eButtonID button, bool bDown, int x, int y);
	virtual void MouseMove(HWND hWnd, int x, int y);


	
private:
	DWORD m_dwVoxelVertShader1;
	DWORD m_dwVoxelVertShader2;
	DWORD m_dwVoxelPixShader1;
	DWORD m_dwVoxelPixShader2;
	LPDIRECT3DVERTEXBUFFER8 m_pVertexBufferVoxel1;
	LPDIRECT3DVERTEXBUFFER8 m_pVertexBufferVoxel2;

	LPDIRECT3DVERTEXBUFFER8 m_pVertexBufferCopyLine1;
	LPDIRECT3DVERTEXBUFFER8 m_pVertexBufferCopyLine1_1;
	LPDIRECT3DVERTEXBUFFER8 m_pVertexBufferCopyLine2;


	DWORD m_dwBlittingVertShader;

	LPDIRECT3DTEXTURE8 m_HighField;

	LPDIRECT3DTEXTURE8 m_TargetTexture1;	
	LPDIRECT3DTEXTURE8 m_LineTexture1;
	LPDIRECT3DTEXTURE8 m_LineTexture2;

	bool m_UsePixelShader;
	bool m_LeftMouseDown;
	int m_LastMouseX;
	int m_LastMouseY;

	float m_alpha;
	float m_height;
	float m_scaling;
	float m_centerx;
	float m_centery;

	int m_trace_steps;


	enum CopyModes
	{
		FILTER = 1,
		CLAMP = 2,
		DIFFUSE = 4,

		CLAMP_DIFFUSE = CLAMP | DIFFUSE,
		FILTER_CLAMP = FILTER | CLAMP,
		FILTER_CLAMP_DIFFUSE = FILTER | CLAMP | DIFFUSE
	};

	HRESULT CopyLine(IDirect3DSurface8* _pColor, IDirect3DSurface8* _pDepth, LPDIRECT3DTEXTURE8 _texture, DWORD _VertexShader, LPDIRECT3DVERTEXBUFFER8 _vertexbuffer, int _index, int _vertexSize, CopyModes _modes = CLAMP_DIFFUSE);
	HRESULT ProzessLine(IDirect3DSurface8* _pColor,	LPDIRECT3DTEXTURE8 _texture0, LPDIRECT3DTEXTURE8 _texture1, LPDIRECT3DTEXTURE8 _texture2, LPDIRECT3DTEXTURE8 _texture3,	DWORD _VertexShader, DWORD _Pixelshader, LPDIRECT3DVERTEXBUFFER8 _vertexbuffer, int _index, CopyModes _Texture0Modes);
	
	float GetHeightfieldLine(int _l);
	float GetTriangleLineTop(int _l, bool _invert=false);
	float GetTriangleLineBottom(int _l, bool _invert=false);
	float GetLine(int _l);
	float GetMinPosition();
	float GetMaxPosition();
	void SetLineCopy(BlittingVertex* _vertices, int _line, D3DXVECTOR4 _diffuse, bool _invert=false);
	void SetProzessLine1(VoxelVertex* _vertices, int _line);
	void SetProzessLine2(VoxelVertex* _vertices, int _line);

};

#endif