//-----------------------------------------------------------------------------
// author:	Laurent MASCHERPA
// date:	03/05/2001
// based on nVidia sample source code.
// TODO:	remove useless code. cleanup some code.
//-----------------------------------------------------------------------------

#include "eb_effect.h"
#include "shader_Plasma.hpp"
#include "nvinc.h"

using namespace std;

//-----------------------------------------------------------------------------
CShaderPlasma::CShaderPlasma()
	: m_pVertexBuffer(NULL),
	m_pLogoTexture(NULL),
	m_dwCurrentShader(0),
	m_bWireframe(false),
	m_pUI(NULL)
{
	m_strEffectName = "Oldskool Plasma Vertex Shader";
	m_strEffectLocation = "3rd Party\\Procedural";
	m_strEffectVertexShader = GetFilePath("plasma.nvv");
	m_strEffectPixelShader = "";
	m_strEffectVersion = "1.0";

	// setting up grid properties
	m_dwGridSize = DEFAULT_GRID_SIZE;
	m_dwNumVertices = m_dwGridSize*m_dwGridSize*2;
}

//-----------------------------------------------------------------------------
void CShaderPlasma::UpdateProperties()
{
	EBEffect::UpdateProperties();

	m_pVertexShaderEnum->AddEnumerant(new EBEnumValue(m_pVertexShaderEnum, "Plasma main code", 
		GetFilePath("plasma.nvv"), EBTYPE_STRING_PROP));
	m_pVertexShaderEnum->AddEnumerant(new EBEnumValue(m_pVertexShaderEnum, "Usefull macros", 
		GetFilePath("macros.nvh"), EBTYPE_STRING_PROP));
	AddProperty(new EBProperty("Wireframe", OBJECT_MEMBER(m_bWireframe), EBTYPE_BOOL_PROP));

    string aboutText = "file://";
    aboutText += GetFilePath("..\\..\\..\\NVEffectsExplained.htm");
    aboutText += "#plasma";
	SetAboutInfo( NULL, _T("Plasma"), _T(aboutText.c_str()));
	SetAboutInfo(_T("Author"),  _T("Laurent MASCHERPA"), _T("http://sanx.multimania.com"));
	SetAboutInfo(_T("Demogroup"), _T("FX Team"), _T("http://www.fxteam.fly.to"));
}

//-----------------------------------------------------------------------------
CShaderPlasma::~CShaderPlasma()
{
	Free();
}

//-----------------------------------------------------------------------------
HRESULT CShaderPlasma::Initialize(LPDIRECT3DDEVICE8 pDev)
{
	HRESULT hr;

	m_pD3DDev = pDev;
	pDev->AddRef();

	//initialize mouse UI
	RECT rect;
	rect.left = rect.top = 0;
	D3DVIEWPORT8 viewport;
	m_pD3DDev->GetViewport(&viewport);
	rect.bottom = viewport.Height;
	rect.right  = viewport.Width;
	m_pUI = new MouseUI((const RECT)rect);

	vector<DWORD> Declaration;
	Declaration.push_back(D3DVSD_STREAM(0));
	Declaration.push_back(D3DVSD_REG(0, D3DVSDT_FLOAT3));
	Declaration.push_back(D3DVSD_REG(1, D3DVSDT_FLOAT3));
	Declaration.push_back(D3DVSD_REG(2, D3DVSDT_FLOAT2));
	Declaration.push_back(D3DVSD_END());

	hr = LoadAndCreateShader(GetFilePath("plasma.vso"), &Declaration[0], 0, SHADERTYPE_VERTEX, &m_dwCurrentShader);
	if (FAILED(hr))
		return hr;

    hr = m_pD3DDev->CreateVertexBuffer( m_dwNumVertices * sizeof(QuadVertex), D3DUSAGE_WRITEONLY, 0, D3DPOOL_MANAGED, &m_pVertexBuffer);
	if (FAILED(hr))
		return hr;
		
	hr = m_pD3DDev->SetStreamSource(0, m_pVertexBuffer, sizeof(QuadVertex));
	if (FAILED(hr))
		return hr;

	hr = m_pD3DDev->SetVertexShader(m_dwCurrentShader);
	if (FAILED(hr))
		return hr;

	if (m_pVertexBuffer)
	{
		const float	grid_dim = 8.0f;
		const float	grid_dim2 = 2.0f*grid_dim;
		QuadVertex* pBuff;

		m_pVertexBuffer->Lock(0, sizeof(QuadVertex) * m_dwNumVertices,(BYTE**)&pBuff, 0);

		int dwCurrentVertex = 0;
		for (int y = 0; y < (m_dwGridSize - 1); y++)
		{
			for (int x = 0; x < m_dwGridSize; x++)
			{
				float	fx = -grid_dim + (x * grid_dim2 / m_dwGridSize);
				float	fy = -grid_dim + (y * grid_dim2 / m_dwGridSize);

				pBuff[dwCurrentVertex].Position = D3DXVECTOR3(fx, fy, 0.0f);
				pBuff[dwCurrentVertex].Normal = D3DXVECTOR3(0.0f, 0.0f, -1.0f);
				pBuff[dwCurrentVertex].Texture = D3DXVECTOR2(fx, fy);
				dwCurrentVertex++;
					
				fy = -grid_dim + ((y + 1) * grid_dim2 / m_dwGridSize);
				pBuff[dwCurrentVertex].Position = D3DXVECTOR3(fx, fy, 0.0f);
				pBuff[dwCurrentVertex].Normal = D3DXVECTOR3(0.0f, 0.0f, -1.0f);
				pBuff[dwCurrentVertex].Texture = D3DXVECTOR2(fx, fy);
				dwCurrentVertex++;
			}
		}
		assert(dwCurrentVertex <= m_dwNumVertices);

		m_pVertexBuffer->Unlock();
	}

	// Create the logo texture
	hr = D3DXCreateTextureFromFileEx(m_pD3DDev, 
		GetFilePath("nvlogo_only.dds").c_str(),
		D3DX_DEFAULT,
		D3DX_DEFAULT,
		0,
		0,
		D3DFMT_UNKNOWN,
		D3DPOOL_MANAGED,
		D3DX_FILTER_LINEAR,
		D3DX_FILTER_LINEAR,
		0,
		NULL,
		NULL,
		&m_pLogoTexture);
	if (FAILED(hr))
	{
		m_strLastError = "Could not create nvlogo_only.dds";
		return hr;
	}

	m_pD3DDev->SetRenderState(D3DRS_CULLMODE, D3DCULL_NONE);

	m_pD3DDev->SetTexture(0, m_pLogoTexture);
	m_pD3DDev->SetTextureStageState(0, D3DTSS_COLOROP, D3DTOP_MODULATE);
	m_pD3DDev->SetTextureStageState(0, D3DTSS_COLORARG1, D3DTA_TEXTURE);
	m_pD3DDev->SetTextureStageState(0, D3DTSS_COLORARG2, D3DTA_DIFFUSE);

	m_pD3DDev->SetTextureStageState(0, D3DTSS_ALPHAOP, D3DTOP_ADD);
	m_pD3DDev->SetTextureStageState(0, D3DTSS_ALPHAARG1, D3DTA_TEXTURE);
	m_pD3DDev->SetTextureStageState(0, D3DTSS_ALPHAARG2, D3DTA_DIFFUSE);

	m_pD3DDev->SetTextureStageState(1, D3DTSS_COLOROP, D3DTOP_DISABLE);
	m_pD3DDev->SetTextureStageState(1, D3DTSS_ALPHAOP, D3DTOP_DISABLE);

	m_pD3DDev->SetTextureStageState(0, D3DTSS_MINFILTER, D3DTEXF_LINEAR);
    m_pD3DDev->SetTextureStageState(0, D3DTSS_MAGFILTER, D3DTEXF_LINEAR);
	m_pD3DDev->SetTextureStageState(0, D3DTSS_MIPFILTER, D3DTEXF_NONE);

	return S_OK;
}

//-----------------------------------------------------------------------------
HRESULT CShaderPlasma::Free()
{
	SAFE_RELEASE(m_pLogoTexture);
	SAFE_RELEASE(m_pVertexBuffer);
	SAFE_DELETE(m_pUI);

	if (m_pD3DDev)
	{
		if (m_dwCurrentShader)
			m_pD3DDev->DeleteVertexShader(m_dwCurrentShader);
		SAFE_RELEASE(m_pD3DDev);
	}
	
	return S_OK;
}

//-----------------------------------------------------------------------------
HRESULT CShaderPlasma::Start()
{
	return S_OK;
}

//-----------------------------------------------------------------------------
HRESULT CShaderPlasma::Tick(EBTimer* pTimer)
{
	HRESULT hr = S_OK;
	D3DXMATRIX matWorld;
    D3DXMATRIX matView;
	D3DXMATRIX matProj;
	D3DXMATRIX matTemp;
	D3DXMATRIX matWorldViewProj;
	D3DXMATRIX matWorldView;
	D3DXMATRIX matWorldI;

	hr = m_pD3DDev->Clear(0, NULL, D3DCLEAR_TARGET|D3DCLEAR_ZBUFFER, D3DCOLOR_XRGB( 0xAA, 0xAA, 0xAA ), 1.0, 0);

	m_pD3DDev->SetRenderState(D3DRS_FILLMODE, m_bWireframe ? D3DFILL_WIREFRAME : D3DFILL_SOLID);

	D3DXVECTOR3 vEyePt = D3DXVECTOR3( 0.0f, 0.0f, -5.0f );
	D3DXVECTOR3 vLookatPt = D3DXVECTOR3( 0.0f, 0.0f, 0.0f );
	D3DXVECTOR3 vUp = D3DXVECTOR3( 0.0f, 1.0f, 0.0f );

	// Setup the world, view and projection matrices
	D3DXMatrixMultiply( &matWorld, &m_pUI->GetRotationMatrix(), &m_pUI->GetTranslationMatrix() );

	D3DXMatrixLookAtLH(&matView, &vEyePt, &vLookatPt, &vUp);
	D3DXMatrixPerspectiveFovLH(&matProj, D3DXToRadian(60.0f), 1.0f, 1.0f, 50.0f);
		
	// Setup the composite matrix to get vertices to projection space
	D3DXMatrixMultiply(&matTemp, &matWorld, &matView);
	D3DXMatrixMultiply(&matWorldViewProj, &matTemp, &matProj);
	D3DXMatrixMultiply(&matWorldView, &matWorld, &matView);
	
	// Create a 3x3 inverse of the worldview for the light transformation back to model space
	D3DXMatrixInverse(&matWorldI, NULL, &matWorld);
	D3DXMatrixTranspose(&matWorldViewProj, &matWorldViewProj);

	// Setup constants
	m_pD3DDev->SetVertexShaderConstant(0, &matWorldViewProj(0, 0), 4);
	
	// Transform from world space to model space and point it away from the vertex towards the light
	// Light is pointing into the world
	D3DXVECTOR3 vecLight(0.0f, 0.0f, 1.0f);
	D3DXVec3TransformNormal(&vecLight, &vecLight, &matWorldI);
	D3DXVec3Normalize((D3DXVECTOR3*)&vecLight, (D3DXVECTOR3*)&vecLight);
	vecLight = -vecLight;
	
	m_pD3DDev->SetVertexShaderConstant(4, &vecLight.x, 1);

	// The material color (green)
	D3DXVECTOR4 matColor(1.0f, 1.0f, 1.0f, 0.0f);
	m_pD3DDev->SetVertexShaderConstant(5, &matColor, 1);

	// animated angles
	D3DXVECTOR4	offset(sinf(GetTickCount() / 600.0f),	sinf(GetTickCount() / 1000.0f),
		sinf(GetTickCount() / 700.0f), sinf(GetTickCount() / 300.0f));
	m_pD3DDev->SetVertexShaderConstant(6, &offset, 1);

	// float_0, and float_1
	D3DXVECTOR4	utils(0, 1, 0.5f, 0);
	m_pD3DDev->SetVertexShaderConstant(7, &utils, 1);

	// usefull numbers used during computation.
	D3DXVECTOR4	pi_utils(D3DX_PI, 2*D3DX_PI, 1/(D3DX_PI*2), 0.5f);
	m_pD3DDev->SetVertexShaderConstant(8, &pi_utils, 1);

	static float f = 0;
	f += 0.05f;
	// modifiers are 4 values :
	// x = offset x.
	// y = offset y.
	// z = scale x.
	// w = scale y.
	D3DXVECTOR4	modifier(sin(f), cos(f - 1.0f), 1 + cos((f + 0.5f)*1.4), sin(-f*0.3f));
	m_pD3DDev->SetVertexShaderConstant(9, &modifier, 1);

	// used to compute sinus and cosine.
	float	taylor_sin[] = { 1.0f, -1.0f / 6.0f, 1.0f / 120.0f, -1.0f / 5040.0f };
	m_pD3DDev->SetVertexShaderConstant(11, &taylor_sin, 1);

	float	taylor_cos[] = { 1.0f, -1.0f / 2.0f, 1.0f / 24.0f, -1.0f / 720.0f };
	m_pD3DDev->SetVertexShaderConstant(10, &taylor_cos, 1);

	// Draw a sequence of stips.
	DWORD dwCurrentVertex = 0;
	DWORD dwVertexStep    = m_dwGridSize * 2;
	DWORD dwPrimCount     = dwVertexStep - 2;
	for (DWORD i = 0; i < m_dwGridSize; i++)
	{
		m_pD3DDev->DrawPrimitive(D3DPT_TRIANGLESTRIP, dwCurrentVertex, dwPrimCount);
		dwCurrentVertex += dwVertexStep;
	}
	return hr;
	
}

//-----------------------------------------------------------------------------
HRESULT CShaderPlasma::ConfirmDevice(D3DCAPS8* pCaps, DWORD dwBehavior, D3DFORMAT Format)
{
	return S_OK;
}

//-----------------------------------------------------------------------------
void CShaderPlasma::MouseButton(HWND hWnd, eButtonID button, bool bDown, int x, int y)
{
	if(button == MOUSE_LEFTBUTTON)
	{
		if(bDown)
		{
			m_pUI->OnLButtonDown(x, y);
		}
		else
		{
			m_pUI->OnLButtonUp(x, y);
		}
	}
	return;
}

//-----------------------------------------------------------------------------
void CShaderPlasma::MouseMove(HWND hWnd, int x, int y)
{
	m_pUI->OnMouseMove(x, y);
	return;
}

//-----------------------------------------------------------------------------
void CShaderPlasma::Keyboard(DWORD dwKey, UINT nFlags, bool bDown)
{
	eEBKeyAction Action = TranslateEffectKey(dwKey, nFlags, bDown);
	
    switch ( Action )
    {
		case EB_HELP:
		{
			::MessageBoxEx( NULL, " Help : F1 - Help \n\n Home - Reset To Defaults \n\n W - Wireframe Toggle \n\n Left Button & Mouse - Rotate Object\n\n Shift Left Button & Mouse - Pan Camera \n\n Ctrl Left Button & Mouse - Move Camera In & Out\n\n",
				   "Help", MB_ICONINFORMATION | MB_TASKMODAL, MAKELANGID( LANG_NEUTRAL, SUBLANG_DEFAULT ) );
		}
		break;

		case EB_WIREFRAME:
        {
            m_bWireframe = !m_bWireframe;
			m_dwEffectDirtyFlags |= EBEFFECT_DIRTY_PUBLICSTATE;
            
        }
		break;

		case EB_RESET:
        {
            m_pUI->Reset();
            m_bWireframe = false;
			m_dwEffectDirtyFlags |= EBEFFECT_DIRTY_PUBLICSTATE;
            
        }
		break;

        default :
            break;
    }
}
