
#ifndef SHADER_Rain_H
#define SHADER_Rain_H
#include "MouseUI.h"
#include <string>

#define NUM_MODELS	1

// Vertex definition for base model
struct BaseVertex
{
	float x, y, z;
	float nx, ny, nz;
	float u,v;    
	enum FVF
	{
		FVF_Flags = D3DFVF_XYZ | D3DFVF_NORMAL | D3DFVF_TEX1
	};
};

// Vertex definition for rain particles
struct RainVertex
{
	float x, y, z;
	float nx, ny, nz;
	float loop; // Loop point
};

class CShaderRain : public EBEffect
{
public:	
	CShaderRain();
	~CShaderRain();

	virtual void PropertyUpdateCallback(const EBProperty* pProperty, bool bWritten );

			void	CreateRainVertices(int idx);
	virtual void    UpdateProperties();
	virtual HRESULT Initialize(IDirect3DDevice8* pDev);
	virtual HRESULT Free();
	virtual HRESULT Start();
	virtual HRESULT Tick(EBTimer* pTimer);
	virtual HRESULT ConfirmDevice(D3DCAPS8* pCaps, DWORD dwBehavior, D3DFORMAT Format);
	virtual void MouseMove(HWND hWnd, int x, int y);
	virtual void MouseButton(HWND hWnd, eButtonID Button, bool bDown, int x, int y);
	virtual void Keyboard(DWORD dwKey, UINT nFlags, bool bDown);
	
private:

	LPDIRECT3DVERTEXBUFFER8 m_pVertexBuffer[NUM_MODELS];
	DWORD					m_numFace[NUM_MODELS];
	LPDIRECT3DINDEXBUFFER8  m_pIndexBuffer[NUM_MODELS];

	LPDIRECT3DVERTEXBUFFER8 m_pRainVertexBuffer[NUM_MODELS];
	DWORD					m_dwRainVerts[NUM_MODELS];

	LPDIRECT3DTEXTURE8 m_pShadeTexture[NUM_MODELS];
	LPDIRECT3DTEXTURE8 m_pShadeRainTexture[NUM_MODELS];
	
	D3DXATTRIBUTERANGE *m_pAttributes[NUM_MODELS];
	
	//our matrices
	D3DXMATRIX m_world;
	D3DXMATRIX m_view;
	D3DXMATRIX m_proj;
	
	//number of sections in the mesh
	DWORD m_dwNumSections[NUM_MODELS];
	
	DWORD m_dwCurrentShader;
	DWORD m_dwCurrentRainShader;

	std::string m_strFileName[NUM_MODELS];
	std::string m_strTextureName[NUM_MODELS];
	std::string m_strRainTextureName[NUM_MODELS];
	std::string m_strEffectRainVertexShader;

	bool m_bWireframe;
	bool m_bUseTrilinear;
	MouseUI* m_pUI;

	
	HRESULT LoadXFile(int idx,const char* fileName, const DWORD dwFVF);
	HRESULT SetVertexShaderMatrices();
	HRESULT SetTextureStageStates(bool Fuzz);

	int m_nCurModel;
};

#endif