/******************************************************************************

  Copyright (C) 1999, 2000 NVIDIA Corporation
  This file is provided without support, instruction, or implied warranty of any
  kind.  NVIDIA makes no guarantee of its fitness for a particular purpose and is
  not liable under any circumstances for any damages or loss whatsoever arising
  from the use or inability to use this file or items derived from it.
  
    Comments:
    
      
        
******************************************************************************/

#include "eb_effect.h"
#include "shader_Spline.h"
#include "nvinc.h"

using namespace std;

DECLARE_EFFECT_MAIN()

extern "C"
{

__declspec(dllexport) unsigned int GetNumEffects() { return 1; }

__declspec(dllexport) EBEffect* CreateEffect(unsigned int EffectNum)
{	
	switch(EffectNum)
	{
		case 0:
			return new CShaderSpline();
			break;
		default:
			return NULL;
	}
}

}

CShaderSpline::CShaderSpline()
	: m_pVertexBuffer(NULL),
	m_dwCurrentShader(0),
	m_pUI(NULL),
	m_bAnimateEndPts(false),
	v1( -2.0f, 0.0f, 0.0f ),
	v1out( -1.0f, 2.0f, 0.0f ),
	v2in( 1.0f, -2.0f, 0.0f ),
	v2( 2.0f, 0.0f, 0.0f )
{
	m_strEffectLocation = "3rd Party\\Deformation";
	m_strEffectName = "Bezier Spline Generation";
	m_strEffectVertexShader = GetFilePath("spline.nvv");
	m_strEffectPixelShader = "";
	m_strEffectVersion = "1.0";

	y_axis = 0.0f;
	dy_axis = 0.002f;
	n_Segments = 16;
}

#define STR_INCREASE_EDGE "Increase Segments(+)"
#define STR_DECREASE_EDGE "Decrease Segments(-)"

void CShaderSpline::UpdateProperties()
{
	EBEffect::UpdateProperties();

	m_pVertexShaderEnum->AddEnumerant(new EBEnumValue(m_pVertexShaderEnum, "Bezier Spline", GetFilePath("spline.nvv"), EBTYPE_STRING_PROP));
	
	AddProperty(new EBTriggerProperty(STR_DECREASE_EDGE));
	AddProperty(new EBTriggerProperty(STR_INCREASE_EDGE));	

	AddProperty(new EBProperty("Animate Spline", OBJECT_MEMBER(m_bAnimateEndPts), EBTYPE_BOOL_PROP));
 
    string aboutText = "file://";
    aboutText += GetFilePath("..\\..\\..\\NVEffectsExplained.htm");
    aboutText += "#splinegeneration";
	SetAboutInfo( NULL, _T("Bezier Spline"), _T(aboutText.c_str()));

}

CShaderSpline::~CShaderSpline()
{
	Free();	
}


void CShaderSpline::SetConstantRegisters( void )
{
	//Setup bezier matrix
	D3DXVECTOR4 vec;

	vec.x = v1.x; 
	vec.y = v1out.x; 
	vec.z = v2in.x; 
	vec.w = v2.x;
	m_pD3DDev->SetVertexShaderConstant(4, &vec, 1);
	
	vec.x = v1.y; 
	vec.y = v1out.y; 
	vec.z = v2in.y; 
	vec.w = v2.y;
	m_pD3DDev->SetVertexShaderConstant(5, &vec, 1);

	vec.x = v1.z; 
	vec.y = v1out.z; 
	vec.z = v2in.z; 
	vec.w = v2.z;
	m_pD3DDev->SetVertexShaderConstant(6, &vec, 1);

	
	D3DXVECTOR4 lineColor(1.0f, 1.0f, 1.0f, 1.0f);
	m_pD3DDev->SetVertexShaderConstant(7, &lineColor, 1);

	//Const
	m_pD3DDev->SetVertexShaderConstant(8, D3DXVECTOR4(0.0f, 1.0f, 3.0f, 0.5f), 1);

	//Animation values
	m_pD3DDev->SetVertexShaderConstant(9, D3DXVECTOR4(0.0f, 0.0f, 0.0f, 0.0f), 1);
}


//Create new number of segments from 0 to 1
HRESULT CShaderSpline::SetupSegmentVerts( void )
{
	SplineVertex* pBuff;
	HRESULT hr;

	if( m_pVertexBuffer )
		SAFE_RELEASE(m_pVertexBuffer);

	hr = m_pD3DDev->CreateVertexBuffer( n_Segments * sizeof(SplineVertex), D3DUSAGE_WRITEONLY, 0, D3DPOOL_MANAGED, &m_pVertexBuffer);
	if (FAILED(hr))
		return hr;
		
	hr = m_pD3DDev->SetStreamSource(0, m_pVertexBuffer, sizeof(SplineVertex));
	if (FAILED(hr))
		return hr;

	if (m_pVertexBuffer)
	{
		float fScale = 1.0f;
		hr = m_pVertexBuffer->Lock(0, sizeof(SplineVertex) * n_Segments,(BYTE**)&pBuff, 0);
		if (FAILED(hr))
		{
			m_strLastError = "Couldn't lock buffer!";
			return hr;
		}

		for( int i = 0; i < n_Segments; i++ )
		{
			pBuff->dt = (float)i/(float)(n_Segments-1);
			pBuff++;
		}
		m_pVertexBuffer->Unlock();
	}
	return S_OK;
}

HRESULT CShaderSpline::Initialize(LPDIRECT3DDEVICE8 pDev)
{
	HRESULT hr;

	m_pD3DDev = pDev;
	pDev->AddRef();

	//initialize mouse UI
	RECT rect;
	rect.left = rect.top = 0;
	D3DVIEWPORT8 viewport;
	m_pD3DDev->GetViewport(&viewport);
	rect.bottom = viewport.Height;
	rect.right  = viewport.Width;
	m_pUI = new MouseUI((const RECT)rect);

	vector<DWORD> Declaration;
	Declaration.push_back(D3DVSD_STREAM(0));
	Declaration.push_back(D3DVSD_REG(0, D3DVSDT_FLOAT1));
	Declaration.push_back(D3DVSD_END());

	hr = LoadAndCreateShader(GetFilePath("spline.vso"), &Declaration[0], 0, SHADERTYPE_VERTEX, &m_dwCurrentShader);
	if (FAILED(hr))
		return hr;

	hr = SetupSegmentVerts();
	if (FAILED(hr))
		return hr;

	hr = m_pD3DDev->SetVertexShader(m_dwCurrentShader);
	if (FAILED(hr))
		return hr;

	SetConstantRegisters();
	
	m_pD3DDev->SetRenderState(D3DRS_CULLMODE, D3DCULL_NONE);

	m_pD3DDev->SetTexture(0, NULL);
	m_pD3DDev->SetTextureStageState(0, D3DTSS_COLOROP, D3DTOP_MODULATE);
	m_pD3DDev->SetTextureStageState(0, D3DTSS_COLORARG1, D3DTA_TEXTURE);
	m_pD3DDev->SetTextureStageState(0, D3DTSS_COLORARG2, D3DTA_DIFFUSE);

	m_pD3DDev->SetTextureStageState(0, D3DTSS_ALPHAOP, D3DTOP_ADD);
	m_pD3DDev->SetTextureStageState(0, D3DTSS_ALPHAARG1, D3DTA_TEXTURE);
	m_pD3DDev->SetTextureStageState(0, D3DTSS_ALPHAARG2, D3DTA_DIFFUSE);

	m_pD3DDev->SetTextureStageState(1, D3DTSS_COLOROP, D3DTOP_DISABLE);
	m_pD3DDev->SetTextureStageState(1, D3DTSS_ALPHAOP, D3DTOP_DISABLE);

	m_pD3DDev->SetTextureStageState(0, D3DTSS_MINFILTER, D3DTEXF_LINEAR);
    m_pD3DDev->SetTextureStageState(0, D3DTSS_MAGFILTER, D3DTEXF_LINEAR);
	m_pD3DDev->SetTextureStageState(0, D3DTSS_MIPFILTER, D3DTEXF_NONE);


	SetConstantRegisters();
	return S_OK;

}

HRESULT CShaderSpline::Free()
{
	SAFE_RELEASE(m_pVertexBuffer);
	SAFE_DELETE(m_pUI);

	if (m_pD3DDev)
	{
		if (m_dwCurrentShader)
			m_pD3DDev->DeleteVertexShader(m_dwCurrentShader);
		SAFE_RELEASE(m_pD3DDev);
	}
	
	return S_OK;
}

HRESULT CShaderSpline::Start()
{
	return S_OK;
}



HRESULT CShaderSpline::Tick(EBTimer* pTimer)
{
	HRESULT hr = S_OK;
	D3DXMATRIX matWorld;
    D3DXMATRIX matView;
	D3DXMATRIX matProj;
	D3DXMATRIX matTemp;
	D3DXMATRIX matWorldViewProj;
	D3DXMATRIX matWorldView;
	D3DXMATRIX matWorldI;
	

	hr = m_pD3DDev->Clear(0, NULL, D3DCLEAR_TARGET|D3DCLEAR_ZBUFFER, D3DCOLOR_XRGB( 0xAA, 0xAA, 0xAA ), 1.0, 0);

	D3DXVECTOR3 vEyePt = D3DXVECTOR3( 0.0f, 0.0f, -5.0f );
	D3DXVECTOR3 vLookatPt = D3DXVECTOR3( 0.0f, 0.0f, 0.0f );
	D3DXVECTOR3 vUp = D3DXVECTOR3( 0.0f, 1.0f, 0.0f );

	// Setup the world, view and projection matrices
	D3DXMatrixMultiply( &matWorld, &m_pUI->GetRotationMatrix(), &m_pUI->GetTranslationMatrix() );

	D3DXMatrixLookAtLH(&matView, &vEyePt, &vLookatPt, &vUp);
	D3DXMatrixPerspectiveFovLH(&matProj, D3DXToRadian(60.0f), 1.0f, 1.0f, 50.0f);
		
	// Setup the composite matrix to get vertices to projection space
	D3DXMatrixMultiply(&matTemp, &matWorld, &matView);
	D3DXMatrixMultiply(&matWorldViewProj, &matTemp, &matProj);
	D3DXMatrixMultiply(&matWorldView, &matWorld, &matView);
	
	// Create a 3x3 inverse of the worldview for the light transformation back to model space
	D3DXMatrixInverse(&matWorldI, NULL, &matWorld);
	D3DXMatrixTranspose(&matWorldViewProj, &matWorldViewProj);

	// Setup constants
	m_pD3DDev->SetVertexShaderConstant(0, &matWorldViewProj(0, 0), 4);
	

	//setup animation values
	
	if( m_bAnimateEndPts )
	{
		D3DXVECTOR4 yaxis(0.0f, 0.0f, 0.0f, 0.0f);
		y_axis += dy_axis;
		if( y_axis > 4.0f )
			dy_axis *= -1.0f;
		if( y_axis < -6.0f )
			dy_axis *= -1.0f;
		yaxis.y = y_axis;
		yaxis.z = -y_axis;
		m_pD3DDev->SetVertexShaderConstant(9, &yaxis, 1);
	}
	

	hr = m_pD3DDev->DrawPrimitive( D3DPT_LINESTRIP, 0, n_Segments - 1 );
	return hr;
}

HRESULT CShaderSpline::ConfirmDevice(D3DCAPS8* pCaps, DWORD dwBehavior, D3DFORMAT Format)
{
	return S_OK;
}

void CShaderSpline::MouseButton(HWND hWnd, eButtonID button, bool bDown, int x, int y)
{
	if(button == MOUSE_LEFTBUTTON)
	{
		if(bDown)
		{
			m_pUI->OnLButtonDown(x, y);
		}
		else
		{
			m_pUI->OnLButtonUp(x, y);
		}
	}
	return;
}

void CShaderSpline::MouseMove(HWND hWnd, int x, int y)
{
	m_pUI->OnMouseMove(x, y);
	return;
}

void CShaderSpline::Keyboard(DWORD dwKey, UINT nFlags, bool bDown)
{
	
	if(bDown)
	{
		switch(dwKey)
		{
            case 'H' :
            case VK_F1 :
            {
		        ::MessageBoxEx( NULL, " Help : F1 - Help \n\n Home - Reset To Defaults \n\n W - Wireframe Toggle \n\n Space\\Pause - Toggle Pause/Resume \n\n Left Button & Mouse - Rotate Object\n\n Shift Left Button & Mouse - Pan Camera \n\n Ctrl Left Button & Mouse - Move Camera In & Out\n\n + increase silhouette width\n\n - decrease silhouette width",
					   "Help", MB_ICONINFORMATION | MB_TASKMODAL, MAKELANGID( LANG_NEUTRAL, SUBLANG_DEFAULT ) );
                break;
            }

            case VK_HOME :
            case VK_END :
			case VK_NUMPAD7 :
			case VK_NUMPAD1 :
            case '7' :
            case '1' :
            {
                m_pUI->Reset();
                m_dwEffectDirtyFlags |= EBEFFECT_DIRTY_PUBLICSTATE;
                break;
            }

			case VK_ADD:
			case 0xBB:
			case '+':
			case '=':
			{
				if(n_Segments <= 32 )
				{
					n_Segments += 1;
					SetupSegmentVerts();
				}
				break;
			}

			case VK_SUBTRACT:
			case 0xBD:
			case '-':
			case '_':
			{
				if(n_Segments > 2 )
				{
					n_Segments -= 1;
					SetupSegmentVerts();
				}
				break;
			}

            default :
            {
                break;
            }
        }
	}
	return;
}

void CShaderSpline::PropertyUpdateCallback(const EBProperty* pProperty, bool bWritten)
{
	if (!bWritten)
		return;

	EBString name;

	if (pProperty->IsKindOf(EBTYPE_TRIGGER_PROP))
	{

		name = pProperty->GetPropertyName();
		
		if(name == EBString(STR_INCREASE_EDGE))
		{
			Keyboard(VK_ADD, 0, true);
		}
		else if (name == EBString(STR_DECREASE_EDGE))
		{
			Keyboard(VK_SUBTRACT, 0, true);
		}
	}
}