// twister.cpp  version 2


#include "eb_effect.h"
#include "nvinc.h"
#include "MouseUI.h"
#include "twister.h"
#include <vector>
#include <NVFileDialog.h>
#include "constants.h"

using namespace std;

#define PI  3.14159265358f

struct TWISTERVERTEX
{
	D3DXVECTOR3 v;
	D3DXVECTOR3 n;
	FLOAT       tu;
	FLOAT       tv;

};

#define D3DFVF_TWISTERVERTEX    (D3DFVF_XYZ|D3DFVF_NORMAL|D3DFVF_TEX1)


DECLARE_EFFECT_MAIN()

extern "C"
{

__declspec(dllexport) unsigned int GetNumEffects() { return 4; }

__declspec(dllexport) EBEffect* CreateEffect(unsigned int EffectNum)
{
	

		switch(EffectNum)
		{
		case 0:
			return new CTwister(0);
			break;
		case 1:
			return new CTwister(1);
			break;
		case 2:
			return new CTwister(2);
			break;
		case 3:
			return new CTwister(3);
			break;

		default:
			return NULL;
		}


}

};


CTwister::CTwister(int num)
{
		m_fRadius = 0.0f;
		m_vecCenter.x = 0.0f;
		m_vecCenter.y = 0.0f;
		m_vecCenter.z = 0.0f;
		m_dwCurrentShader = 0;
		m_bWireframe = false;
		m_bPause = false;
		m_pVertexBuffer = NULL;
		m_pIndexBuffer = NULL;
		m_pUI = NULL;
		m_bAxeY    = FALSE;

		m_pMaterials = NULL;
		m_pTextures  = NULL;

		D3DXMatrixIdentity(&m_world);
		D3DXMatrixIdentity(&m_view);
		D3DXMatrixIdentity(&m_proj);

		m_iNum = num;

		if(m_iNum == 0)
		{
		m_strEffectName = "Twister2";
		m_strEffectLocation = "3rd Party\\Deformation";
		m_strEffectPixelShader = "";
		m_strEffectVertexShader = GetFilePath("Twister2.nvv");
		m_strFileName = "Twister.x";
		}
		else if(m_iNum == 1)
		{
		m_strEffectName = "Twister2 Scaling";
		m_strEffectLocation = "3rd Party\\Deformation";
		m_strEffectPixelShader = "";
		m_strEffectVertexShader = GetFilePath("Scaling.nvv");
		m_strFileName = "scaling.x";
		}
		else if(m_iNum == 2)
		{
		m_strEffectName = "Twister2 Sine";
		m_strEffectLocation = "3rd Party\\Deformation";
		m_strEffectPixelShader = "";
		m_strEffectVertexShader = GetFilePath("Sine.nvv");
		m_strFileName = "sinusoid.x";
		}
		else if(m_iNum == 3)
		{
		m_strEffectName = "Twister2 Manta";
		m_strEffectLocation = "3rd Party\\Deformation";
		m_strEffectPixelShader = "";
		m_strEffectVertexShader = GetFilePath("Manta.nvv");
		m_strFileName = "spineless.x";
		}

		m_strEffectVersion = "2.0";

		
}


CTwister::~CTwister()
{
		
	Free();	
		
}

void CTwister::UpdateProperties()
{
	EBEffect::UpdateProperties();

	AddProperty(new EBProperty("Wireframe", OBJECT_MEMBER(m_bWireframe), EBTYPE_BOOL_PROP));
	AddProperty(new EBProperty("Pause", OBJECT_MEMBER(m_bPause), EBTYPE_BOOL_PROP));

	AddProperty(new EBProperty("Axe Y", OBJECT_MEMBER(m_bAxeY), EBTYPE_BOOL_PROP));

	AddProperty(new EBTriggerProperty("Load Another X File Mesh..."));

	if(m_iNum == 0)
		m_pVertexShaderEnum->AddEnumerant(new EBEnumValue(m_pVertexShaderEnum, "Twisting Vertex", GetFilePath("Twister2.nvv"), EBTYPE_STRING_PROP));
	else if(m_iNum == 1)
		m_pVertexShaderEnum->AddEnumerant(new EBEnumValue(m_pVertexShaderEnum, "Scaling Vertex", GetFilePath("Scaling.nvv"), EBTYPE_STRING_PROP));
	else if(m_iNum == 2)
		m_pVertexShaderEnum->AddEnumerant(new EBEnumValue(m_pVertexShaderEnum, "Sine Vertex", GetFilePath("Sine.nvv"), EBTYPE_STRING_PROP));
	else if(m_iNum == 3)
		m_pVertexShaderEnum->AddEnumerant(new EBEnumValue(m_pVertexShaderEnum, "Manta Vertex", GetFilePath("Manta.nvv"), EBTYPE_STRING_PROP));

    string aboutText = "file://";
    aboutText += GetFilePath("..\\..\\..\\NVEffectsExplained.htm");
    aboutText += "#twister";
	SetAboutInfo( NULL, _T("Twister"), _T(aboutText.c_str()));
}


HRESULT CTwister::ConfirmDevice(D3DCAPS8* pCaps, DWORD dwBehavior, D3DFORMAT Format)
{
	return S_OK;
}

HRESULT CTwister::Initialize(LPDIRECT3DDEVICE8 pDev)
{

	HRESULT hr;

	m_pD3DDev = pDev;
	pDev->AddRef();


	//initialize mouse UI
	RECT rect;
	rect.left = rect.top = 0;
	D3DVIEWPORT8 viewport;
	m_pD3DDev->GetViewport(&viewport);
	rect.bottom = viewport.Height;
	rect.right  = viewport.Width;
	m_pUI = new MouseUI((const RECT)rect);


	// Values by default
	m_iSpeed = 10;
	m_fStep = 10.0f;
	m_fLoop = 90.0f;
	m_fTick = 0.0f;
	m_fOffset = 0.0f;




 	vector<DWORD> Declaration;
	Declaration.push_back(D3DVSD_STREAM(0));
	Declaration.push_back(D3DVSD_REG(D3DVSDE_POSITION, D3DVSDT_FLOAT3));
	Declaration.push_back(D3DVSD_REG(D3DVSDE_NORMAL, D3DVSDT_FLOAT3));
	Declaration.push_back(D3DVSD_REG(D3DVSDE_TEXCOORD0, D3DVSDT_FLOAT2));
	Declaration.push_back(D3DVSD_END());

	if(m_iNum == 0)
		hr = LoadAndCreateShader(GetFilePath("Twister2.vso"), &Declaration[0], 0, SHADERTYPE_VERTEX, &m_dwCurrentShader);
	else if(m_iNum == 1)
		hr = LoadAndCreateShader(GetFilePath("Scaling.vso"), &Declaration[0], 0, SHADERTYPE_VERTEX, &m_dwCurrentShader);
	else if(m_iNum == 2)
		hr = LoadAndCreateShader(GetFilePath("Sine.vso"), &Declaration[0], 0, SHADERTYPE_VERTEX, &m_dwCurrentShader);
	else if(m_iNum == 3)
		hr = LoadAndCreateShader(GetFilePath("Manta.vso"), &Declaration[0], 0, SHADERTYPE_VERTEX, &m_dwCurrentShader);

	if (FAILED(hr))
		return hr;

	// select the just created vertex shader as the thing to transform and light vertices
	hr = m_pD3DDev->SetVertexShader(m_dwCurrentShader);
	if (FAILED(hr))
		return hr;


	hr = LoadXFile(GetFilePath(m_strFileName).c_str(), D3DFVF_TWISTERVERTEX);
	if(FAILED(hr))
		return hr;

		    // Set miscellaneous render states
    m_pD3DDev->SetRenderState( D3DRS_ZENABLE, TRUE );
    m_pD3DDev->SetRenderState( D3DRS_AMBIENT, 0x00404040 );




	//set view matrix depending on the bounding sphere
	D3DXVECTOR3 eye, lookAt, up;
	
	eye.x    = m_vecCenter.x; eye.y    = m_vecCenter.y; eye.z    = -m_fRadius*2;
	lookAt.x = m_vecCenter.x; lookAt.y = m_vecCenter.y; lookAt.z = m_vecCenter.z;
	up.x	 = 0.0f;          up.y     = 1.0f;          up.z     = 0.0f;
	

	
	D3DXMatrixIdentity(&m_world);
	D3DXMatrixLookAtLH(&m_view, &eye, &lookAt, &up);
	
	D3DXMatrixPerspectiveFovLH(&m_proj,
		D3DXToRadian(60),
		1,
		1.0f,
		10000.0f);
	m_pD3DDev->SetTransform(D3DTS_PROJECTION, (D3DMATRIX*)&m_proj);
	
	
	SetVertexShaderMatrices();
	


	SetVSConst();



	return S_OK;
}

void CTwister::SetVSConst()
{

	   //Ambient light
       FLOAT       fAmbient[] = { 0.25f, 0.25f, 0.25f, 0.25f };
       m_pD3DDev->SetVertexShaderConstant( 5, &fAmbient, 1 );


	   //constant for identity matrix 
	   D3DXVECTOR4 v1(1.0f, 0.0, 0.0, 0.0);
	   m_pD3DDev->SetVertexShaderConstant(8,&v1,1);


	   FLOAT fxmin = m_vecCenter.x - m_fRadius;
	   FLOAT fNegativeParty = 0.0f;

       if(fxmin < 0.0f)
			fNegativeParty = - fxmin;        

	
		D3DXVECTOR4  lVector;
   

	   lVector.x = fNegativeParty;
	   lVector.y = (m_fRadius*2.0f)/m_fStep;   //Weight of effect   
	   lVector.z = 2.0f; 
	   lVector.w = m_fOffset; 
	   m_pD3DDev->SetVertexShaderConstant(9,&lVector,1);
	

	   D3DXVECTOR4 v10(PI, 1.0f/2.0f, 2.0f*PI, 1.0f/(2.0f*PI));
	   m_pD3DDev->SetVertexShaderConstant(10,&v10,1);
	
	
	   D3DXVECTOR4 v11(1.0f, -1.0f/2.0f, 1.0f/24.0f, -1.0f/720.0f);
	   m_pD3DDev->SetVertexShaderConstant(11,&v11,1);
	   

	   D3DXVECTOR4 v12(1.0f, -1.0f/6.0f, 1.0f/120.0f, -1.0f/5040.0f);
	   m_pD3DDev->SetVertexShaderConstant(12,&v12,1);


		D3DXMATRIX translation;
		D3DXVECTOR3 vector = -m_vecCenter;

		D3DXMatrixTranslation(&translation,vector.x,vector.y,vector.z);
		m_pD3DDev->SetVertexShaderConstant(  15, &translation, 4 );





}


void CTwister::PropertyUpdateCallback(const EBProperty* pProperty, bool bWritten)
{

	EBString name;

	if (!bWritten)
		return;

	if (pProperty->IsKindOf(EBTYPE_TRIGGER_PROP))
	{

		name = pProperty->GetPropertyName();

		if(name == EBString("Load Another X File Mesh..."))
		{
			// Now do file dialog
			NVXFileDialog aDialog;

			std::string theResult;
			bWritten = ( aDialog.Open( theResult ) );
			
			if ( bWritten )
			{
				m_strFileName = theResult;
				HRESULT hr;
				
				
				hr = LoadXFile(m_strFileName.c_str(), D3DFVF_TWISTERVERTEX);

				bWritten = hr == S_OK;
				if(bWritten)
				SetVSConst();
				
			}
		}

	}
}


HRESULT CTwister::SetVertexShaderMatrices()
{
	D3DXMATRIX worldViewProjMat, worldITMat;
	D3DXMATRIX tempMat;
	
	D3DXMatrixMultiply(&tempMat, &m_pUI->GetRotationMatrix(), &m_pUI->GetTranslationMatrix());
	
		

	
	D3DXMatrixIdentity(&worldViewProjMat);
	
	D3DXMatrixMultiply(&worldViewProjMat, &tempMat, &m_view);
	
	D3DXMatrixInverse(&worldITMat, NULL, &tempMat);
	
	D3DXMatrixMultiply(&worldViewProjMat, &worldViewProjMat, &m_proj);
	D3DXMatrixTranspose(&worldViewProjMat, &worldViewProjMat);

	
	m_pD3DDev->SetVertexShaderConstant(0, &worldViewProjMat, 4);
	
	
	D3DXMatrixInverse(&worldITMat, NULL, &tempMat);
	D3DXVECTOR3 vecLight(0.0f, 0.0f, 1.0f);
	D3DXVec3TransformNormal(&vecLight, &vecLight, &worldITMat);
	D3DXVec3Normalize((D3DXVECTOR3*)&vecLight, (D3DXVECTOR3*)&vecLight);
	vecLight = -vecLight;
	
	m_pD3DDev->SetVertexShaderConstant(6, &vecLight.x, 1);



	
	return S_OK;
}


HRESULT CTwister::Free()
{

	
	SAFE_RELEASE(m_pVertexBuffer);
	SAFE_RELEASE(m_pIndexBuffer);
	
	SAFE_DELETE(m_pUI);

	SAFE_DELETE_ARRAY(m_pMaterials);
	SAFE_DELETE_ARRAY(m_pTextures);


	if (m_pD3DDev)
	{
		if (m_dwCurrentShader)
			m_pD3DDev->DeleteVertexShader(m_dwCurrentShader);
		SAFE_RELEASE(m_pD3DDev);
	}
	

	return S_OK;
}






HRESULT CTwister::Start()
{
	return S_OK;
}



void CTwister::MouseButton(HWND hWnd, eButtonID button, bool bDown, int x, int y)
{
	if( button == MOUSE_LEFTBUTTON)
	{
		if(bDown)
		{
			m_pUI->OnLButtonDown(x, y);
		}
		else
		{
			m_pUI->OnLButtonUp(x, y);
		}
	}

	return;
}


void CTwister::MouseMove(HWND hWnd, int x, int y)
{
	
	m_pUI->OnMouseMove(x, y);
	return;
}





HRESULT CTwister::Tick(EBTimer* pTimer)
{

	HRESULT hr;


		

	hr = m_pD3DDev->Clear(0, NULL, D3DCLEAR_TARGET|D3DCLEAR_ZBUFFER, 0x00aaaaaa, 1.0, 0);

	SetVertexShaderMatrices();

	m_pD3DDev->SetRenderState(D3DRS_FILLMODE, (m_bWireframe ? D3DFILL_WIREFRAME : D3DFILL_SOLID));

		
		
		static bool freverse = false;

		if(!m_bPause)
		{
			if(freverse == false && m_fTick == m_fLoop)
			{
				freverse = true;
			
			}
			else if(freverse == true && m_fTick == 0)
			{
				freverse = false;

			}
			else if(freverse == false)
			{
				m_fTick += 1.0f;
			}
			else if(freverse == true)
			{
				m_fTick -= 1.0f;
			}
		}


		
				
		D3DXVECTOR4 vT1(D3DXToRadian(m_fTick)*(FLOAT)m_iSpeed,D3DXToRadian(360.0f),m_bAxeY == FALSE ? 1.0f : -1.0f,36.0f);
		m_pD3DDev->SetVertexShaderConstant(  7, &vT1, 1 );


		

		for(int i = 0; i < m_dwNumSections; i++)
		{


			
			m_pD3DDev->SetVertexShaderConstant( 4, &m_pMaterials[i].Diffuse, 1 );
			m_pD3DDev->SetTexture( 0, m_pTextures[i] );
			m_pD3DDev->DrawIndexedPrimitive(D3DPT_TRIANGLELIST,
				m_pAttributes[i].VertexStart, 
			    m_pAttributes[i].VertexCount,
				m_pAttributes[i].FaceStart*3,
				m_pAttributes[i].FaceCount);
		}




	return hr;
}


HRESULT CTwister::LoadXFile(const char* fileName, const DWORD dwFVF)
{
	ID3DXMesh* tempMesh, *tempMeshFVF, *tempMeshOpt;
	ID3DXBuffer* tempMaterials;
	
	HRESULT hr = D3DXLoadMeshFromX(const_cast<char*>(fileName), D3DXMESH_VB_SYSTEMMEM, m_pD3DDev, NULL, 
		&tempMaterials, &m_dwNumSections, &tempMesh);


	if(FAILED(hr))
		return hr;

	// colors and textures
    if( tempMaterials && m_dwNumSections > 0 )
    {
        // Allocate memory for the materials and textures
        D3DXMATERIAL* d3dxMtrls = (D3DXMATERIAL*)tempMaterials->GetBufferPointer();
        m_pMaterials = new D3DMATERIAL8[m_dwNumSections];
        m_pTextures  = new LPDIRECT3DTEXTURE8[m_dwNumSections];

        // Copy each material and create it's texture
        for( DWORD i=0; i<m_dwNumSections; i++ )
        {
            // Copy the material
            m_pMaterials[i]         = d3dxMtrls[i].MatD3D;
            m_pMaterials[i].Ambient = m_pMaterials[i].Diffuse;
            m_pTextures[i]          = NULL;

            // Create a texture
            if( d3dxMtrls[i].pTextureFilename )
            {



				std::string  TextureFile = d3dxMtrls[i].pTextureFilename ;
                if( FAILED( D3DXCreateTextureFromFile( m_pD3DDev, GetFilePath(TextureFile).c_str(), 
                                                       &m_pTextures[i] ) ) )
                    m_pTextures[i] = NULL;
				
            }
        }
    }

	m_pD3DDev->SetRenderState(D3DRS_CULLMODE, D3DCULL_NONE);
	m_pD3DDev->SetTextureStageState(0, D3DTSS_COLORARG1, D3DTA_TEXTURE);
	m_pD3DDev->SetTextureStageState(0, D3DTSS_COLOROP, D3DTOP_MODULATE);
	m_pD3DDev->SetTextureStageState(0, D3DTSS_COLORARG2, D3DTA_DIFFUSE);

	//compute bounding sphere
	TWISTERVERTEX* pBuff;
	hr = tempMesh->LockVertexBuffer(D3DLOCK_READONLY, (BYTE**)&pBuff);
	if(FAILED(hr))
		return hr;
	D3DXComputeBoundingSphere(pBuff, tempMesh->GetNumVertices(), tempMesh->GetFVF(), &m_vecCenter, &m_fRadius);
	tempMesh->UnlockVertexBuffer();
	
	//optimize in attribute order
	hr = tempMesh->Optimize(D3DXMESHOPT_ATTRSORT, NULL, NULL, NULL, NULL, &tempMeshOpt);
	if(FAILED(hr))
		return hr;

	//get attributes
	DWORD attribSize = m_dwNumSections;
	hr = tempMeshOpt->GetAttributeTable(NULL, &attribSize);
	if(FAILED(hr))
		return hr;
	
	m_pAttributes = new D3DXATTRIBUTERANGE[attribSize];
	
	hr = tempMeshOpt->GetAttributeTable(m_pAttributes, &attribSize);
	if(FAILED(hr))
		return hr;

	//convert to our format, and also make writeonly
	hr = tempMeshOpt->CloneMeshFVF(D3DXMESH_MANAGED, dwFVF, m_pD3DDev, &tempMeshFVF);
	if(FAILED(hr))
		return hr;

	D3DXComputeNormals( tempMeshFVF);

	SAFE_RELEASE( m_pVertexBuffer );
	SAFE_RELEASE( m_pIndexBuffer );

	tempMeshFVF->GetVertexBuffer(&m_pVertexBuffer);
	tempMeshFVF->GetIndexBuffer(&m_pIndexBuffer);
	

	SAFE_RELEASE(tempMesh);
	SAFE_RELEASE(tempMeshFVF);
	SAFE_RELEASE(tempMeshOpt);


	hr = m_pD3DDev->SetStreamSource(0, m_pVertexBuffer, sizeof(TWISTERVERTEX) );
	if (FAILED(hr))
		return hr;
	//set index buffer
	m_pD3DDev->SetIndices( m_pIndexBuffer, 0);

	return S_OK;
}


void CTwister::Keyboard(DWORD dwKey, UINT nFlags, bool bDown)
{


	eEBKeyAction Action = TranslateEffectKey(dwKey, nFlags, bDown);

	switch ( Action )
    {
		case EB_HELP:
		{
			::MessageBoxEx( NULL, "Help : F1 - Help \n\n"
				                  "Home - Reset To Defaults \n\n"
								  "(+) Increase value of Step\n\n"
								  "(-) Decrease value of Step\n\n"
				                  "(Alt Gr +) Increase Time of loop\n\n"
								  "(Alt Gr -) Decrease Time of loop\n\n"
								  "(PgUp) Increase Speed\n\n"
								  "(PgDown) Decrease Speed\n\n"
								  "Right Arrow - Move right the start point\n\n"
								  "Left Arrow - Move left the start point\n\n"
								  "W - Wireframe Toggle \n\n"
								  "Space\\Pause - Toggle Pause/Resume \n\n"
								  "Left Button & Mouse - Rotate Object\n\n"
								  "Shift Left Button & Mouse - Pan Camera \n\n"
								  "Ctrl Left Button & Mouse - Move Camera In & Out\n\n",
				   "Help", MB_ICONINFORMATION | MB_TASKMODAL, MAKELANGID( LANG_NEUTRAL, SUBLANG_DEFAULT ) );
		}
		break;

		case EB_WIREFRAME:
        {
            m_bWireframe = !m_bWireframe;
			m_dwEffectDirtyFlags |= EBEFFECT_DIRTY_PUBLICSTATE;

            
        }
		break;

		case EB_PAUSE:
		{
			m_bPause = !m_bPause;
			m_dwEffectDirtyFlags |= EBEFFECT_DIRTY_PUBLICSTATE;
		}
		break;


		case EB_RESET:
		{
			m_bAxeY = FALSE;
			m_dwEffectDirtyFlags |= EBEFFECT_DIRTY_PUBLICSTATE;
			m_iSpeed = 64;

			m_fStep = 10.0f;

			m_fLoop = 90.0f;

			m_fTick = 0.0f;

         	m_fOffset = 0.0f;

		}
		break;

		case EB_PAGEDOWN:
			
			m_iSpeed += 1;
			break;
		case EB_PAGEUP:
			if(m_iSpeed > 0)
			 m_iSpeed -= 1;
		    break;
		case EB_ADD:
				if(nFlags & (1<<13))
				{
					m_fLoop += 1.0f;
					
				}
				else
				{
					m_fStep += 1.0f;

					
				}
				break;

		case EB_SUBTRACT:
				if(nFlags & (1<<13))
				{
					if(m_fLoop > 0.0f)
					{
						m_fLoop -= 1.0f;
						m_fTick -= 1.0f;
						if(m_fTick < 0.0f)
							m_fTick = 0.0f;
					}
					else
						m_fLoop = 0.0f;
					
				}
				else
				{
					if(	m_fStep > 1.0f)
						m_fStep -= 1.0f;
					else
						m_fStep = 1.0f;
				}
				break;

	}

	switch(dwKey)
	{
	case VK_RIGHT:
			m_fOffset += (m_fRadius/20.0f);

	break;
	case VK_LEFT:
		
		m_fOffset -= (m_fRadius/20.0f);
		if(m_fOffset < 0.0f)
			m_fOffset = 0.0f;
			
	break;
	}

	SetVSConst();
	SetVertexShaderMatrices();

}